jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sap.ui.model.resource.ResourceModel");
jQuery.sap.require("sas.hc.m.MessageDialog");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.config.model.SheetConfig");
jQuery.sap.require("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.config.model.HierarchyMapPair");
jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.wkbench.model.SheetCategory");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.UserPrefs");

sap.ui.controller("sas.stress.config.view.CreateWorksheetTemplateEditor", {

    TIME_HIERARCHY_CODE: "TIME",
    HIDDEN: "hidden",
    NO_VERSION: "__NO__VERSION__",
    DEFAULT_HISTORY_PERIODS: 4,
    DEFAULT_FUTURE_PERIODS: 9,
    globalAxis: "",
    globalSelectedIndex: -1,
    globalSelectedNodePath: "",
    globalSelectedNode: "",
    globalSelectedMemberList: [],

    onInit: function (event) {
        this.refresh();

        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());
    },

    refresh: function () {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var controller = this;
        var view = this.getView();

        var json = { sheetCategories: [], hierarchyDetails: [], axisTable: [], templates: [], editedTemplate: {} };

        json.editedTemplate = view.getViewData().template;

        view.breadCrumb.removeAllItems();
        view.breadCrumb.addBreadcrumbItem("Top", rb.getText("sheetTemplates.header.title"));
        view.breadCrumb.addBreadcrumbItem("New", (json.editedTemplate) ? json.editedTemplate.name : rb.getText("sheetTemplate.breadCrumb.txt"));

        var selectedCategoryKey = view.getViewData().selectedCategoryKey;

        view.setBusyIndicatorDelay(0);
        view.setBusy(true);

        $.when(sas.stress.config.model.Hierarchy.getHierarchies(),
            sas.stress.wkbench.model.SheetCategory.getSheetCategories(true)).done(function (hierarchies,sheetCategories) {
            var rootNode = { children: [] };
            var tableX = { children: [] };
            var tableY = { children: [] };
            var tableF = { children: [] };
            var tableH = { children: [] };

            _.each(hierarchies, function (hierarchy) {
                var hierarchyCategoryRootNode = {
                    name: hierarchy.getCategoryName(),
                    categoryHeader: true,
                    children: []
                };

                // Search to see if an existing hierarchy Category already exists
                var node = _.find(rootNode.children, function (childNode) {
                    return childNode.name === hierarchy.getCategoryName();
                });
                if (node !== undefined) {
                    hierarchyCategoryRootNode = node;
                }

                var hierarchyRootNode = {
                    hierarchyKey: hierarchy.getHierarchyKey(),
                    categoryKey: hierarchy.getCategoryKey(),
                    code: hierarchy.getHierarchyCode(),
                    categoryName: hierarchy.getCategoryName(),
                    name: hierarchy.getName(),
                    version: hierarchy.getVersion(),
                    categoryHeader: false,
                    children: [],
                    startLevel: 1,
                    lineage: hierarchy.levelNames.join(" "),
                    endLevel: hierarchy.getLevelNames().length,
                    readOnly: false,
                    taxonomyPackageOwners: hierarchy.getTaxonomyPackageOwners(),
                    // HF2 changes---
                    subtotalCode:""
                    // HF2 changes---
                };
                if (hierarchyRootNode.endLevel === undefined || hierarchyRootNode.endLevel === 0) {
                    hierarchyRootNode.endLevel = 1;
                }

                if (hierarchyRootNode.taxonomyPackageOwners === undefined || hierarchyRootNode.taxonomyPackageOwners.length === 0) {
                    hierarchyCategoryRootNode.children.push(hierarchyRootNode);
                }

                if (node === undefined) {
                    rootNode.children.push(hierarchyCategoryRootNode);
                }
            });
            _.each(sheetCategories, function (cat) {
                var templates = cat.getSheetTemplates();
                json.sheetCategories.push({ id: cat.getId(), label: cat.getName() });
                _.each(templates, function (template) {
                    json.templates.push({ id: template.getId(), name: template.getName(), version: (template.getVersion() === controller.NO_VERSION) ? "" : template.getVersion() });
                });
            });

            // Sort the Sheet Category List
            controller._sortSheetCategories(json.sheetCategories);

            //Sort the children of the hierarchies first
            _.each(rootNode.children, function (node) {
                controller._sortHierarchies(node.children);
            });
            //Now sort the hierarchy itself
            controller._sortHierarchies(rootNode.children);

            if (json.editedTemplate !== undefined) {
                view.worksheetNameInput.setModel(new sas.risk.common.models.RiskJSONModel(json.editedTemplate));
                view.worksheetDescInput.setModel(new sas.risk.common.models.RiskJSONModel(json.editedTemplate));
                if (json.editedTemplate.version === controller.NO_VERSION) {
                    json.editedTemplate.version = "";
                }
                view.worksheetVersionInput.setModel(new sas.risk.common.models.RiskJSONModel(json.editedTemplate));
                view.worksheetCategoryList.setSelectedKey(json.editedTemplate.category.id);
                view.hiddenHierarchies.setSelected(json.editedTemplate.aggregateMissingValues);
                controller._hideHierarchySection(json.editedTemplate.aggregateMissingValues);
                selectedCategoryKey = json.editedTemplate.category.id;
                controller.historyPeriod = json.editedTemplate.numHistoryPeriods;
                controller.futurePeriod = json.editedTemplate.numFuturePeriods;
            }

            if (selectedCategoryKey !== undefined) {
                view.worksheetCategoryList.setSelectedKey(selectedCategoryKey);
            }

            if (json.editedTemplate !== undefined) {
                _.each(json.editedTemplate.axes, function (axis) {
                    var hierarchy = sas.stress.config.model.Hierarchy.getHierarchy(axis.hierarchyKey);
                    var axisItem = {
                        hierarchyKey: axis.hierarchyKey,
                        categoryKey: selectedCategoryKey,
                        axis: axis.axis,
                        code: axis.hierarchyCode,
                        order: axis.order,
                        startLevel: axis.startLevel,
                        endLevel: axis.endLevel,
                        nameVersion: ((hierarchy === undefined) ? rb.getText("sheetTemplate.accessIsDenied.txt") : hierarchy.getNameVersion()),
                        name: axis.hierarchyName,
                        members: axis.members,
                        sheetTemplateAxisKey: axis.sheetTemplateAxisKey,
                        sheetTemplateKey: axis.sheetTemplateKey,
                        // HF2 changes---
                        subtotalCode:axis.subtotalCode
                        // HF2 changes---
                    };

                    if (axis.hierarchyCode === controller.TIME_HIERARCHY_CODE) {
                        var historyTimePeriod = rb.getText("addTimeOptionsDialog.historyPeriod.control.label");
                        var futureTimePeriod = rb.getText("addTimeOptionsDialog.futurePeriod.control.label");

                        axisItem.name = axis.hierarchyName + " (" + historyTimePeriod + " " + controller.historyPeriod + " " + futureTimePeriod + " " + controller.futurePeriod + ")";
                    }

                    if (axis.members.length > 0) {
                      //  axisItem.name = "* " + axisItem.name;
                        axisItem.nameVersion = "* " + axisItem.nameVersion;
                    }

                    _.each(rootNode.children, function (node) {
                        _.each(node.children, function (hierarchy) {
                            if (hierarchy.hierarchyKey === axis.hierarchyKey && (hierarchy.startLevel !== axisItem.startLevel || hierarchy.endLevel !== axisItem.endLevel)
                                // HF2 changes---
                                || ("YES_SUBTOTALS"===axisItem.subtotalCode)&&("X"===axisItem.axis)) {
                                // HF2 changes---
                                if (
                                    //!axisItem.name.startsWith("*")  // startWith does not work in IE
                                    (axisItem.nameVersion.indexOf("*") === -1)
                                ) {
                                    //axisItem.name = "* " + axisItem.name;
                                    axisItem.nameVersion = "* " + axisItem.nameVersion;
                                }
                            }
                        });
                    });

                    if (axis.axis === "X") {
                        tableX.children.push(axisItem);
                    } else if (axis.axis === "Y") {
                        tableY.children.push(axisItem);
                    } else if (axis.axis === "Slicer") {
                        tableF.children.push(axisItem);
                    } else if (axis.axis === "Hidden") {
                        tableH.children.push(axisItem);
                    }

                });
            }

            var hierarchiesModel = new sas.risk.common.models.RiskJSONModel(rootNode);
            view.hierarchiesTreeTable.setModel(hierarchiesModel);
            view.hierarchiesTreeTable.bindRows("/children");

            if (json.editedTemplate !== undefined) {
                controller._sortAxes(tableX.children);
                controller._sortAxes(tableY.children);
                controller._sortAxes(tableF.children);
                controller._sortAxes(tableH.children);
            }

            var xAxisModel = new sas.risk.common.models.RiskJSONModel(tableX);
            view.xAxisTable.setModel(xAxisModel);
            view.xAxisTable.bindRows("/children");

            var yAxisModel = new sas.risk.common.models.RiskJSONModel(tableY);
            view.yAxisTable.setModel(yAxisModel);
            view.yAxisTable.bindRows("/children");

            var filterModel = new sas.risk.common.models.RiskJSONModel(tableF);
            view.filterTable.setModel(filterModel);
            view.filterTable.bindRows("/children");

            var hiddenModel = new sas.risk.common.models.RiskJSONModel(tableH);
            view.hiddenTable.setModel(hiddenModel);
            view.hiddenTable.bindRows("/children");



            if (json.editedTemplate === undefined) {
                // Set default generated name for a new worksheet
                json.name = controller._getNewSheetName(json.templates);
            }
            // Set focus on the name field
           // view.worksheetNameInput.focus();

            var model = new sas.risk.common.models.RiskJSONModel(json);
            view.setModel(model);

            if (json.editedTemplate === undefined) {
                view.hiddenHierarchies.setSelected(true);
                controller._hideHierarchySection(true);
            }

            view.setBusy(false);
        });
        this.getView().saveButton.setEnabled(false);
    },

    breadcrumbClickHandler: function (event) {
        if (sas.stress.wkbench.StressUtil.confirmToContinue(this.navChecker.getDirty())) {
            switch (event.getParameter("item").getKey()) {
                case "Top":
                    sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_WORKSHEET_TEMPLATES, {});
                    break;
            }
        }
    },
    _sortSheetCategories: function (sheetCategories) {
        sheetCategories.sort(function (a, b) {
            var aName = a.label.toLowerCase();
            var bName = b.label.toLowerCase();
            return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
        });
    },
    _sortHierarchies: function (hierarchies) {
        //Sort by Name ascending
        hierarchies.sort(function (a, b) {
            var aName = a.name.toLowerCase();
            var bName = b.name.toLowerCase();
            return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
        });
    },

    _sortAxes: function (axes) {
        //Sort by Order no ascending
        axes.sort(function (a, b) {
            var aOrderNo = a.order;
            var bOrderNo = b.order;
            return ((aOrderNo < bOrderNo) ? -1 : ((aOrderNo > bOrderNo) ? 1 : 0));
        });
    },

    hierarchyTreeTableExpandButtonPress: function (event) {
        var view = this.getView();
        for (var i = 0; i < view.hierarchiesTreeTable.getBinding("rows").getLength(); i++) {
            view.hierarchiesTreeTable.expand(i);
        }
    },

    hierarchyTreeTableCollapseButtonPress: function (event) {
        var view = this.getView();
        for (var i = view.hierarchiesTreeTable.getBinding("rows").getLength() - 1; i >= 0; i--) {
            view.hierarchiesTreeTable.collapse(i);
        }
    },

    refineTreeTableExpandButtonPress: function (event) {

        var sourceTable = sap.ui.getCore().byId("refineHierarchiesDialog.sourceHierarchiesTable");

        var selectedNodePath = sourceTable.getContextByIndex(0).getPath();
        var selectedNode = sourceTable.getModel().getProperty(selectedNodePath);

        for (var i = 0; i < sourceTable.getBinding("rows").getLength(); i++) {
            var nodePath = sourceTable.getContextByIndex(i).getPath();
            var node = sourceTable.getModel().getProperty(nodePath);
            if (node.hierarchyKey === selectedNode.hierarchyKey && node.children.length > 0) {
                sourceTable.expand(i);
            }
        }
    },

    refineTreeTableCollapseButtonPress: function (event) {
        var sourceTable = sap.ui.getCore().byId("refineHierarchiesDialog.sourceHierarchiesTable");
        if (sourceTable.getBinding("rows") !== undefined) {
            for (var i = sourceTable.getBinding("rows").getLength() - 1; i >= 0; i--) {
                sourceTable.collapse(i);
            }
        }
    },

    /* Adds a branch of nodes listed in levelNames to the rootNode. */
    _addToTreeNode: function (rootNode, levelName) {
        var currentNode = rootNode;
        //$.each(lineItem.levelNames, function(index, levelName){
        //check if currentNode.children contains node with levelName[i].
        var node = _.find(currentNode.children, function (childNode) {
            return childNode.name === levelName;
        });
        //If not, add node with levelName[i].
        if (node === undefined) {
            node = {
                name: levelName,
                hierarchyKey: currentNode.hierarchyKey,
                categoryKey: currentNode.categoryKey,
                code: currentNode.code,
                children: [],
                readOnly: true
            };
            //set liKey and contra on the leaf node.
            //if(index == lineItem.getLevelNames().length - 1){
            //        node.liKey = lineItem.getLiKey();
            //        node.contra = lineItem.getContra();
            //        node.aggregationType = lineItem.getAggregationType();
            //        node.format = lineItem.getFormat();
            //    }
            currentNode.children.push(node);
        }
        //set currentNode as the node with levelName[i] for levelName[i+1].
        currentNode = node;

        //});
    },
    /* Adds a branch of nodes listed in levelNames to the rootNode. */
    _addToHierarchyDetailsTable: function (rootNode, hNode, lineItem, members, globalList) {
        var curRootNode = rootNode;
        var currentNode = hNode;

        $.each(lineItem.levelNames, function (index, levelName) {
            //check if currentNode.children contains node with levelName[i].
            var node = _.find(currentNode.children, function (childNode) {
                return childNode.name === levelName;
            });
            //If not, add node with levelName[i].
            if (node === undefined) {
                node = {
                    name: levelName,
                    hierarchyKey: lineItem.getHierarchyKey(),
                    categoryKey: currentNode.categoryKey,
                    code: currentNode.code,
                    levelNum: ((lineItem.levelNum === undefined) ? -99 : lineItem.levelNum),
                    levelName: ((lineItem.levelNum === undefined) ? "" : curRootNode.levelNames[lineItem.levelNum - 1]),
                    children: [],
                    checked: false,
                    liKey: lineItem.getLiKey(),
                    enabled: true

                };
                //Check if the item is saved in the member list, if it is, set the checked value to true
                if (members !== undefined && members.length > 0) {
                    var foundMember = members.indexOf(node.liKey);
                    if (foundMember !== -1) {
                        node.checked = true;
                        //globalList.push(node.name);
                        globalList.push(node.liKey);
                    }
                }


                //set liKey and contra on the leaf node.
                //DD: commenting out below
                //if (index == lineItem.getLevelNames().length - 1) {
                //    node.liKey = lineItem.getLiKey();
                //    node.contra = lineItem.getContra();
                //    node.aggregationType = lineItem.getAggregationType();
                //    node.format = lineItem.getFormat();
                //}
                currentNode.children.push(node);
            }
            //set currentNode as the node with levelName[i] for levelName[i+1].
            currentNode = node;
        });
    },

    saveCreateWorksheetButtonPress: function (event) {
        var view = this.getView();
        var controller = this;

        var json = this._getCreateSheetJson();

        // Edit existing worksheet

        if (view.getModel().getData().editedTemplate !== undefined) {
            sas.stress.config.StressAPI.updateSheetTemplate(json).done(function () {
                var projects = sas.stress.wkbench.model.Project._projects;
                _.each(projects, function (project) {
                    var sheets = project.getSheets();
                    _.each(sheets, function (sheet) {
                        sheet.clearPreferredSheetFilters(); // there should not be any, however, remove if any
                    });
                });
                controller._setDirty(false);
                sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_WORKSHEET_TEMPLATES, {});
                controller.refresh();
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        } else {
            sas.stress.config.StressAPI.createSheetTemplate(json).done(function () {
                controller._setDirty(false);
                sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_WORKSHEET_TEMPLATES, {});
                //controller.refresh();
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },

    // clearContents: function () {
    //     var view = this.getView();
    //     var controller = this;
    //
    //     controller._setDirty(false);
    //     controller.refresh();
    // },

    _getCreateSheetJson: function () {
        var view = this.getView();
        var controller = this;

        var completeJson = { axes: [] };

        var XTable = sap.ui.getCore().byId(view.XAXIS + "_Table");
        var xAxisData = XTable.getModel().getData();

        var x = 1;

        _.each(xAxisData, function (value, key) {
            _.each(value, function (value, key) {
                var jsonItem = {
                    hierarchyKey: value.hierarchyKey,
                    code: value.code,
                    axis: "X",
                    order: x++,
                    startLevel: value.startLevel,
                    endLevel: value.endLevel,
                    members: value.members,
                    name: value.name,
                    // HF2 changes---
                    subtotalCode:value.subtotalCode
                    // HF2 changes---
                };
                completeJson.axes.push(jsonItem);
            });
        });

        var YTable = sap.ui.getCore().byId(view.YAXIS + "_Table");
        var yAxisData = YTable.getModel().getData();

        var y = 1;

        _.each(yAxisData, function (value, key) {
            _.each(value, function (value, key) {
                var jsonItem = {
                    hierarchyKey: value.hierarchyKey,
                    code: value.code,
                    axis: "Y",
                    order: y++,
                    startLevel: value.startLevel,
                    endLevel: value.endLevel,
                    members: value.members,
                    name: value.name,
                    // HF2 changes---
                    // for Y.. the subtotals are always turned ON.. the mid-tier will enforce this anyways
                    subtotalCode:"YES_SUBTOTALS"
                    // HF2 changes---
                };
                completeJson.axes.push(jsonItem);
            });
        });


        var FilterTable = sap.ui.getCore().byId(view.FILTER + "_Table");
        var filterData = FilterTable.getModel().getData();

        var f = 1;

        _.each(filterData, function (value, key) {
            _.each(value, function (value, key) {
                var jsonItem = {
                    hierarchyKey: value.hierarchyKey,
                    code: value.code,
                    axis: "Slicer",
                    order: f++,
                    startLevel: value.startLevel,
                    endLevel: value.endLevel,
                    members: value.members,
                    name: value.name
                };
                completeJson.axes.push(jsonItem);
            });
        });

        var HiddenTable = sap.ui.getCore().byId(view.HIDDENHIERARCHY + "_Table");
        var hiddenData = HiddenTable.getModel().getData();

        var h = 1;

        _.each(hiddenData, function (value, key) {
            _.each(value, function (value, key) {
                var jsonItem = {
                    hierarchyKey: value.hierarchyKey,
                    code: value.code,
                    axis: "Hidden",
                    order: h++,
                    startLevel: value.startLevel,
                    endLevel: value.endLevel,
                    members: value.members,
                    name: value.name
                };
                completeJson.axes.push(jsonItem);
            });
        });


        // Header information
        //var worksheetName = view.getModel().getData().name;
        //var worksheetDesc = view.getModel().getData().desc;
        //var worksheetCategoryList = sap.ui.getCore().byId("WorksheetCategoryList");

        var worksheetName = view.worksheetNameInput.getValue();
        var worksheetVersion = view.worksheetVersionInput.getValue();
        var worksheetDesc = view.worksheetDescInput.getValue();
        var worksheetCategoryKey = view.worksheetCategoryList.getSelectedKey();
        var aggregateMissingValues = view.hiddenHierarchies.getSelected();

        var numHistoryPeriods = controller.historyPeriod;
        var numFuturePeriods = controller.futurePeriod;

        completeJson.name = worksheetName;
        completeJson.version = worksheetVersion;
        completeJson.description = worksheetDesc;
        completeJson.categoryKey = worksheetCategoryKey;
        completeJson.numFuturePeriods = numFuturePeriods;
        completeJson.numHistoryPeriods = numHistoryPeriods;
        completeJson.aggregateMissingValues = aggregateMissingValues;

        if (view.getModel().getData().editedTemplate !== undefined) {
            completeJson.id = view.getModel().getData().editedTemplate.id;
        }

        return completeJson;
    },

    hierarchiesTreeTableRowSelectionChange: function (event) {
        var view = this.getView();
        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (selectedIndex === -1) {
            return;
        }

        var context = view.hierarchiesTreeTable.getContextByIndex(selectedIndex);
        var selectedHierarchy = context.getObject();
        if (selectedHierarchy.categoryHeader === false) {
            this._toggleAddMappingButton();
        } else {
            this._disableAllAddButton();
        }
    },

    _toggleRefineButton: function (axis) {
        var view = this.getView();
        var axisTable = sap.ui.getCore().byId(axis + "_Table");
        var selectedItem = axisTable.getSelectedIndex();

        //var length = view[axis + "Table"]._getRowCount();
        //var length = axisTable._getRowCount();
        var length = axisTable.getModel().getData().children.length;

        //if  ((selectedItem === -1) || (length < 1))
        if (length < 1)
        {
            view[axis + "RefineButton"].setEnabled(false);
        } else {
            view[axis + "RefineButton"].setEnabled(true);
        }
    },

    _toggleRemoveMappingButton: function (axis) {
        var view = this.getView();

        //var length = view[axis + "Table"]._getRowCount();
        var length = view[axis + "Table"].getModel().getData().children.length;

        if (length < 1) {
            view[axis + "RemoveButton"].setEnabled(false);
        } else {
            view[axis + "RemoveButton"].setEnabled(true);
        }
    },

    _toggleAddMappingButton: function () {
        var view = this.getView();
        var controller = this;

        var selectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (selectedIndex === -1) {
            return;
        }

        var context = view.hierarchiesTreeTable.getContextByIndex(selectedIndex);
        var selectedHierarchy = context.getObject();

        this._disableAllAddButton();

        if (!this._hierarchyExistsInAnyAxis(selectedHierarchy)) {
            this._enableAddButton(view.XAXIS);
        }
        if (!this._hierarchyExistsInAnyAxis(selectedHierarchy)) {
            this._enableAddButton(view.YAXIS);
        }
        if (!this._hierarchyExistsInAnyAxis(selectedHierarchy)) {
            this._enableAddButton(view.FILTER);
        }
        if (!this._hierarchyExistsInAnyAxis(selectedHierarchy) && (selectedHierarchy.code !== controller.TIME_HIERARCHY_CODE)) {
            this._enableAddButton(view.HIDDENHIERARCHY);
        }
    },

    _hierarchyExistsInAnyAxis: function (selectedHierarchy) {
        // Checks all axis to see if the selected Hierarchy exists
        // if it finds a hierarchy that matches, returns true
        var view = this.getView();
        var controller = this;
        var axisList = [view.XAXIS, view.YAXIS, view.FILTER, view.HIDDENHIERARCHY];
        var i = 0;

        var retCode = false;

        for (i = 0; i < 4; i++) {
            var axisTable = sap.ui.getCore().byId(axisList[i] + "_Table");
            var targetRootNode = axisTable.getModel().getData();
            if (targetRootNode.children === 0) {
                continue;
            }
            _.each(targetRootNode.children, function (targetHierarchy) {
                if (((selectedHierarchy.code === targetHierarchy.code) &&
                    (selectedHierarchy.code === controller.TIME_HIERARCHY_CODE)) ||
                    (selectedHierarchy.name.toLowerCase() === targetHierarchy.name.toLowerCase())) {
                    retCode = true;
                }
            });
        }
        return retCode;
    },

    // _enableAllAddButton: function () {
    //     var view = this.getView();
    //     var axisList = [view.XAXIS, view.YAXIS, view.FILTER];
    //     var i = 0;
    //
    //     for (i = 0; i < 3; i++) {
    //         view[axisList[i] + "AddButton"].setEnabled(true);
    //     }
    // },
    _disableAllAddButton: function () {
        var view = this.getView();
        var axisList = [view.XAXIS, view.YAXIS, view.FILTER, view.HIDDENHIERARCHY];
        var i = 0;

        for (i = 0; i < 4; i++) {
            view[axisList[i] + "AddButton"].setEnabled(false);
        }
    },


    // _hierarchyExistsInAxis: function (axis, selectedHierarchy) {
    //     var axisTable = sap.ui.getCore().byId(axis + "_Table");
    //     var targetRootNode = axisTable.getModel().getData();
    //
    //     if (targetRootNode.children === 0) {
    //         return false;
    //     }
    //
    //     var retCode = false;
    //
    //     _.each(targetRootNode.children, function (targetHierarchy) {
    //         if (selectedHierarchy.hierarchyKey === targetHierarchy.hierarchyKey &&
    //             selectedHierarchy.categoryKey === targetHierarchy.categoryKey &&
    //             selectedHierarchy.name === targetHierarchy.name
    //         ) {
    //             retCode = true;
    //         }
    //     });
    //     return retCode;
    // },

    _enableAddButton: function (axis) {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        view[axis + "AddButton"].setEnabled(capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE));
    },

    _enableRemoveButton: function (axis) {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        view[axis + "RemoveButton"].setEnabled(false);

        var selectedIndex = view[axis + "Table"].getSelectedIndex();
        if (selectedIndex !== -1) {
            view[axis + "RemoveButton"].setEnabled(capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE));
        }
    },

    // _disableRemoveButton: function (axis) {
    //     var view = this.getView();
    //     var axisTable = sap.ui.getCore().byId(axis + "_Table");
    //
    //     var selectedIndex = view[axis + "Table"].getSelectedIndex();
    //     if (selectedIndex !== -1) {
    //         view[axis + "RemoveButton"].setEnabled(false);
    //     }
    // },

    _toggleUpButton: function (axis) {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        view[axis + "UpButton"].setEnabled(false);

        var selectedIndex = view[axis + "Table"].getSelectedIndex();
        // Only enable it if the first item is not selected
        if (selectedIndex > 0) {
            view[axis + "UpButton"].setEnabled(capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE));
        }

    },
    _toggleDownButton: function (axis) {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        view[axis + "DownButton"].setEnabled(false);

        var selectedIndex = view[axis + "Table"].getSelectedIndex();
        //var length = view[axis + "Table"]._getRowCount();
        var length = view[axis + "Table"].getModel().getData().children.length;

        if (selectedIndex !== -1 && selectedIndex !== (length - 1)) {
            view[axis + "DownButton"].setEnabled(capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE));
        }
    },

    // _disableAddButton: function (axis) {
    //     var view = this.getView();
    //     view[axis + "AddButton"].setEnabled(false);
    // },

    addMappingButtonPress: function (event) {
        // var view = this.getView();
        var controller = this;

        //Get the source, to determine if its xaxis, yaxis or filter
        var axis = this._extractAxisFromSource(event);

        controller._addModelMapping(axis);
        controller._setDirty(true);
    },

    _extractAxisFromSource: function (event) {
        // Assuming eventId is of the form xAxis_addButton, function returns the string xAxis
        return event.getSource().getId().split("_")[0];
    },

    _addModelMapping: function (axis) {
        var view = this.getView();
        var controller = this;

        //Hierarchy table selected

        //var selectedIndex = view[axis + "Table"].getSelectedIndex();

        var hierarchiesTreeTableSelectedIndex = view.hierarchiesTreeTable.getSelectedIndex();
        if (hierarchiesTreeTableSelectedIndex === -1) {
            return; //return if no source row selected
        }

        var axisTable = sap.ui.getCore().byId(axis + "_Table");

        //Get the list of selected items in the hierarchy table.

        var context = view.hierarchiesTreeTable.getContextByIndex(hierarchiesTreeTableSelectedIndex);
        var selectedHierarchy = context.getObject();

        // Special consideration for Time Category
        // Popup a dialog to populate Time dimension details
        // Time is hardcoded to Category Code TIME
        //if (selectedHierarchy.categoryName == "Time") {
        if (selectedHierarchy.code === this.TIME_HIERARCHY_CODE) {
            var existingDialog = sap.ui.getCore().byId("sas.stress.config.view.AddTimeOptionsDialog");
            if (existingDialog) {
                existingDialog.destroy();
            }
            var dialog = sap.ui.jsfragment("sas.stress.config.view.AddTimeOptionsDialog", this);

            var timeOptionsJSON = {};
            if (view.getViewData().template === undefined) {
                timeOptionsJSON = {
                    nameVersion: sas.stress.wkbench.StressUtil.getNameVersion(selectedHierarchy.name,selectedHierarchy.version),
                    name: selectedHierarchy.name,
                    numHistoryPeriods: this.DEFAULT_HISTORY_PERIODS,
                    numFuturePeriods: this.DEFAULT_FUTURE_PERIODS,
                    hierarchyKey: selectedHierarchy.hierarchyKey,
                    categoryKey: selectedHierarchy.categoryKey,
                    code: selectedHierarchy.code,
                    categoryName: selectedHierarchy.categoryName,
                    startLevel: selectedHierarchy.startLevel,
                    endLevel: selectedHierarchy.endLevel,
                    axisTable: axisTable,
                    axis: axis,
                    members: []
                };
            } else {
                timeOptionsJSON = {
                    nameVersion: sas.stress.wkbench.StressUtil.getNameVersion(selectedHierarchy.name,selectedHierarchy.version),
                    name: selectedHierarchy.name,
                    numHistoryPeriods: view.getViewData().template.numHistoryPeriods,
                    numFuturePeriods: view.getViewData().template.numFuturePeriods,
                    hierarchyKey: selectedHierarchy.hierarchyKey,
                    categoryKey: selectedHierarchy.categoryKey,
                    code: selectedHierarchy.code,
                    categoryName: selectedHierarchy.categoryName,
                    startLevel: selectedHierarchy.startLevel,
                    endLevel: selectedHierarchy.endLevel,
                    axisTable: axisTable,
                    axis: axis,
                    members: []
                };
            }

            dialog.setModel(new sas.risk.common.models.RiskJSONModel(timeOptionsJSON));
            dialog.open();
        } else {
            var targetNode = {
                hierarchyKey: selectedHierarchy.hierarchyKey,
                categoryKey: selectedHierarchy.categoryKey,
                code: selectedHierarchy.code,
                categoryName: selectedHierarchy.categoryName,
                nameVersion: sas.stress.wkbench.StressUtil.getNameVersion(selectedHierarchy.name,selectedHierarchy.version),
                name: selectedHierarchy.name,
                children: [],
                startLevel: selectedHierarchy.startLevel,
                endLevel: selectedHierarchy.endLevel,
                members: [],
                readOnly: false
            };

            var targetRootNode = axisTable.getModel().getData();
            targetRootNode.children.push(targetNode);


            //Refresh models to update view.
            view.hierarchiesTreeTable.getModel().refresh();
            axisTable.getModel().refresh();

            //var length = view[axis + "Table"]._getRowCount();
            //length = axisTable._getRowCount();
            length = axisTable.getModel().getData().children.length;

            if (length > 0) {
                axisTable.setSelectedIndex(length - 1);
            }
            controller._disableAllAddButton();
            controller._toggleRemoveMappingButton(axis);
            controller._toggleRefineButton(axis);
            controller._setDirty(true);
        }
    },

    addTimeOptionsDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var controller = this;


        controller.historyPeriod = sap.ui.getCore().byId("addTimeOptionsDialog.historyPeriodInput").getValue();
        controller.futurePeriod = sap.ui.getCore().byId("addTimeOptionsDialog.futurePeriodInput").getValue();

        var dialog = sap.ui.getCore().byId("sas.stress.config.view.AddTimeOptionsDialog");
        var dialogData = dialog.getModel().getData();
        var axisTable = dialogData.axisTable;
        var axis = dialogData.axis;

        var historyTimePeriod = rb.getText("addTimeOptionsDialog.historyPeriod.control.label");
        var futureTimePeriod = rb.getText("addTimeOptionsDialog.futurePeriod.control.label");
        var displayName = dialogData.nameVersion + " (" + historyTimePeriod + " " + controller.historyPeriod + " " + futureTimePeriod + " " + controller.futurePeriod + ")";

        var targetNode = {
            hierarchyKey: dialogData.hierarchyKey,
            categoryKey: dialogData.categoryKey,
            code: dialogData.code,
            categoryName: dialogData.categoryName,
            nameVersion: displayName,
            name: dialogData.name,
            children: [],
            startLevel: dialogData.startLevel,
            endLevel: dialogData.endLevel,
            readOnly: false
        };

        var targetRootNode = axisTable.getModel().getData();
        targetRootNode.children.push(targetNode);
        //Refresh models to update view.
        view.hierarchiesTreeTable.getModel().refresh();
        axisTable.getModel().refresh();
        dialog.close();
        //var length = axisTable._getRowCount();
        var length = axisTable.getModel().getData().children.length;

        if (length > 0) {
            axisTable.setSelectedIndex(length - 1);
        }
        controller._toggleRefineButton(axis);
        controller._toggleRemoveMappingButton(axis);
        controller._disableAllAddButton();
        controller._setDirty(true);
    },

    addHistoryPeriodInputLiveChange: function (event) {
        //var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var isValid = true;
        var nameInput = event.getSource();
        var newName = nameInput.getValue();
        ////Check if name is empty.
        if (newName === null || newName.trim() === "") {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        }
        else if (!newName.match(/^\d+$/)) {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.numericField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        }
        else if (newName === "0" && (sap.ui.getCore().byId("addTimeOptionsDialog.futurePeriodInput").getValue() === "0")) {
            isValid = false;
            nameInput.setValueStateText(rb.getText("addTimeOptionsDialog.errorMsg.control.label"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            sap.ui.getCore().byId("addTimeOptionsDialog.futurePeriodInput").setValueState(sap.ui.core.ValueState.Error);
        } else {
            nameInput.setValueStateText("");
            nameInput.setValueState(sap.ui.core.ValueState.None);
            if ((sap.ui.getCore().byId("addTimeOptionsDialog.futurePeriodInput").getValue() === "0")) {
                sap.ui.getCore().byId("addTimeOptionsDialog.futurePeriodInput").setValueState(sap.ui.core.ValueState.None);
            }
        }

        if ((sap.ui.getCore().byId("addTimeOptionsDialog.futurePeriodInput").getValueState() !== sap.ui.core.ValueState.None) ||
            (nameInput.getValueState() !== sap.ui.core.ValueState.None)
        ) {
            isValid = false;
        }
        sap.ui.getCore().byId("addTimeOptionsDialog.saveTimePeriodButton").setEnabled(isValid);
    },

    addFuturePeriodInputLiveChange: function (event) {
        //var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var isValid = true;
        var nameInput = event.getSource();
        var newName = nameInput.getValue();
        ////Check if name is empty.
        if (newName === null || newName.trim() === "") {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        }
        else if (!newName.match(/^\d+$/)) {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.numericField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else if (newName === "0" && (sap.ui.getCore().byId("addTimeOptionsDialog.historyPeriodInput").getValue() === "0")) {
            isValid = false;
            nameInput.setValueStateText(rb.getText("addTimeOptionsDialog.errorMsg.control.label"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            sap.ui.getCore().byId("addTimeOptionsDialog.historyPeriodInput").setValueState(sap.ui.core.ValueState.Error);
        } else {
            nameInput.setValueStateText("");
            nameInput.setValueState(sap.ui.core.ValueState.None);
            if ((sap.ui.getCore().byId("addTimeOptionsDialog.historyPeriodInput").getValue() === "0")) {
                sap.ui.getCore().byId("addTimeOptionsDialog.historyPeriodInput").setValueState(sap.ui.core.ValueState.None);
            }
        }

        if ((nameInput.getValueState() !== sap.ui.core.ValueState.None) ||
            (sap.ui.getCore().byId("addTimeOptionsDialog.historyPeriodInput").getValueState() !== sap.ui.core.ValueState.None)
        ) {
            isValid = false;
        }

        sap.ui.getCore().byId("addTimeOptionsDialog.saveTimePeriodButton").setEnabled(isValid);
    },

    refineHistoryPeriodInputLiveChange: function (event) {
        //var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var isValid = true;
        var nameInput = event.getSource();
        var newName = nameInput.getValue();
        ////Check if name is empty.
        if (newName === null || newName.trim() === "") {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else if (!newName.match(/^\d+$/)) {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.numericField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else if (newName === "0" && (sap.ui.getCore().byId("refineDialog.futurePeriodInput").getValue() === "0")) {
            isValid = false;
            nameInput.setValueStateText(rb.getText("addTimeOptionsDialog.errorMsg.control.label"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            sap.ui.getCore().byId("refineDialog.futurePeriodInput").setValueState(sap.ui.core.ValueState.Error);

        } else {
            nameInput.setValueStateText("");
            nameInput.setValueState(sap.ui.core.ValueState.None);
            if ((sap.ui.getCore().byId("refineDialog.futurePeriodInput").getValue() === "0")) {
                sap.ui.getCore().byId("refineDialog.futurePeriodInput").setValueState(sap.ui.core.ValueState.None);
            }
        }
         //Toggle Save button based on the result of validation.
        if ((sap.ui.getCore().byId("refineDialog.futurePeriodInput").getValueState() !== sap.ui.core.ValueState.None) ||
            (nameInput.getValueState() !== sap.ui.core.ValueState.None)
           ) {
               isValid = false;
        }

        sap.ui.getCore().byId("refineHierarchiesDialog.saveButton").setEnabled(isValid);
        controller._setDirty(true);
    },

    refineFuturePeriodInputLiveChange: function (event) {
       //var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var isValid = true;
        var nameInput = event.getSource();
        var newName = nameInput.getValue();
        ////Check if name is empty.
        if (newName === null || newName.trim() === "") {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else if (!newName.match(/^\d+$/)) {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.numericField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else if (newName === "0" && (sap.ui.getCore().byId("refineDialog.historyPeriodInput").getValue() === "0")) {
            isValid = false;
            nameInput.setValueStateText(rb.getText("addTimeOptionsDialog.errorMsg.control.label"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            sap.ui.getCore().byId("refineDialog.historyPeriodInput").setValueState(sap.ui.core.ValueState.Error);

        } else {
            nameInput.setValueStateText("");
            nameInput.setValueState(sap.ui.core.ValueState.None);
            if ((sap.ui.getCore().byId("refineDialog.historyPeriodInput").getValue() === "0")) {
                sap.ui.getCore().byId("refineDialog.historyPeriodInput").setValueState(sap.ui.core.ValueState.None);
            }
        }
        //Toggle Save button based on the result of validation.
        if ((sap.ui.getCore().byId("refineDialog.historyPeriodInput").getValueState() !== sap.ui.core.ValueState.None) ||
            (nameInput.getValueState() !== sap.ui.core.ValueState.None)
           ) {
               isValid = false;
        }

        sap.ui.getCore().byId("refineHierarchiesDialog.saveButton").setEnabled(isValid);
        controller._setDirty(true);

    },
    removeMappingButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var axis = this._extractAxisFromSource(event);
        var axisTable = sap.ui.getCore().byId(axis + "_Table");

        var targetRootNode = axisTable.getModel().getData();


        //Get the list of selected nodes. We get this list first because getContextByIndex will return a different row once we remove a node.
        //var selectedNodes = [];
        var selectedIndex = view[axis + "Table"].getSelectedIndex();
        //var selectedNodePath = view[axis + "Table"].getContextByIndex(selectedIndex).getPath();
        //var selectedNode = view[axis + "Table"].getProperty(selectedNodePath);

        targetRootNode.children.splice(selectedIndex, 1);
        axisTable.getModel().refresh();
        //this._disableRemoveButton(axis);
        controller._toggleRefineButton(axis);
        controller._setDirty(true);

        //var length = view[axis + "Table"]._getRowCount();
       // var length = axisTable._getRowCount();
        var length = axisTable.getModel().getData().children.length;

        if (selectedIndex !== 0 && selectedIndex !== (length - 1)) {
            axisTable.setSelectedIndex(selectedIndex - 1);
        }
        controller._toggleRemoveMappingButton(axis);

    },

    onAxisTableRowSelectionChanged: function (event) {
        //Enable the refine and remove mapping button
        var axis = this._extractAxisFromSource(event);
        this._toggleRefineButton(axis);
        this._enableRemoveButton(axis);
        this._toggleUpButton(axis);
        this._toggleDownButton(axis);
    },

    onAxisMoveUpPressed: function (event) {
        var controller = this;

        controller._setDirty(true);

        var axis = this._extractAxisFromSource(event);
        var axisTable = sap.ui.getCore().byId(axis + "_Table");

        var selectedIndex = axisTable.getSelectedIndex();

        var oModel = axisTable.getModel();
        var oData = oModel.getData();
        var temp = oData.children[selectedIndex - 1];
        oData.children[selectedIndex - 1] = oData.children[selectedIndex];
        oData.children[selectedIndex] = temp;
        oModel.setData(oData);

        //No need to check for outOfBound condition as the up/down button gets disabled at the boundaries and check is made there
        axisTable.setSelectedIndex(selectedIndex - 1);

    },

    onAxisMoveDownPressed: function (event) {
        var controller = this;

        controller._setDirty(true);

        var axis = this._extractAxisFromSource(event);
        var axisTable = sap.ui.getCore().byId(axis + "_Table");
        var selectedIndex = axisTable.getSelectedIndex();

        var oModel = axisTable.getModel();
        var oData = oModel.getData();
        var temp = oData.children[selectedIndex + 1];
        oData.children[selectedIndex + 1] = oData.children[selectedIndex];
        oData.children[selectedIndex] = temp;
        oModel.setData(oData);

        axisTable.setSelectedIndex(selectedIndex + 1);

    },

    refineAxisButtonPress: function (event) {
        var view = this.getView();
        var controller = this;

        controller.globalAxis = this._extractAxisFromSource(event);

        var existingDialog = sap.ui.getCore().byId("sas.stress.config.view.RefineHierarchiesDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        // Get the selected node
        controller.globalSelectedIndex = view[controller.globalAxis + "Table"].getSelectedIndices();
        controller.globalSelectedNodePath = view[controller.globalAxis + "Table"].getContextByIndex(controller.globalSelectedIndex).getPath();
        controller.globalSelectedNode = view[controller.globalAxis + "Table"].getModel().getProperty(controller.globalSelectedNodePath);

        //Reset the list of selected members
        controller.globalSelectedMemberList = [];

        sas.stress.config.model.Hierarchy.getSpecificHierarchyDetails(controller.globalSelectedNode.hierarchyKey).done(function (hierarchy) {
            var dialog = sap.ui.jsfragment("sas.stress.config.view.RefineHierarchiesDialog", controller);

            var rootNode = {
                name: "root",
                checked: false,
                enabled: true,
                children: [],
                levelNames: [],
                totalLevelNames: 1,
                hierarchyKey: hierarchy.hierarchyKey,
                numHistoryPeriods: controller.historyPeriod,
                numFuturePeriods: controller.futurePeriod,
                // HF2 changes---
                subtotalCode:controller.globalSelectedNode.subtotalCode,
                axis:controller.globalAxis
                // HF2 changes---
            };

            var hierarchyNode = {
                //S1459288 Need name so that the root hierarchy name is displayed
                name: ((hierarchy === undefined) ? rb.getText("sheetTemplate.accessIsDenied.txt") : hierarchy.getNameVersion()),
                nameVersion: ((hierarchy === undefined) ? rb.getText("sheetTemplate.accessIsDenied.txt") : hierarchy.getNameVersion()),
                levelNum: ((hierarchy.levelNum === undefined) ? -99 : hierarchy.levelNum),
                levelName: "",
                children: [],
                hierarchyKey: hierarchy.hierarchyKey,
                checked: false,
                enabled: true
            };

            // Create a list of levelNames
            _.each(hierarchy.getLevelNames(), function (levelName) {
                rootNode.levelNames.push(levelName);
            });

            rootNode.totalLevelNames = rootNode.levelNames.length - 1;

            //Set the level name from the level number // SOPHIE TODO: what's with the magic number?
            hierarchyNode.levelName = (hierarchyNode.levelNum === -99 ? "" : rootNode.levelNames[hierarchyNode.levelNum]);

            rootNode.children.push(hierarchyNode);

            _.each(hierarchy.getLineItems(), function (lineItem) {
                controller._addToHierarchyDetailsTable(rootNode, hierarchyNode, lineItem, controller.globalSelectedNode.members, controller.globalSelectedMemberList);
            });

            if (hierarchy.getLineItems().length === 0) {
                // Clear the data structure so that the NoDataText is displayed
                rootNode.children = [];
            }

            // Update the treeTable with ranges set from Horizontal Slider
            controller.updateTreeTable(rootNode, controller.globalSelectedNode.startLevel, controller.globalSelectedNode.endLevel);

            if (rootNode.levelNames.length === 0) {
                sap.ui.getCore().byId("refineHierarchiesDialog.horizontalSlider").setVisible(false);
            }
            //sap.ui.getCore().byId("refineHierarchiesDialog.horizontalSlider").invalidate();

            var hierarchiesModel = new sas.risk.common.models.RiskJSONModel(rootNode);

            dialog.setModel(hierarchiesModel);
            setTimeout(function () {
                sap.ui.getCore().byId("refineHierarchiesDialog.horizontalSlider").invalidate();
            }, 0);
            dialog.open();
        }).fail(function(jqxhr){
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    updateTreeTable: function (treeData, min, max) {
        var controller = this;
        if (treeData.levelNum !== 'undefined' &&
            treeData.levelNum !== '-99' && // SOPHIE TODO: magic number?
            ((treeData.levelNum < min) || (treeData.levelNum > max))) {
            treeData.enabled = false;
        }

        if (treeData.levelNum !== 'undefined' &&
            treeData.levelNum !== '-99' &&
            ((treeData.levelNum >= min) && (treeData.levelNum <= max))) {
            treeData.enabled = true;
        }
        if (treeData.children.length > 0) {
            _.each(treeData.children, function (data) {
                controller.updateTreeTable(data, min, max);
            });
        }
    },
    //
    // refineHierarchiesDialogSearch: function (event) {
    //     var view = this.getView();
    //     var controller = this;
    // },

    checkAndEnableRefineSaveButton: function () {
        var controller = this;

        var timePeriodsValid = true;

        //if its a Time hierarchy, verify the history and future inputs are not in invalid state, then enable save button
        if (controller.globalSelectedNode.code === controller.TIME_HIERARCHY_CODE) {
            if ((sap.ui.getCore().byId("refineDialog.historyPeriodInput").getValueState() !== sap.ui.core.ValueState.None) ||
                (sap.ui.getCore().byId("refineDialog.futurePeriodInput").getValueState() !== sap.ui.core.ValueState.None)) {
                timePeriodsValid = false;
            }
        }
        sap.ui.getCore().byId("refineHierarchiesDialog.saveButton").setEnabled(timePeriodsValid);
        controller._setDirty(true);

        // Update the tree table in the Refine window
    },

    // refineHierarchiesTreeTableRowSelectionChange: function (event) {
    //     var view = this.getView();
    //     var controller = this;
    //
    //     var saveButton = sap.ui.getCore().byId("refineHierarchiesDialog.saveButton");
    //
    //     //this._toggleAddMappingButton();
    // },
    //
    // refineHierarchiesSliderChange: function (event) {
    //     //this._toggleAddMappingButton();
    // },

    rebuildTimeHierarchyName: function (hierarchyItem, historyPeriod, futurePeriod) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        var hIndex = hierarchyItem.name.indexOf("(");
        var hName = hierarchyItem.name.substr(0, hIndex);

        var historyTimePeriod = rb.getText("addTimeOptionsDialog.historyPeriod.control.label");
        var futureTimePeriod = rb.getText("addTimeOptionsDialog.futurePeriod.control.label");
        hierarchyItem.name = hName + "(" + historyTimePeriod + " " + historyPeriod + " " + futureTimePeriod + " " + futurePeriod + ")"; // SOPHIE TODO: externalize format???
    },

    refineHierarchiesDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var dialog = sap.ui.getCore().byId("sas.stress.config.view.RefineHierarchiesDialog");
        dialog.setBusy(true);

        var horizontalSlider = sap.ui.getCore().byId("refineHierarchiesDialog.horizontalSlider");

        var hierarchyItem = view[controller.globalAxis + "Table"].getModel().getProperty(controller.globalSelectedNodePath);

        if (hierarchyItem.nameVersion.charAt(0) !== "*") {
            //hierarchyItem.name = "* " + hierarchyItem.name; // SOPHIE TODO: externalize format?
            hierarchyItem.nameVersion = "* " + hierarchyItem.nameVersion;
        }

        // If time hierarchy, get history and future periods and update the name
        if (controller.globalSelectedNode.code === controller.TIME_HIERARCHY_CODE) {
            controller.historyPeriod = sap.ui.getCore().byId("refineDialog.historyPeriodInput").getValue();
            controller.futurePeriod = sap.ui.getCore().byId("refineDialog.futurePeriodInput").getValue();
            this.rebuildTimeHierarchyName(hierarchyItem, controller.historyPeriod, controller.futurePeriod);
        }

        hierarchyItem.members = controller.globalSelectedMemberList;
        hierarchyItem.startLevel = horizontalSlider.getValues()[0] + 1;
        hierarchyItem.endLevel = horizontalSlider.getValues()[1] + 1;


        if ((controller.globalSelectedMemberList.length === 0 || hierarchyItem.startLevel !== 1)
            && hierarchyItem.nameVersion.charAt(0) == "*") {
            hierarchyItem.nameVersion =  hierarchyItem.nameVersion.substring(2);
        }
        //DD
        if (controller.globalSelectedMemberList.length === 0 && hierarchyItem.nameVersion.charAt(0) == "*") {
            hierarchyItem.nameVersion =  hierarchyItem.nameVersion.substring(2);
        }
        //Refresh the axis list
        view[controller.globalAxis + "Table"].getModel().refresh();

        dialog.setBusy(false);
        dialog.close();
        //
        //
        //var sourceHierarchiesTable = sap.ui.getCore().byId("refineHierarchiesDialog.sourceHierarchiesTable");
        ////var hierarchyMapPairs = [];
        //_.each(sourceHierarchiesTable.getSelectedIndices(), function (selectedIndex) {
        //    var context = sourceHierarchiesTable.getContextByIndex(selectedIndex);
        //    var obj = context.getObject();
        //    hierarchyMapPairs.push(new sas.stress.config.model.HierarchyMapPair(-1, parseInt(obj.hierarchyKey),
        //        parseInt(controller.targetHierarchyKey), ""));
        //});
        //if (hierarchyMapPairs.length > 0) {
        //    sas.stress.config.model.HierarchyMapPair.addHierarchyMapPairs(hierarchyMapPairs).done(function () {
        //            dialog.setBusy(false);
        //            dialog.close();
        //            view.sourceTreeTable.clearSelection();
        //            //Reload data in the view
        //            controller._loadData();
        //        })
        //        .fail(function (jqxhr) {
        //            dialog.setBusy(false);
        //            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        //        });
        //} else {
        //    dialog.setBusy(false);
        //    dialog.close();
        //}
    },

    _status: function (code, msg) {
        return { code: code, msg: msg };
    },

    onExit: function () {
        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_WORKSHEET_TEMPLATES, {});
        //S1312016: Need to make sure the router is back working after save or navigate to another page under configure without hash change
        var router = sap.ui.core.routing.Router.getRouter("router");
        if (!router._bIsInitialized) {
            router.initialize();
        }
        this.navChecker.stop();
    },

    _setDirty: function (isDirty) {
        this._checkSaveButton();
        // this._toggleRemoveHierarchyButton(); //Disable Remove button if dirty
        // Let Appswitcher know
        this.navChecker.setDirty(isDirty);
    },

    _checkSaveButton: function () {
        var controller = this;
        if (controller._xAndYaxisPopulated() && controller._worksheetNamePopulated() && controller._worksheetCategoryExists()) {
            if ((sap.ui.getCore().byId("createWorksheetNameInput").getValueState() === sap.ui.core.ValueState.None) &&
                (sap.ui.getCore().byId("createWorksheetVersionInput").getValueState() === sap.ui.core.ValueState.None)) {
                this.getView().saveButton.setEnabled(true); //Enable Save button
            } else {
                this.getView().saveButton.setEnabled(false);
            }
        } else {
            this.getView().saveButton.setEnabled(false);
        }
    },

    _xAndYaxisPopulated: function () {
        var view = this.getView();
        var axisList = [view.XAXIS, view.YAXIS];
        var axisChildren = [0, 0];
        var retCode = false;

        for (var i = 0; i < 2; i++) {
            var axisTable = sap.ui.getCore().byId(axisList[i] + "_Table");
            var targetRootNode = axisTable.getModel().getData();
            if (targetRootNode.children && targetRootNode.children.length !== 0) {
                axisChildren[i] = 1;
            }
        }
        // X and Y axis have to be populated
        if (axisChildren[0] && axisChildren[1]) {
            return true;
        }
        return retCode;
    },

    _worksheetCategoryExists: function () {
        var selectedItem = sap.ui.getCore().byId("createWorksheetCategoryList").getSelectedItem();
        //Check if name is empty.
        return (selectedItem !== null);
    },

    _worksheetNamePopulated: function () {
        var newName = sap.ui.getCore().byId("createWorksheetNameInput").getValue();
        //Check if name is empty.
        return !(newName === null || newName.trim() === "");
    },

    worksheetCategoryChange: function (event) {
        this._setDirty(true);
    },
    _getNewSheetName: function (templates) {
        var view = this.getView();
        var rb=sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        for (var i = 1; ; i++) {
            var tempSheetName = rb.getText("sheetTemplate.genName.fmt", [i]);
            //check if a worksheet with this name exists
            if (!_.find(templates, function (template) {
                    return  template.name.trim().toLocaleLowerCase() === tempSheetName.trim().toLocaleLowerCase();
                })) {
                return tempSheetName;
            }
        }
    },

    createWorksheetNameVersionLiveChange: function (event) {
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var isValid = true;

        var nameInput = sap.ui.getCore().byId("createWorksheetNameInput");
        var newName = nameInput.getValue().trim();
        var versionInput = sap.ui.getCore().byId("createWorksheetVersionInput");
        var newVersion = versionInput.getValue().trim();

        ////Check if name is empty.
        if (newName === null || newName.trim() === "") {
            //isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            controller._setDirty(isValid);
        } else {
            //Check if a worksheet with this name already exists
            var templateName = _.find(view.getModel().getData().templates, function (template) {
                var version = (template.version === controller.NO_VERSION) ? "" : template.version;
                var found = (template.name.trim().toLowerCase() === newName.trim().toLowerCase())
                    && (version.trim().toLowerCase() === newVersion.trim().toLowerCase());
                var editedTemplate = view.getModel().getData().editedTemplate;
                if (editedTemplate !== undefined && editedTemplate.id === template.id) {
                    found = false; // current edited template
                }
                return found;
            });
            if (templateName) {
                //isValid = false;
                nameInput.setValueStateText(rb.getText("error.nameAndVersionInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
                versionInput.setValueStateText(rb.getText("error.nameAndVersionInUse.txt"));
                versionInput.setValueState(sap.ui.core.ValueState.Error);
                controller._setDirty(isValid);
            } else {
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
                versionInput.setValueStateText("");
                versionInput.setValueState(sap.ui.core.ValueState.None);
                controller._setDirty(isValid);
            }
        }
    },

    toggleHiddenHierarchies: function (event) {
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        var toggle = event.getParameter("selected");

        //var length = view.hiddenTable._getRowCount();
        var length =view.hiddenTable.getModel().getData().children.length;

        if (length > 0 && !toggle) {
            sas.hc.m.MessageDialog.warning({
                text: rb.getText("sheetTemplate.axis.hidden.delete.txt"),
                title: rb.getText("sheetTemplate.axis.hidden.delete.window.title"),
                buttonActions: [sas.hc.m.MessageDialogAction.Yes, sas.hc.m.MessageDialogAction.No],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getProperty("messageDialogAction");
                    if (action === sas.hc.m.MessageDialogAction.Yes) {
                        //Loop through all the hierarches in the list and remove them
                        var targetRootNode = view.hiddenTable.getModel().getData();
                        while (targetRootNode.children.length > 0) {
                            targetRootNode.children.pop();
                        }
                        view.hiddenTable.getModel().refresh();
                        view.hiddenTable.setSelectedIndex(-1);
                        controller._toggleRefineButton("hidden");
                        controller._toggleUpButton("hidden");
                        controller._toggleDownButton("hidden");
                        //Toggle the Hidden Hierarchy Section
                        controller._hiddenHierarchySection(toggle);
                    }
                    if (action === sas.hc.m.MessageDialogAction.No) {
                        var checkBox = sap.ui.getCore().byId("createWorksheetHiddenHierarchiesInput");
                        checkBox.setSelected(true);
                    }
                }
            });
        } else {
            controller._hiddenHierarchySection(toggle);
        }
    },

    _hiddenHierarchySection: function (toggle) {
        var controller = this;
        controller._setDirty(true);

        controller._hideHierarchySection(toggle);
    },

    _hideHierarchySection: function (toggle) {
        var view = this.getView();

        view.hiddenTable.setVisible(toggle);
        view.hiddenAddButton.setVisible(toggle);
        view.hiddenRemoveButton.setVisible(toggle);
        view.hiddenRefineButton.setVisible(toggle);
        view.hiddenUpButton.setVisible(toggle);
        view.hiddenDownButton.setVisible(toggle);

        if (toggle) {
            view.xLayout.setHeight("25%");
            view.yLayout.setHeight("25%");
            view.filterLayout.setHeight("25%");
            view.hiddenHLayout.setHeight("25%");

            view.contentInnerContainerArea.rerender();
        } else {
            view.xLayout.setHeight("33%");
            view.yLayout.setHeight("33%");
            view.filterLayout.setHeight("34%");

            view.contentInnerContainerArea.rerender();
        }
    },

    /* Called when the user changes Name field in the worksheet template view. Check to see if changes to the name would
     * affect (bulk update) any other worksheet template with the same name.  Display a warning message if it does. */
    handleWorksheetNameChange: function (event) {
        var nameInput = sap.ui.getCore().byId("createWorksheetNameInput");
        var newWorksheetName = nameInput.getValue().trim();
        var view = this.getView();
        var editedTemplate = view.getModel().getData().editedTemplate;    // might be different from the original template
        var otherSheetsWithSameName = this.findOtherSheetsWithName(newWorksheetName, editedTemplate && editedTemplate.id);
        if (otherSheetsWithSameName && otherSheetsWithSameName.length > 0 &&
            otherSheetsWithSameName[0].name !== newWorksheetName) {   // only possible when case is different
            var nameToBeChanged = otherSheetsWithSameName[0].name;
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageDialog.warning({
                text: rb.getText("sheetTemplate.renameAllVersions.confirmation.msg.fmt.txt", [nameToBeChanged, newWorksheetName]),
                title: rb.getText("sheetTemplate.edit.window.title")
            });
        }
    },

    findOtherSheetsWithName: function (name, excludeId) {
        var modelData = this.getView().getModel().getData();
        return _.filter(modelData.templates, function (template) {
            return (template.name.toLowerCase() === name.toLowerCase() && template.id !== excludeId);
        });
    },

    createWorksheetDescLiveChange: function (event) {
        this._setDirty(true);
    },

//     refineListLiveChange: function () {
// // Search does not work with treeTable model.
// // If we rearrange the model and remove the root->children
// // then its possible to get the search to work correctly
//
//         var search = sap.ui.getCore().byId("refineHierarchiesDialog.searchField").getValue() || "";
//         var table = sap.ui.getCore().byId("refineHierarchiesDialog.sourceHierarchiesTable");
//
//         var filters = [];
//         if (search && search != "") {
//             var oFilter = new sap.ui.model.Filter("levelName", sap.ui.model.FilterOperator.Contains, search);
//             filters.push(oFilter);
//         }
//         var binding = table.getBinding("rows");
//         binding.filter(oFilter);
//     },

    filterHierarchyListLiveChange: function () {
        var view = this.getView();
        var search = view.searchField.getValue() || "";
        var table = view.hierarchiesTreeTable;
        var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, search);
        var lineageFilter = new sap.ui.model.Filter("lineage", sap.ui.model.FilterOperator.Contains, search);
        var versionFilter = new sap.ui.model.Filter("version", sap.ui.model.FilterOperator.Contains, search);
        var searchFilter = new sap.ui.model.Filter([nameFilter, versionFilter, lineageFilter]);

        var binding = table.getBinding("rows");
        binding.filter(searchFilter);
    },

    // HF2 changes---
    _refineDialogSubtotalSelect:function(e){
        var hierarchyItem = this.getView()[this.globalAxis + "Table"].getModel().getProperty(this.globalSelectedNodePath);
        // set up the subtotalCode property of the item..
        // for HF2 this will be called only for X-axis
        hierarchyItem.subtotalCode= e.getParameter("selected")?"YES_SUBTOTALS":"NO_SUBTOTALS";
        sap.ui.getCore().byId("refineHierarchiesDialog.saveButton").setEnabled(true);
        // the view is dirty
        this._setDirty(true);
    }
});