jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.hc.m.DatePicker");
jQuery.sap.require("sas.hc.m.Input");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.m.IconTabBar");
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.hc.ui.commons.windowshade.WindowShade");
jQuery.sap.require("sas.hc.ui.commons.windowshade.WindowShadeSection");
jQuery.sap.require("sas.hc.ui.layout.form.Form");
jQuery.sap.require("sas.hc.ui.layout.form.ResponsiveGridLayout");
jQuery.sap.require("sas.hc.ui.layout.form.FormContainer");
jQuery.sap.require("sas.hc.ui.layout.form.FormElement");
jQuery.sap.require("sas.hc.ui.table.VisibleRowCountMode");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.risk.common.widgets.HelpLabel");
jQuery.sap.require("sas.risk.common.widgets.SelectTable");

sap.ui.jsfragment("sas.stress.config.view.HierarchyPropertiesDialog", {

    createContent: function (controller) {
        var NO_VERSION = "__NO__VERSION__";
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        var tabBar = new sas.hc.m.IconTabBar({
            firstTabClosable: false,
            showOverflowPopOver: false
        });

        var settingsTab = this._createSettingsTab(controller);
        tabBar.addItem(settingsTab);

        var memberPropertiesTab = this._createMemberPropertiesTab();
        tabBar.addItem(memberPropertiesTab);

        var containerTab =  sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({
            content: tabBar
        });

        var dialog = new sas.hc.m.Dialog({
            id: "sas.stress.config.view.HierarchyPropertiesDialog",
            title: "{{/subledgerData/accountingHierarchy/name}}", //rb.getText("editProjectDialog.window.title"),
            //initialFocus: nameInput,
            content: [containerTab],
            contentWidth: "60rem", // S1387014
            contentHeight: "30rem",
            beginButton: new sas.hc.m.Button({
                id: this.createId("editHierarchyProperties.savePropertiesButton"),
                text: rb.getText("editHierarchyProperties.button.save.action.label"),
                enabled: false,
               // press: $.proxy(controller.handleHierarchyPropertiesSaveButtonPress, controller)
            }),
            endButton: new sas.hc.m.Button({
                text: rb.getText("editHierarchyProperties.button.cancel.action.label"),
                press: function () {
                    dialog.close();
                }
            }),
            afterClose: function () {
                dialog.destroy();
            }
        });

        return dialog;
    },

    _createSettingsTab: function(controller) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        this.table = this._createPropertiesTable(controller);

        var that = this;
        var layout = new sas.hc.ui.layout.VerticalLayout({
            content: [
                this.table
            ]
        });


        var settingsTab = new sas.hc.m.IconTabFilter({
            key: rb.getText("editHierarchyProperties.tab1.title") ,
            text: rb.getText("editHierarchyProperties.tab1.title"),
            closable: false
        });

        settingsTab.addContent(layout);
        return settingsTab;
    },

    _createPropertiesTable: function (controller) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        var table = new sas.hc.ui.table.Table({
            id: "propertiesTable",
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowSelector,
            toolbar: this._createPropertiesToolbar(controller),
            noData: rb.getText("editHierarchyProperties.propertiesTable.noData.txt"),
            showSelectionControls: true,
            showTableOptions: true,
            //cellClick: $.proxy(controller.projectsTableCellClick, controller),
            //rowSelectionChange: $.proxy(controller.propertiesTableRowSelectionChange, controller),
            //afterColumnsDialogClose: $.proxy(controller.onColumnVisibility, controller) //S1389396
        });

        table.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("propertyName"),
            label: new sas.hc.m.Label({text: rb.getText("editHierarchyProperties.propertiesTable.column.name.header.title")}),
            template: new sas.hc.m.Toolbar({
                width: "100%",
                content: [
                    new sas.hc.m.Text({
                        text: "{name}",
                        tooltip: "{name}"
                    }),
                    new sas.hc.m.ToolbarSpacer(),
                    new sas.hc.ui.core.Icon({
                        size: "1rem",
                        src: {
                            path: 'locked',
                            formatter: function (flag) {
                                return (flag) ? sas.icons.HC.TABLEHEADERLOCK : sas.icons.HC.BLANK;
                            }
                        }
                    })
                ]
            }),
            sortProperty: "name",
            width: "15rem",
            name: "name"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("description"),
            label: new sas.hc.m.Label({text: rb.getText("editHierarchyProperties.propertiesTable.column.description.header.title")}),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: "{description}",
                tooltip: "{description}"
            }),
            sortProperty: "description",
            name: "description"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("asOfDate"),
            label: new sas.hc.m.Label({text: rb.getText("editHierarchyProperties.propertiesTable.column.asOfDate.header.title")}),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: {
                    path: "asOfDate",
                    formatter: function (dttm) {
                        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance();
                        return dateFormat.format(new Date(dttm), true);  // The true sets it to use UTC time zone;  without it, it subtracts a day
                    }
                },
                tooltip: {
                    path: "asOfDate",
                    formatter: function (dttm) {
                        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance();
                        return dateFormat.format(new Date(dttm), true);  // The true sets it to use UTC time zone;  without it, it subtracts a day
                    }
                }
            }),
            sortProperty: "asOfDate"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("valueType"),
            label: new sas.hc.m.Label({text: rb.getText("editHierarchyProperties.propertiesTable.column.valueType.header.title")}),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: "{valueType}",
                tooltip: "{valueType}"
            }),
            sortProperty: "valueType",
            name: "valueType"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("Required"),
            label: new sas.hc.m.Label({text: rb.getText("editHierarchyProperties.propertiesTable.column.required.header.title")}),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: "{Required}",
                tooltip: "{Required}"
            }),
            sortProperty: "Required",
            name: "Required"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: this.createId("DefaultValue"),
            label: new sas.hc.m.Label({text: rb.getText("editHierarchyProperties.propertiesTable.column.defaultValue.header.title")}),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: "{DefaultValue}",
                tooltip: "{DefaultValue}"
            }),
            sortProperty: "DefaultValue",
            name: "DefaultValue"
        }));

        table.addStyleClass("projectManagerTable");
        table.bindRows("/");

        //S1354153: Do sorting here instead of in controller so it won't overwrite LayoutService
        var bindingInfo = table.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [new sap.ui.model.Sorter("name", false)];
            bindingInfo.sorter = sorters;
        }

        return table;
    },

    _createPropertiesToolbar: function (controller) {

        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        this.newPropertyButton = new sas.hc.m.Button({
            icon: sas.icons.HC.NEWSTARBURST,
          //  press: $.proxy(controller.newPropertyButtonPress, controller),
           // enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.CREATE),
            tooltip: rb.getText("editHierarchyProperties.createButton.tip")
        });

        this.editPropertyButton = new sas.hc.m.Button({
            icon: sas.icons.HC.EDIT,
           // press: $.proxy(controller.editPropertyButtonPress, controller),
            enabled: false,
            tooltip: rb.getText("editHierarchyProperties.editButton.tip")
        });

        this.deletePropertyButton = new sas.hc.m.Button({
            icon: sas.icons.HC.DELETE,
           // press: $.proxy(controller.deletePropertyButtonPress, controller),
            enabled: false,
            tooltip: rb.getText("editHierarchyProperties.deleteButton.tip")
        });

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                //this._createSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                this.newPropertyButton,
                this.editPropertyButton,
                this.deletePropertyButton
            ]
        });

    },



    _createMemberPropertiesTab: function(controller) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        var that = this;

        // Define some sample data
        var oData = this.generateData(50, 0);

        // Create columns for the data.
        var aColumns = [
            new sas.hc.ui.table.Column({
                label: "Name",
                template: "name",
                width: "50px"
            })
            // new sas.hc.ui.table.Column({
            //     label: "Description",
            //     template: "description",
            //     width: "50px"
            // })
        ];
        // for (var i = 0; i < 3; i++) {
        //     aColumns.push(new sas.hc.ui.table.Column({
        //         label: rb.getText("samples.TreeTable.common.labels.col.fmt", [i]),
        //         template: "Col_" + i,
        //         width: "100px"
        //     }));
        // }

        var oTable = new sas.hc.ui.table.TreeTable({
            columns: aColumns,
            width: "30%",
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            enableColumnReordering: true,
            expandFirstLevel: true
            // toggleOpenState: function(oEvent) {
            //     var iRowIndex = oEvent.getParameter("rowIndex");
            //     var oRowContext = oEvent.getParameter("rowContext");
            //     var bExpanded = oEvent.getParameter("expanded");
            //     var sLabelText = rb.getText("samples.TreeTable.common.toggleText.fmt", [iRowIndex, oRowContext.getPath(), bExpanded]);
            //     oTableLabel.setText(sLabelText);
            // },
            // extension: [
            //     oTableLabel,
            //     oToggleBtn
            //]
        });

        var oModel = new sap.ui.model.json.JSONModel(oData);
        oTable.setModel(oModel);
        oTable.bindRows("/root");

        var dimPropertyTreeTable = new sas.hc.ui.table.TreeTable({
            columns: [
                new sas.hc.ui.table.Column({
                        id: this.createId("name"),
                        label: new sas.hc.m.Label({ text: rb.getText("editHierarchyProperties.treeTable.column.name.header.title") }),
                        template: new sas.hc.m.Text({ text: "{name}", tooltip: "{name}", wrapping: false }),
                        sortProperty: "name",
                     width: "15rem"
                    })
                ],
                visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
                minAutoRowCount: 1,
                selectionMode: sas.hc.ui.table.SelectionMode.Single,
                selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
               // rowSelectionChange: $.proxy(controller.dimensionsTreeTableRowSelectionChange, controller),
                noData: rb.getText("dimensionsTreeTable.noData.loading.txt"),
        });

        var dimPropertyItemTemplate = new sas.hc.ui.core.Item({key: "{id}", text:"{name}"});
        var dimPropertyDetails = new sas.hc.m.Select({
            id: this.createId("CreateOrEditProjectTemplate.processDefnNameSelect"),
            items: {
                path: "/definitions",
                template: dimPropertyItemTemplate
            },
            change: function() {
                var processDefn = parseInt(this.getSelectedItem().getKey());
                //processNotification.setEnabled(processDefn > 0);
                //processNotification.setSelected(false);
                //processStartDatePicker.setEnabled(processDefn > 0);
                //processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", !processStartDatePicker.getEnabled());
                //controller.toggleEditProjectTemplateDialogSaveButton();
            },
            enabled: {
                path:"/projectIsLocked",
                formatter:function(isLocked){
                    if(isLocked===undefined||isLocked===null){
                        return true;
                    }
                    return isLocked;
                }
            }
        });
        var processDefnLabel = new sas.hc.m.Label({
            text: "Property",
            labelFor: dimPropertyDetails
        })//.addStyleClass("createProjectTemplateDialogProcessDefn ");


        var layout = new sas.hc.ui.layout.HorizontalLayout({
            content: [
                //dimPropertyTreeTable,
                oTable,
                dimPropertyDetails
            ]
        });


        var memberPropertiesTab = new sas.hc.m.IconTabFilter({
            key: rb.getText("editHierarchyProperties.tab2.title"),
            text: rb.getText("editHierarchyProperties.tab2.title"),
            closable: false
        });

        memberPropertiesTab.addContent(layout);
        return memberPropertiesTab;
    },

    generateData: function(children, columns) {

        var data = [{
            name: "root"
        }];

        for (var i = 0; i < children; i++) {
            var node = {
                name: "Name"+ [i],
                description: "Desc"+ [i]
            };

            for (var col = 0; col < columns; col++) {
                node[("Col_" + col)] = i + " : " + col;
            }

            data.push(node);
        }
        // Connect parent and children nodes
        var dataLength = data.length;
        var numChildren = 5;
        data.forEach(function(item, index) {
            var pos = (index * numChildren);
            for (var offset = 1; offset <= numChildren; offset++) {
                if (pos + offset < dataLength) {
                    item[offset-1] = data[pos+offset];
                }
            }
        });

        return {
            root: data[0]
        };
    }

});