jQuery.sap.declare("sas.stress.data.model.DataLoadHistoryItem");
jQuery.sap.require("sas.stress.data.DataAPI");

sap.ui.base.Object.extend("sas.stress.data.model.DataLoadHistoryItem", {
    constructor: function (dataLoadTypeCode, projectId, projectName, projectIteration, userName, uploadDate, description,
                           fileName, fileLocationType, numRecords, fileSize, dataFormat, dataLoadMapName, dataPointSource) {
        this.dataLoadTypeCode = dataLoadTypeCode;
        this.projectId = projectId;
        this.projectName = projectName;
        this.projectIteration = projectIteration;
        this.userName = userName;
        this.uploadDate = uploadDate;
        this.description = description;
        this.fileName = fileName;
        // Handle conversion here, to ensure filtering works correctly.
        switch (fileLocationType) {
            case "client":
            case "server":
            case "landingArea":
            case "modelGroupRunLoad":
            case "archive":
                var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
                this.fileLocationType = rb.getText("fileLocationType." + fileLocationType + ".txt");
                break;
            default:
                this.fileLocationType = fileLocationType;
        }
        this.numRecords = numRecords;
        this.fileSize = fileSize;
        this.dataFormat = dataFormat;
        this.dataLoadMapName = dataLoadMapName;
        this.dataPointSource = dataPointSource;
    }
});

sas.stress.data.model.DataLoadHistoryItem.prototype.getDataLoadTypeCode = function () {
    return this.dataLoadTypeCode;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getProjectId = function () {
    return this.projectId;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getProjectName = function () {
    return this.projectName;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getProjectIteration = function () {
    return this.projectIteration;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getUserName = function () {
    return this.userName;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getUploadDate = function () {
    return this.uploadDate;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getDescription = function () {
    return this.description;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getFileName = function () {
    return this.fileName;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getFileLocationType = function () {
    return this.fileLocationType;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getNumRecords = function () {
    return this.numRecords;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getFileSize = function () {
    return this.fileSize;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getDataFormat = function () {
    return this.dataFormat;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getDataLoadMapName = function () {
    return this.dataLoadMapName;
};

sas.stress.data.model.DataLoadHistoryItem.prototype.getDataPointSource = function () {
    return this.dataPointSource;
};

sas.stress.data.model.DataLoadHistoryItem.getProjectDataLoadHistory = function (projectId) {
    var deferred = $.Deferred();
    sas.stress.data.DataAPI.listProjectDataLoadHistory(projectId).done(function (dataLoadHistoryItemsJSON) {
        var dataLoadHistoryItems = [];
        _.each(dataLoadHistoryItemsJSON, function (dataLoadHistoryItemJSON) {
            var dataLoadHistoryItem = new sas.stress.data.model.DataLoadHistoryItem(dataLoadHistoryItemJSON.code,
                dataLoadHistoryItemJSON.eventDetail.projectId, dataLoadHistoryItemJSON.eventDetail.dataLoad.projectName,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.iterationNum, dataLoadHistoryItemJSON.user,
                dataLoadHistoryItemJSON.createdTimeStamp, dataLoadHistoryItemJSON.description,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.fileName,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.fileLocationType,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.numRecords,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.fileSize,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.dataFormat,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.dataLoadMapName,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.dataPointSource);
            dataLoadHistoryItems.push(dataLoadHistoryItem);
        });
        deferred.resolve(dataLoadHistoryItems);
    })
        .fail(function (jqXHR) {
            deferred.reject(jqXHR);
        });
    return deferred;
};

sas.stress.data.model.DataLoadHistoryItem.getConfigDataLoadHistory = function (global) {
    var deferred = $.Deferred();
    sas.stress.data.DataAPI.listConfigDataLoadHistory(global).done(function (dataLoadHistoryItemsJSON) {
        var dataLoadHistoryItems = [];
        _.each(dataLoadHistoryItemsJSON, function (dataLoadHistoryItemJSON) {
            var dataLoadHistoryItem = new sas.stress.data.model.DataLoadHistoryItem(dataLoadHistoryItemJSON.code,
                dataLoadHistoryItemJSON.eventDetail.projectId, dataLoadHistoryItemJSON.eventDetail.dataLoad.projectName,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.iterationNum, dataLoadHistoryItemJSON.user,
                dataLoadHistoryItemJSON.createdTimeStamp, dataLoadHistoryItemJSON.description,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.fileName,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.fileLocationType,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.numRecords,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.fileSize,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.dataFormat,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.dataLoadMapName,
                dataLoadHistoryItemJSON.eventDetail.dataLoad.dataPointSource);
            dataLoadHistoryItems.push(dataLoadHistoryItem);
        });
        deferred.resolve(dataLoadHistoryItems);
    })
        .fail(function (jqXHR) {
            deferred.reject(jqXHR);
        });
    return deferred;
};
