jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.data.model.DataLoadHistoryItem");
jQuery.sap.require("sas.stress.data.model.DataLoadMapItem");
jQuery.sap.require("sas.stress.data.model.DataLoadMapItemDetail");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sap.ui.unified.FileUploaderParameter");

sap.ui.controller("sas.stress.data.view.DataEditor", {

    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("DataEditor", this._init, this);
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());

        new sas.risk.common.utils.LayoutService(this.getView().globalDataTable, "DataEditorGlobalDataTable");
        new sas.risk.common.utils.LayoutService(this.getView().projectDataTable, "DataEditorProjectDataTable");
        new sas.risk.common.utils.LayoutService(this.getView().dataLoadMapTable, "DataLoadMapTable");

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.DATA);
        this.rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
    },
    _init: function (event) {
    },

    onBeforeRendering: function () {
        this.refreshGlobalData();
        this.refreshProjectData();
    },

    refreshGlobalData: function() {
        this._getGlobalDataLoadHistory();
    },

    refreshProjectData: function() {
        this._getProjectDataLoadHistory();
    },

    refreshDataLoadMap: function(){
        this._getDataLoadMap();
    },

    onExit: function () {
        this.router.detachExactRouteMatched("DataEditor", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.data.view.DataEditor", null);
        this.navChecker.stop();
    },

    _getProjectDataLoadHistory: function () {
        var controller = this;
        var view = this.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({bundleName: "sas.stress.data.messagebundle"});
        view.setModel(i18N, "i18N");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        view.projectDataTable.setNoData(rb.getText("projectDataLoadHistoryTable.noData.loading.txt"));
        view.projectDataTable.setBusyIndicatorDelay(0);
        view.projectDataTable.setBusy(true);

        var projectData = [];
        sas.stress.data.model.DataLoadHistoryItem.getConfigDataLoadHistory(false).done(function (dataLoadHistoryItems) {
            _.each(dataLoadHistoryItems, function (dataLoadHistoryItem) {
                var translatedSource=dataLoadHistoryItem.getDataPointSource();
                switch (dataLoadHistoryItem.getDataPointSource()) {
                    case "calculatedField":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.calculatedfield.column.value.txt");
                        break;
                    case "sheetOverride":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.sheetoverride.column.value.txt");
                        break;
                    case "dataLoad":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.dataload.column.value.txt");
                        break;
                    case "modelExecution":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.modelExecution.column.value.txt");
                        break;
                    case "rollup":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.rollup.column.value.txt");
                        break;
                    case "excelImport":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.excelimport.column.value.txt");
                        break;
                    case "archiveImport":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.archiveImport.column.value.txt");
                        break;
                    case "allocationOverride":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.allocationOverride.column.value.txt");
                        break;
                    case "externalOverride":
                        translatedSource = rb.getText("dataLoadHistoryTable.source.externalOverride.column.value.txt");
                        break;
                }
                projectData.push({
                    projectName: dataLoadHistoryItem.getProjectName(),
                    projectId: dataLoadHistoryItem.getProjectId(),
                    projectIteration: dataLoadHistoryItem.getProjectIteration(),
                    userName: dataLoadHistoryItem.getUserName(),
                    uploadDate: dataLoadHistoryItem.getUploadDate(),
                    fileName: dataLoadHistoryItem.getFileName(),
                    fileLocationType: dataLoadHistoryItem.getFileLocationType(),
                    numRecords: dataLoadHistoryItem.getNumRecords(),
                    fileSize: dataLoadHistoryItem.getFileSize(),
                    dataFormat: dataLoadHistoryItem.getDataFormat(),
                    dataLoadMapName: dataLoadHistoryItem.getDataLoadMapName(),
                    dataPointSource: translatedSource
                });
            });

            var model = view.projectDataTable.getModel("projectDataLoadHistory");
            if (model instanceof sap.ui.model.json.JSONModel) {
                model.setData(projectData);
            } else {
                var newModel = new sas.risk.common.models.RiskJSONModel(projectData);
                view.projectDataTable.setModel(newModel, "projectDataLoadHistory");
                //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
                //var sorters = [new sap.ui.model.Sorter("uploadDate", true),
                //    new sap.ui.model.Sorter("projectName", false),
                //    new sap.ui.model.Sorter("fileName", false)];
                //view.projectDataTable.getBinding("rows").sort(sorters);
            }
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.projectDataTable.setBusy(false);
            view.projectDataTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                text: "{i18N>dataLoadHistoryTable.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>dataLoadHistoryTable.noData.button.txt}",
                        enabled: view.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.IMPORT),
                        press: $.proxy(controller.loadProjectDataButtonPress, controller)
                    })
                ]
            }));
        });
    },

    loadProjectDataButtonPress: function (event) {
        var controller = this;
        var view = sas.stress.wkbench.ViewManager.loadLoadDataEditor({projectMode: true});
        var i18N = new sap.ui.model.resource.ResourceModel({bundleName: "sas.stress.data.messagebundle"});
        var dialog = new sas.hc.m.Dialog({
            title: i18N.getProperty("loadDataEditor.window.title"),
            contentWidth: sas.stress.wkbench.StressUtil.dialogMedium,
            content: view,
            beginButton: view.okButton,
            endButton: view.cancelButton,
            afterClose: function () {
                dialog.destroy();
            },
            initialFocus: view.cancelButton
        });
        dialog.setModel(i18N, "i18N");
        view.attachClosing(function () {
            this.destroy();
        }, dialog);
        view.attachDataLoaded(function () {
            //Reload data in the table
            controller._getProjectDataLoadHistory();
        }, dialog);
        dialog.open();
    },

    onProjectDataFilterChanged: function (e) {
        var value = e.getSource().getValue() || "";
        var projectIterationFilter = new sap.ui.model.Filter("projectIteration", sap.ui.model.FilterOperator.EQ, value);
        var projectNameFilter = new sap.ui.model.Filter("projectName", sap.ui.model.FilterOperator.Contains, value);
        var filenameFilter = new sap.ui.model.Filter("fileName", sap.ui.model.FilterOperator.Contains, value);
        var loadedByFilter = new sap.ui.model.Filter("userName", sap.ui.model.FilterOperator.Contains, value);
        var fileLocationTypeFilter = new sap.ui.model.Filter("fileLocationType", sap.ui.model.FilterOperator.Contains, value);
        var allFilters = new sap.ui.model.Filter([projectIterationFilter, projectNameFilter, filenameFilter, loadedByFilter, fileLocationTypeFilter]);
        var binding = this.getView().projectDataTable.getBinding("rows");
        if (binding !== undefined) {
            binding.filter(allFilters);
        }
    },

    _getGlobalDataLoadHistory: function () {
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        view.globalDataTable.setNoData(rb.getText("dataLoadHistoryTable.noData.loading.txt"));
        view.globalDataTable.setBusyIndicatorDelay(0);
        view.globalDataTable.setBusy(true);

        sas.stress.data.model.DataLoadHistoryItem.getConfigDataLoadHistory(true).done(function (dataLoadHistoryItems) {
            var json = [];
            _.each(dataLoadHistoryItems, function (dataLoadHistoryItem) {
                var translatedSource=dataLoadHistoryItem.getDataPointSource();
                switch (dataLoadHistoryItem.getDataPointSource()) {
                    case "calculatedField":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.calculatedfield.column.value.txt");
                        break;
                    case "sheetOverride":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.sheetoverride.column.value.txt");
                        break;
                    case "dataLoad":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.dataload.column.value.txt");
                        break;
                    case "modelExecution":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.modelExecution.column.value.txt");
                        break;
                    case "rollup":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.rollup.column.value.txt");
                        break;
                    case "excelImport":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.excelimport.column.value.txt");
                        break;
                    case "archiveImport":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.archiveImport.column.value.txt");
                        break;
                    case "allocationOverride":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.allocationOverride.column.value.txt");
                        break;
                    case "externalOverride":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.externalOverride.column.value.txt");
                        break;
                }
                json.push({
                    userName: dataLoadHistoryItem.getUserName(),
                    uploadDate: dataLoadHistoryItem.getUploadDate(),
                    fileName: dataLoadHistoryItem.getFileName(),
                    fileLocationType: dataLoadHistoryItem.getFileLocationType(),
                    numRecords: dataLoadHistoryItem.getNumRecords(),
                    fileSize: dataLoadHistoryItem.getFileSize(),
                    dataFormat: dataLoadHistoryItem.getDataFormat(),
                    dataLoadMapName: dataLoadHistoryItem.getDataLoadMapName(),
                    dataPointSource: translatedSource
                });
            });
            // To maintain any sort already applied on the table, if model exists, set data on the model.
            // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
            var model = view.globalDataTable.getModel("globalDataLoadHistory");
            if (model instanceof sap.ui.model.json.JSONModel) {
                model.setData(json);
            } else {
                var newModel = new sas.risk.common.models.RiskJSONModel(json);
                view.globalDataTable.setModel(newModel, "globalDataLoadHistory");
                //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
                //var sorters = [new sap.ui.model.Sorter("uploadDate", true), new sap.ui.model.Sorter("fileName", false)];
                //view.globalDataTable.getBinding("rows").sort(sorters);
            }
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.globalDataTable.setBusy(false);
            view.globalDataTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                text: "{i18N>dataLoadHistoryTable.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>dataLoadHistoryTable.noData.button.txt}",
                        enabled: view.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.IMPORT),
                        press: $.proxy(controller.loadGlobalDataButtonPress, controller)
                    })
                ]
            }));
        });
    },

    loadGlobalDataButtonPress: function (event) {
        var controller = this;
        var view = sas.stress.wkbench.ViewManager.loadLoadDataEditor({projectMode: false});
        var i18N = new sap.ui.model.resource.ResourceModel({bundleName: "sas.stress.data.messagebundle"});
        var dialog = new sas.hc.m.Dialog({
            title: i18N.getProperty("loadDataEditor.window.title"),
            contentWidth: sas.stress.wkbench.StressUtil.dialogMedium,
            content: view,
            beginButton: view.okButton,
            endButton: view.cancelButton,
            afterClose: function () {
                dialog.destroy();
            },
            initialFocus: view.cancelButton
        });
        dialog.setModel(i18N, "i18N");
        view.attachClosing(function () {
            this.destroy();
        }, dialog);
        view.attachDataLoaded(function () {
            //Reload data in the table
            controller._getGlobalDataLoadHistory();
        }, dialog);
        dialog.open();
    },

    onGlobalDataFilterChanged: function (e) {
        var value = e.getSource().getValue() || "";
        var filenameFilter = new sap.ui.model.Filter("fileName", sap.ui.model.FilterOperator.Contains, value);
        var loadedByFilter = new sap.ui.model.Filter("userName", sap.ui.model.FilterOperator.Contains, value);
        var fileLocationTypeFilter = new sap.ui.model.Filter("fileLocationType", sap.ui.model.FilterOperator.Contains, value);
        var allFilters = new sap.ui.model.Filter([filenameFilter, loadedByFilter, fileLocationTypeFilter]);
        var binding = this.getView().globalDataTable.getBinding("rows");
        if (binding !== undefined) {
            binding.filter(allFilters);
        }
    },

    _getDataLoadMap:function(){
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        view.dataLoadMapTable.setNoData(rb.getText("dataLoadHistoryTable.noData.loading.txt"));
        view.dataLoadMapTable.setBusyIndicatorDelay(0);
        view.dataLoadMapTable.setBusy(true);

        sas.stress.data.model.DataLoadMapItem.getDataLoadMap().done(function (DataLoadMapItems) {
           var dataLoadMapData = [];
            _.each(DataLoadMapItems, function (DataLoadMapItem) {
                dataLoadMapData.push({
                    mapId: DataLoadMapItem.getMapId(),
                    mapName: DataLoadMapItem.getMapName(),
                    createdBy: DataLoadMapItem.getCreatedBy(),
                    modifiedBy: DataLoadMapItem.getModifiedBy(),
                    mapDesc: DataLoadMapItem.getMapDesc(),
                    dateCreated: DataLoadMapItem.getDateCreatedTimeStamp(),
                    dateModified: DataLoadMapItem.getDateModifiedTimeStamp(),
                    variables: DataLoadMapItem.getVariables()
                });
            });

            // To maintain any sort already applied on the table, if model exists, set data on the model.
            // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
            var model = view.dataLoadMapTable.getModel("dataLoadMap");
            if (model instanceof sap.ui.model.json.JSONModel) {
                model.setData(dataLoadMapData);
                model.refresh();
            } else {
                var newModel = new sas.risk.common.models.RiskJSONModel(dataLoadMapData);
                view.dataLoadMapTable.setModel(newModel, "dataLoadMap");
            }
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.dataLoadMapTable.setBusy(false);
            view.dataLoadMapTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                text: "{i18N>dataLoadMapTable.noData.title}"
            }));
        });
    },

    onDataLoadMapFilterChanged:function(event){
        var value = event.getSource().getValue() || "";
        var mapNameFilter = new sap.ui.model.Filter("mapName", sap.ui.model.FilterOperator.Contains, value);
        var createdByFilter = new sap.ui.model.Filter("createdBy", sap.ui.model.FilterOperator.Contains, value);
        var allFilters = new sap.ui.model.Filter([mapNameFilter, createdByFilter]);
        var binding = this.getView().dataLoadMapTable.getBinding("rows");
        if (binding !== undefined) {
            binding.filter(allFilters);
        }
    },

    dataLoadMapTableRowSelectionChanged:function(event){
        var view = this.getView();
        var selectedDataLoadMaps = this._getSelectedDataLoadMap();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var hasOneSelection = (selectedDataLoadMaps.length === 1);
        var hasNoSelection = (selectedDataLoadMaps.length === 0);
        var hasMoreSelection = (selectedDataLoadMaps.length > 1);
        view.editDataLoadMapButton.setEnabled(hasOneSelection && capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.UPDATE_MAP));
        view.deleteDataLoadMapButton.setEnabled( (hasOneSelection || hasMoreSelection) && capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.DELETE_MAP));

        if (hasOneSelection )
            this._loadDataLoadMapDetails(selectedDataLoadMaps);
        else if (hasNoSelection){
            view.dataLoadMapDetailTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                text: this.rb.getText("dataLoadMapDetailTable.noData.title")
            }));
            this._loadDataLoadMapDetails();
        }
        else if (hasMoreSelection){
            view.dataLoadMapDetailTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                text: this.rb.getText("dataLoadMapDetailTable.multiple.noData.title")
            }));
            this._loadDataLoadMapDetails();
        }

        this._clearDataLoadMapDetailTableFilter();
    },

    _clearDataLoadMapDetailTableFilter: function () {
        //Clear text in Perturbation table Search field and clear the filter.
        var dataLoadMapDetailTableSearchField = sap.ui.getCore().byId("dataLoadMapDetailTableSearchFld");
        dataLoadMapDetailTableSearchField.setValue("");
        this._filterDataLoadMapDetailTableSearchField("");
    },

    _filterDataLoadMapDetailTableSearchField: function (value) {
        var nameFilter = new sap.ui.model.Filter("sourceVariableName", sap.ui.model.FilterOperator.Contains, value);
        var binding = this.getView().dataLoadMapDetailTable.getBinding("rows");
        binding.filter(nameFilter);
    },

    _loadDataLoadMapDetails:function(selectedItem){
        var view = this.getView();
        var model = view.dataLoadMapTable.getModel("dataLoadMap");
        var mapDetail =[];

        if (model===undefined)
            return;

        if (selectedItem===undefined){
            var title = this.rb.getText("dataLoadMapDetailTable.noData.table.title");
            view.dataLoadMapDetailTable.setTitle(title);
        }else{
            mapDetail = selectedItem[0].variables;
            view.dataLoadMapDetailTable.setTitle(selectedItem[0].mapName);
        }

         var detailModel = view.dataLoadMapDetailTable.getModel("dataLoadMapDetail");
         if (detailModel instanceof sap.ui.model.json.JSONModel) {
                detailModel.setData(mapDetail);
                detailModel.refresh();
         } else {
                var newModel = new sas.risk.common.models.RiskJSONModel(mapDetail);
                view.dataLoadMapDetailTable.setModel(newModel, "dataLoadMapDetail");
         }
    },

    _getSelectedDataLoadMap:function(){
        var view = this.getView();
        var model = view.dataLoadMapTable.getModel("dataLoadMap");
        var data = model.getData();
        var indices = view.dataLoadMapTable.getSelectedIndices();
        var items = [];
        _.each(indices, function (selectedIdx) {
            var context = view.dataLoadMapTable.getContextByIndex(selectedIdx);
            var obj = context.getObject();
            var dataLoadMap = _.find(data, function (dataLoadMapData) {
                return dataLoadMapData.mapId === obj.mapId;
            });
            items.push(dataLoadMap);
        });
        return items;

    },

    //Create new data load map
    onOpenDataLoadMapDialog:function(event){
        var existingDialog = sap.ui.getCore().byId("sas.stress.data.view.DataLoadMapDialog");
        var model;

        if (existingDialog) {
            existingDialog.destroy();
        }
        else{
            var dialog = sap.ui.jsfragment("sas.stress.data.view.DataLoadMapDialog", this);
            var newDataLoadMapJSON = {
                title:this.rb.getText("dataLoadMap.createDialog.title"),
                type: "create",
                mapName
                    : sas.stress.data.model.DataLoadMapItem.getNewMapName()
            };
            var editDataLoadMapJSON = {
                title: this.rb.getText("dataLoadMap.editDialog.title"),
                type: "edit"
            };
            if (event.getParameter("id")==="editMap") {
                this.saveButton.setEnabled(true);
                model = new sas.risk.common.models.RiskJSONModel(editDataLoadMapJSON);
                var selectedDataloadMap = this._getSelectedDataLoadMap();
                if (selectedDataloadMap) {
                    var modelData = model.getData();
                    modelData['mapName'] = selectedDataloadMap[0].mapName;
                    modelData['mapDesc'] = selectedDataloadMap[0].mapDesc;
                }
            }
            else
                model = new sas.risk.common.models.RiskJSONModel(newDataLoadMapJSON);

            dialog.setModel(model);
            dialog.open();
        }
    },

    //Delete data load map
    onDeleteDataLoadMapClick:function(){
        var ctrl = this;
        var view = ctrl.getView();
        var dataLoadMap = ctrl._getSelectedDataLoadMap();
        var singleMap = (dataLoadMap.length ===1);
        var mapId, confirmMessage;
        var mapIds = [],
            mapNameList = [];

        if (dataLoadMap && dataLoadMap.length && dataLoadMap[0]) {
            var title = this.rb.getText("dataLoadMapTable.delete.confirmation.window.title");

            if (singleMap){
                mapNameList.push(dataLoadMap[0].mapName);
                mapId = dataLoadMap[0].mapId;
            }else{
                $.each(dataLoadMap, function(  i,selectedItem ) {
                    var id = selectedItem.mapId;
                    var name = selectedItem.mapName;
                    mapIds.push(id);
                    mapNameList.push(name);
                });
            }
            confirmMessage = sas.stress.wkbench.StressUtil.getConfirmationMessage(this.rb.getText("dataLoadMapTable.delete.confirmation.fmt.txt"), mapNameList);

            sas.hc.m.MessageDialog.warning({
                text: confirmMessage,
                title: title,
                buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getMessageDialogAction();
                    if (action === sas.hc.m.MessageDialogAction.Delete) {
                        if (singleMap){
                            sas.stress.data.DataAPI.deleteDataLoadMap(mapId).done(function () {
                                ctrl.refreshDataLoadMap();
                                view.dataLoadMapTable.clearSelection();
                            }).fail(function(jqxhr){
                                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                            });
                        }else{
                            sas.stress.data.DataAPI.deleteMultipleDataLoadMap(mapIds).done(function () {
                                ctrl.refreshDataLoadMap();
                                view.dataLoadMapTable.clearSelection();
                            }).fail(function(jqxhr){
                                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                            });
                        }
                    }
                }
            });

        }

    },

    handleDataLoadMapDialogSaveButtonPress: function (event) {
        if (this._isEditDataLoadMap()) {
            this.editDataLoadMapDialogSaveButtonPress(event);
        } else {
            this.addDataLoadMapDialogSaveButtonPress(event);
        }
    },


    //Save button is pressed when creating DataLoadMapDialog
    addDataLoadMapDialogSaveButtonPress:function(){
        var mapName = this.nameField.getValue();
        var mapDesc = this.descriptionField.getValue();
        var file = this.fileUploader.getValue();
        if (file) {
            this.busyDialog = new sas.hc.m.BusyDialog({
                title: this.rb.getText("DataLoadMapDialog.create.busyDialog.title")
            });
            this.busyDialog.open();

            var nameMetadata = new sap.ui.unified.FileUploaderParameter({name:"name",value:mapName});
            var descMetadata = new sap.ui.unified.FileUploaderParameter({name:"description",value:mapDesc});
            this.fileUploader.addParameter(nameMetadata);
            this.fileUploader.addParameter(descMetadata);
            this.fileUploader.setUploadUrl("rest/dataLoadMaps");
            this.fileUploader.upload();
            this.busyDialog.close();
        }
    },

    //Save button is pressed when editing DataLoadMapDialog
    editDataLoadMapDialogSaveButtonPress:function(){
        var ctrl = this;
        var name = this.nameField.getValue();
        var desc = this.descriptionField.getValue();
        var selectedItem = this._getSelectedDataLoadMap();
        var id = selectedItem[0].mapId;

        this.busyDialog = new sas.hc.m.BusyDialog({
            title: this.rb.getText("DataLoadMapDialog.update.busyDialog.title")
        });
        this.busyDialog.open();

        //Two separate update data load map steps
        sas.stress.data.DataAPI.updateDataLoadMapMetadata(id,name,desc).done(
                function(){
                    //Update the file
                  ctrl._updateFileUpload(id);
                  //Update detail table
                  ctrl._loadDataLoadMapDetails(selectedItem);
                  ctrl._closeBusyDialog();
                }
            ).fail(function (jqxhr){
                ctrl._closeBusyDialog();
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    _closeBusyDialog:function(){
        if (this.busyDialog)
            this.busyDialog.close();
    },

    _updateFileUpload:function(id){
        var ctrl = this;
        var file = this.fileUploader.getValue();
        if (file) {
            var fileName = new sap.ui.unified.FileUploaderParameter({name:"file",value:file});
            this.fileUploader.addParameter(fileName);
            this.fileUploader.setUploadUrl("rest/dataLoadMaps/vars/" +id);
            this.fileUploader.upload();
        }else{
            var dialog = sap.ui.getCore().byId("sas.stress.data.view.DataLoadMapDialog");
            dialog.setVisible(false);
            //Update the table view
            setTimeout(function(){
                ctrl.refreshDataLoadMap();
            },500);
            //clear table selection
            ctrl._clearDataLoadMapTable();
            dialog.close();
        }
    },

    onFileUploaded:function(event){
        var ctrl = this;
        var view = this.getView();
        var dialog = sap.ui.getCore().byId("sas.stress.data.view.DataLoadMapDialog");
        var response = event.getParameter("responseRaw");
        if (response) {
            var responseJSON = JSON.parse(response);
            if (responseJSON.hasOwnProperty("errorCode")) {
                this.busyDialog.close();
                sas.stress.wkbench.StressUtil.handleServerFailure({response:responseJSON});
            } else {
                this.busyDialog.close();
                dialog.setVisible(false);
                //Update the table view
                setTimeout(function(){
                    ctrl.refreshDataLoadMap();
                },500);
                //clear table selection
                ctrl._clearDataLoadMapTable();
                //view.fireEvent(view.getId() + "_DATAMAP_LOADED", { "view": view, "action": "DATA_MAP_LOADED" });
                dialog.close();
            }
        }
    },

    //File type mismatch
    onTypeMismatch:function(event){
        var aFileTypes = event.getSource().getFileType();
        $.each(aFileTypes, function (key, value) {
            aFileTypes[key] = "*." + value;
        });
        var sSupportedFileTypes = aFileTypes.join(", ");
        sas.hc.m.MessageDialog.error({text: this.rb.getText("fileUploadTypeMismatch.msg.fmt.txt", [event.getParameter("fileType"), sSupportedFileTypes]), title: this.rb.getText("loadDataEditor.window.title")});

    },

    //When file size exceeded
    onFileSizeExceed:function(event){
        sas.hc.m.MessageDialog.error({
            text: this.rb.getText("fileUploadFileSizeExceed.msg.fmt.txt", [event.getSource().getMaximumFileSize()]),
            title: this.rb.getText("loadProjectDataEditor.window.title")});

    },

    /* Called when the user changes any field in the Edit Data Load Map dialog. Validates the name by
     * checking if it is empty or duplicate of an existing data load map.
     * If validation fails, displays an error message around the Name and disables the Save button. */
    handleMapFieldChange:function(){
        var isValid = this.validateNameFields();
        //Toggle Save button based on the result of validation.
        if (!this._isEditDataLoadMap())
            isValid = (isValid && this.fileUploader.getValue()!=="");

        this.saveButton.setEnabled(isValid);
    },

    validateNameFields: function () {
        var view = this.getView();
             var isValid = true;
        var newName = this.nameField.getValue().trim();

        //Check if name is empty.
        if (newName === null || newName.trim() === "") {
            isValid = false;
            this.nameField.setValueStateText(this.rb.getText("error.requiredField.txt"));
            this.nameField.setValueState(sap.ui.core.ValueState.Error);
        } else {
            //Check if data load map with this name already exists.
            var exist = _.find(view.dataLoadMapTable.getModel("dataLoadMap").getData(), function (dataLoadMap) {
                return (dataLoadMap.mapName && dataLoadMap.mapName.toLowerCase() === newName.toLowerCase());
            });
            if (exist) {
                isValid = false;
                this.nameField.setValueStateText(this.rb.getText("error.duplicate.name.txt"));
                this.nameField.setValueState(sap.ui.core.ValueState.Error);
            } else {
                //Name is good.
                this.nameField.setValueStateText("");
                this.nameField.setValueState(sap.ui.core.ValueState.None);
            }
        }

        return isValid;
    },

    _isEditDataLoadMap: function () {
        var dataLoadMapDialog = this._getDataLoadMapDialog();
        return dataLoadMapDialog.getModel().getData().type ==="edit" ;
    },

    _getDataLoadMapDialog: function () {
        return sap.ui.getCore().byId("sas.stress.data.view.DataLoadMapDialog");
    },

    onDataLoadMapDetailTableFilterChanged:function(event){
        var value = event.getSource().getValue() || "";
        var sourceVarFilter = new sap.ui.model.Filter("sourceVarName", sap.ui.model.FilterOperator.Contains, value);
        var dataTypeFilter = new sap.ui.model.Filter("dataType", sap.ui.model.FilterOperator.Contains, value);
        var dimNameFilter = new sap.ui.model.Filter("dimName", sap.ui.model.FilterOperator.Contains, value);
        var mappingFilter = new sap.ui.model.Filter("mapping", sap.ui.model.FilterOperator.Contains, value);
        var hierarchyNameFilter = new sap.ui.model.Filter("hierarchyName", sap.ui.model.FilterOperator.Contains, value);
        var hierarchyVersion = new sap.ui.model.Filter("hierarchyVersion", sap.ui.model.FilterOperator.Contains, value);
        var levelNameFilter = new sap.ui.model.Filter("levelName", sap.ui.model.FilterOperator.Contains, value);

        var allFilters = new sap.ui.model.Filter([sourceVarFilter,dataTypeFilter,dimNameFilter, mappingFilter, hierarchyNameFilter, hierarchyVersion, levelNameFilter]);
        var binding = this.getView().dataLoadMapDetailTable.getBinding("rows");
        if (binding !== undefined) {
            binding.filter(allFilters);
        }

    },

    _clearDataLoadMapTable:function(){
        var view = this.getView();
        view.dataLoadMapTable.clearSelection();
    }

});