jQuery.sap.require("sas.stress.models.model.ModelGroupRun");
jQuery.sap.require("sas.stress.models.ModelGroupRunUtils");
jQuery.sap.require("sas.stress.models.ModelGroupRunConstants");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");

jQuery.sap.declare("sas.stress.models.ModelGroupRunAPI");
sas.stress.models.ModelGroupRunAPI = {

    getModelGroupRuns: function (projectKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/modelGroups/runs?projectKey=" + projectKey, function (data) {
            var runs = _.map(data.items, function (modelGroupRunJSON) {
                return sas.stress.models.model.ModelGroupRun._fromJSON(modelGroupRunJSON);
            });
//            deferred.resolve({});
            deferred.resolve(runs);
        }).fail(function (jqxhr, textStatus, error) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    getScenarioData:function(projectKey, scenarioKey) {
        var deferred = $.Deferred();
        $.ajax({
            type:'GET',
            url: "rest/projects/" + projectKey + "/scenarios/" + scenarioKey,
            dataType: "json",
            accepts: { json: "application/vnd.sas.risk.scenario+json;charset=utf-8" }
        }).done(function(data) {
            deferred.resolve(data);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });

        return deferred;
    },

    getScenariosForRuns: function (projectKey) {
        var deferred = $.Deferred();
        var rb = sas.stress.models.ModelGroupRunUtils.getResourceBundle();
        $.getJSON("rest/projects/" + projectKey + "/scenarios", function (data) {
            if (data.items && data.items.length > 0) {
                _.each(data.items, function (scenario) {
                    var displayablePeriodType = sas.stress.models.ModelGroupRunUtils.intervalFormatter(scenario.periodType);
                    if (scenario.scenarioVersion) {
                        scenario.listName = rb.getText("modelGroupRuns.scenario.listname.fmt", [scenario.name, scenario.scenarioVersion, displayablePeriodType]);
                    }else{
                        scenario.listName = rb.getText("modelGroupRuns.scenario.no.version.listname.fmt", [scenario.name, displayablePeriodType]);
                    }
                });
            }
            deferred.resolve(data.items);
        }).fail(function (jqxhr, textStatus, error) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    getEntityLineItemsForProject: function (projectKey) {
        var deferred = $.Deferred();
        $.ajax(
            {
                type: 'GET',
                url: "rest/projects/" + projectKey + "/entity",
                dataType: "json",
                accepts: { json: "application/json;charset=utf-8"/*+" application/vnd.sas.collection+json"*/ },
                dataFilter: function(data, type) {
                    // use MIP fixes for S1232756 for RFW defect S1301961
                    if(!data) {
                        return "{}"; // valid JSON
                    }
                    return data;
                }
            }
        ).done(function (data) {
            deferred.resolve(data.lineItems);
        }).fail(function (jqxhr, textStatus, error) {
            deferred.reject(jqxhr);
        });
        /*$.getJSON("rest/projects/" + projectKey + "/entity", function (data) {
            deferred.resolve(data.lineItems);
        }).fail(function (jqxhr, textStatus, error) {
            deferred.reject(jqxhr);
        });*/
        return deferred;
    },

    getModelGroupsForRuns: function (projectKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/modelGroups/projects/" + projectKey, function (data) {  // fixes for S1299354
            deferred.resolve(data.items);
        }).fail(function (jqxhr, textStatus, error) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    getModelGroupRunAccessControls: function (runKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/modelGroups/runs/" + runKey + "/accessControls", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, errorThrown) {
            jQuery.sap.log.error("loading access controls FAILURE: " + errorThrown);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadModelGroupRunsCreateArtifacts: function (projectId, success, failure) {
        $.when(sas.stress.models.ModelGroupRunAPI.getScenariosForRuns(projectId),
               sas.stress.models.ModelGroupRunAPI.getEntityLineItemsForProject(projectId),
               sas.stress.models.ModelGroupRunAPI.getModelGroupsForRuns(projectId))
         .done(function (scenarios, entities, modelGroups) {
             var CONSTANTS = sas.stress.models.ModelGroupRunConstants;
             var model = sap.ui.getCore().getModel(CONSTANTS.MODEL_MODEL_GROUP_RUNS_ARTIFACTS_MODEL);
             if (!model) {
                 model = new sas.risk.common.models.RiskJSONModel();
                 sap.ui.getCore().setModel(model, CONSTANTS.MODEL_MODEL_GROUP_RUNS_ARTIFACTS_MODEL);
             }
             model.setProperty(CONSTANTS.PATH_SCENARIOS, scenarios);
             model.setProperty(CONSTANTS.PATH_ENTITIES, entities);
             model.setProperty(CONSTANTS.PATH_MODEL_GROUPS, modelGroups);
             success();
         })
         .fail(failure);
    },

    loadModelGroupRunsSensitivityArtifacts: function (project, scenarioId, runIds, howToRun, workGroup, success, failure) {
        $.when(sas.stress.models.ModelGroupRunAPI.getScenarioData(project.getId(), scenarioId))
            .done(function (scenario) {
                var CONSTANTS = sas.stress.models.ModelGroupRunConstants;
                var model = sap.ui.getCore().getModel(CONSTANTS.MODEL_MODEL_GROUP_SENSITIVITY_ARTIFACTS_MODEL);
                if (!model) {
                    model = new sas.risk.common.models.RiskJSONModel();
                    sap.ui.getCore().setModel(model, CONSTANTS.MODEL_MODEL_GROUP_SENSITIVITY_ARTIFACTS_MODEL);
                }
                model.setProperty("/workGroup", workGroup);
                model.setProperty("/howToRun", howToRun);
                model.setProperty("/modelGroupRuns", runIds);
                model.setProperty("/project", project);
                model.setProperty("/scenario", scenario);
                success();
            })
            .fail(failure);
    },

    getScenarios: function () {
        var model = sap.ui.getCore().getModel(sas.stress.models.ModelGroupRunConstants.MODEL_MODEL_GROUP_RUNS_ARTIFACTS_MODEL);
        if (!model) {
            return [];
        }
        var items = model.getProperty(sas.stress.models.ModelGroupRunConstants.PATH_SCENARIOS);
        if (!items) {
            return [];
        }
        return items;
    },

    getEntities: function () {
        var model = sap.ui.getCore().getModel(sas.stress.models.ModelGroupRunConstants.MODEL_MODEL_GROUP_RUNS_ARTIFACTS_MODEL);
        if (!model) {
            return [];
        }
        var items = model.getProperty(sas.stress.models.ModelGroupRunConstants.PATH_ENTITIES);
        if (!items) {
            return [];
        }
        return items;
    },

    getModelGroups: function () {
        var model = sap.ui.getCore().getModel(sas.stress.models.ModelGroupRunConstants.MODEL_MODEL_GROUP_RUNS_ARTIFACTS_MODEL);
        if (!model) {
            return [];
        }
        var items = model.getProperty(sas.stress.models.ModelGroupRunConstants.PATH_MODEL_GROUPS);
        if (!items) {
            return [];
        }
        return items;
    },

    _updateModelGroupRunsWithAssociatedNames: function (controller, modelGroupRuns) {
        // For sorting on scenarios, entities, or model groups we have to add a sort property for each
        // to our list of runs, so here goes.
        _.each(modelGroupRuns, function (run) {
            var scenario = controller.getScenario(run.scenarioKey);
            if (scenario) {
                run.scenarioName = scenario.name;
            }
            var entity = controller.getEntity(run.entityLIKey);
            if (entity) {
                run.entityName = entity.memberName;
            }
            var modelGroup = controller.getModelGroup(run.modelGroupKey);
            if (modelGroup) {
                run.modelGroupName = modelGroup.name;
            }
        });

    },

    executeModelGroupRuns: function (projectKey, modelGroupRuns, workGroup) {
        var deferred = $.Deferred();
        var urlString = "rest/modelGroups/runs?projectKey=" + projectKey;
        if (!!workGroup && workGroup.length > 0) {
            urlString += "&workGroupName="+ workGroup;
        }
        if (modelGroupRuns.runMode && modelGroupRuns.runMode.length > 0) {
            urlString += "&howToRun=" + modelGroupRuns.runMode;
        }
        delete modelGroupRuns.runType;

        var stringifiedData = JSON.stringify(modelGroupRuns);

        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
                type: "POST",
                url: urlString,
                data: stringifiedData,
                dataType: "json"
            })
         .done(function (data) {
             deferred.resolve(data);
         })
         .fail(function (jqxhr, textStatus, error) {
             jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
             deferred.reject(jqxhr);
         });
        return deferred;
    },

    executeModelGroupRunSensitivity: function (shocks, workGroup, howToRun) {

        var deferred = $.Deferred();
        var stringifiedData = JSON.stringify(shocks);
        var urlString = "rest/modelGroups/runs/sensitivityAnalysis?howToRun=" + howToRun;
        if (!!workGroup && workGroup.length > 0) {
            urlString += "&workGroupName=" + workGroup;
        }
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: urlString,
            data: stringifiedData,
            dataType: "json"
        }).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });

        return deferred;

    },

    /*
     * Deletes modelGroupRun(s)
     * @param runsToDelete An array of modelGroupRun ids to delete
     * @returns {*}
     */
    deleteRuns: function (runsToDelete) {
        var deferred = $.Deferred();

        if (runsToDelete.length === 1){ // S1398605
            $.ajax(
                {
                    type: "DELETE",
                    url: "rest/modelGroups/runs/" + runsToDelete[0]
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }else {
            var stringifiedData = JSON.stringify(runsToDelete);
            $.ajax({
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "DELETE",
                url: "rest/modelGroups/runs",
                data: stringifiedData,
                dataType: "json"
            }).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr, textStatus, error) {
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    /*
     * reruns the selected runs
     * @param runsIds An array of modelGroupRun keys to be rerun
     * @returns {*}
     */
    rerunRuns: function (runIds) {
        // fixes for S1304448
        var deferred=$.Deferred();  // the one deferred we are going to return
        var id=runIds[0];           // we allow only one..
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: "rest/modelGroups/runs/" + id,
                data: {},
                dataType: "json"
            }
        ).done(function(run) {
            deferred.resolve(run);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });

        return deferred;
    },

    /*
     * loads the Data for a run
     * @param resultsFile The file information for the result file to have the data loaded for
     * @returns {*}
     */
    loadDataFromRun: function (resultsFile, async) {
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/modelGroups/runs/"+resultsFile.runId+"/projectData?resultsFileUpdatedTimestamp="+resultsFile.updatedTimestamp,
            data: {},
            async: async,
            dataType: "json"
        }).done(function(data) {
            deferred.resolve(data);
        }).fail(function( jqxhr, textStatus, error ) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    downloadFile: function (file) {
        var url = "rest/modelGroups/runs/" + file.runId + "/results/" + file.filename;
        window.open(url, "_self");
    }

};