jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.models.ModelGroupRunConstants");
jQuery.sap.require("sas.stress.notification.NotificationUtil");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.SASStudioProxy");
jQuery.sap.require("sas.stress.wkbench.widget.CustomBreadcrumb");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");
jQuery.sap.require("sas.stress.wkbench.widget.StressObjStatus");
jQuery.sap.require("sas.hc.m.objectInspector.ObjectInspector");
jQuery.sap.require("sas.hc.m.FlexItemData");
jQuery.sap.require("sas.risk.common.widgets.Button");

jQuery.sap.require({ modName: "sas.stress.models.view.execution.ModelGroupRunsManager", type: "controller" });

sap.ui.jsview("sas.stress.models.view.execution.ModelGroupRunsManager", (function () {

    var self,
        myController,
        breadcrumb,
        createRunButton,
        runsTable,
        searchField,
        runButton,
        sensitivityButton,
        deleteButton;

    function _getResourceBundle() {
        return sas.stress.models.ModelGroupRunUtils.getResourceBundle();
    }

    function _createTableHeader(controller) {

        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        createRunButton = new sas.hc.m.Button({
            id: "modelGroupRuns-createRunBtn",
            icon: sas.icons.HC.NEWSTARBURST,
            tooltip: _getResourceBundle().getText("modelGroupRuns.createRun.tooltip.txt"),
            enabled: {
                path: "/projectIsLocked",
                formatter:function(isLocked){
                    return !isLocked && capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MGR.CREATE);
                }
            },
            press: $.proxy(controller.navigateToCreateView, controller)
        });

        runButton = new sas.risk.common.widgets.Button({	// prevent double click
            id: "modelGroupRuns-runSelectedBtn",
            icon: sas.icons.HC.PLAY,
            tooltip: _getResourceBundle().getText("modelGroupRuns.runSelected.tooltip.txt"),
            press: $.proxy(controller.onRunSelected, controller),
            enabled: false
        });

        loadButton = new sas.risk.common.widgets.Button({
            id: "modelGroupRuns-loadSelectedBtn",
            icon: sas.icons.HC.LOADINTO,
            tooltip: _getResourceBundle().getText("modelGroupRuns.loadSelected.tooltip.txt"),
            press: $.proxy(controller.onLoadSelected, controller),
            enabled: false
        });

        sensitivityButton = new sas.hc.m.MenuButton({
            icon: sas.icons.HC.GOALSEEKING,
            tooltip: "{i18N>modelGroupRuns.sensitivityBtn.tooltip.txt}",
            menu:  new sas.hc.m.Menu({
                items: [
                    new sas.hc.m.MenuItem({
                        text: "{i18N>modelGroupRuns.sensitivityBtn.parallel.tooltip.txt}",
                        press: $.proxy(controller.onSensitivitySelected, controller, "RunAsyncParallel")
                    }),
                    new sas.hc.m.MenuItem({
                        text: "{i18N>modelGroupRuns.sensitivityBtn.serial.tooltip.txt}",
                        press: $.proxy(controller.onSensitivitySelected, controller, "RunAsyncSerial")
                    })
                ]
            }),
            showDownArrowIconOnly: false,
            enabled: false
        });

        deleteButton = new sas.hc.m.Button({
            id: "modelGroupRuns-deleteRunBtn",
            icon: sas.icons.HC.DELETE,
            tooltip: "{i18N>modelGroupRuns.removeBtn.tooltip.txt}",
            press: $.proxy(controller.onDeleteSelected, controller),
            enabled: false
        });

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                _createSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                createRunButton,
                runButton,
                loadButton,
                sensitivityButton,
                deleteButton
            ]
        });
    }

    function _createSearchField(controller) {
        searchField = new sas.hc.m.SearchField({
            id: "modelgroupruns-searchFld",
            liveChange: $.proxy(controller.filterModelGroupRuns, controller),
            search: $.proxy(controller.filterModelGroupRuns, controller),
            filterMode: true
        });
        //searchField.onAfterRendering = function () {
        //    var landmarkText = controller.i18N.getResourceBundle().getText("modelGroupRunsTable.searchField.prompt.txt");
        //    sas.hc.m.SearchField.prototype.onAfterRendering.apply(searchField, arguments);
        //    $("form", searchField.getDomRef()).attr(
        //        {
        //            "data-landmark-label": landmarkText
        //        }
        //    );
        //};
        return searchField;
    }

    function _addLoadOptionIfApplicable(menuText, subMenu) {
		if (menuText === _getResourceBundle().getText('modelGroupRun.outputfile.sas.results.txt') ) {
            // subMenu.addItem(
            //    new sas.hc.ui.unified.MenuItem({
            //       text: _getResourceBundle().getText("modelGroupRunsTable.button.load.global.label"),
            //       select: function (evt) {
            //          var topMenuItem = evt.getParameters().item.getParent().getParent();
            //          var topMenuItemText = topMenuItem.getText();
            //          var data = this.getModel().getData();
            //          _executeLoad(_.findWhere(this.getModel().getData(), {displayName: topMenuItem.getText()}),false);
            //       }
            //    })
            // );
            subMenu.addItem(
                new sas.hc.ui.unified.MenuItem({
                    text: _getResourceBundle().getText("modelGroupRunsTable.button.load.label"),
                    select: function (evt) {
                        var topMenuItem = evt.getParameters().item.getParent().getParent();
                        $.proxy(myController.onExecuteLoad(_.findWhere(this.getModel().getData(), { displayName: topMenuItem.getText() })), myController);
                    },
                    enabled: {
                        path: "/projectIsLocked",
                        formatter: function(isLocked){
                            return !isLocked;
                        }
                    }
                })
            );
        }
    }

    /*
     * Returns a button for displaying and opening multiple results of the same type.
     * @param {Array} results
     * @param iconURL
     * @param {String} tooltipText
     * @param {Boolean} includeOpen - should the open option be included
     * @param {Boolean} projectIsLocked - should the Load option be enabled
     * @returns {sas.hc.m.Button}
     * @private
     */
    function _createMultipleResultsButton(results, iconURL, tooltipText, includeOpen, projectIsLocked) { // S1388487
        var outputFileButton = new sas.hc.m.Button({
            icon: iconURL,
            tooltip: _getResourceBundle().getText(tooltipText),
            enabled: true
        });

        var outputFileMenuTemplate = new sas.hc.ui.unified.MenuItem({
            text: "{displayName}"
        });

        var outputFileMenu = new sas.hc.ui.unified.Menu({
            items: {
                path: "/",
                template: outputFileMenuTemplate
            }
        });

        outputFileMenu.setModel(new sas.risk.common.models.RiskJSONModel(results.files)/*, "results"*/);
        outputFileMenu.getModel().setProperty("/projectIsLocked", projectIsLocked); // S1388487.. setup the property on the model for the menu
        _.each(outputFileMenu.getItems(), function (menuItem) {
            var subMenu = new sas.hc.ui.unified.Menu();
            if (includeOpen) {
                subMenu.addItem(
                    new sas.hc.ui.unified.MenuItem({
                        text: _getResourceBundle().getText("modelGroupRunsTable.button.open.label"),
                        select: function (evt) {
                            var topMenuItem = evt.getParameters().item.getParent().getParent();
                            var model = this.getModel();
                            var data = model.getData();
                            var typeName = topMenuItem.getText();
                            var file = _.findWhere(data, { displayName: typeName });
                            _openFile(file);
                        }
                    })
                );
            }

            subMenu.addItem(
                new sas.hc.ui.unified.MenuItem({
                    text: _getResourceBundle().getText("modelGroupRunsTable.button.download.label"),
                    select: function (evt) {
                        var topMenuItem = evt.getParameters().item.getParent().getParent();
                        _downloadFile(_.findWhere(this.getModel().getData(), { displayName: topMenuItem.getText() }));
                    }
                })
            );

            // Results gets the load options
            var text = menuItem.getText();
            _addLoadOptionIfApplicable(text, subMenu);
            menuItem.setSubmenu(subMenu);
        });

        outputFileButton.attachPress(function () {
            var eDock = sap.ui.core.Popup.Dock;
            outputFileMenu.open(
                false, /*First item already highlighted*/
                null, /*Dom reference which gets the focus back when the menu is closed*/
                eDock.BeginTop, /*"Edge" of the menu (see sap.ui.core.Popup)*/
                eDock.BeginBottom, /*"Edge" of the related opener position (see sap.ui.core.Popup)*/
                outputFileButton.getDomRef()
            );
            /*Related opener position (see sap.ui.core.Popup)*/
        });
        return outputFileButton;
    }

    /*
	 * Returns a button for displaying and opening single results of the same type.
     * @param {Array} results
     * @param iconURL
     * @param {String} tooltipText
     * @param {Boolean} includeOpen
     * @returns {sas.hc.m.Button}
     * @private
     */
    function _createOneFileResultsButton(results, iconURL, tooltipText, includeOpen) {
        var tooltip = "";
        if (tooltipText.length > 0) {
            tooltip = _getResourceBundle().getText(tooltipText);
        }
        var button = new sas.hc.m.Button({
            icon: iconURL,
            tooltip: tooltip,
            enabled: true
        });

        var menu = new sas.hc.ui.unified.Menu({});

        menu.setModel(new sas.risk.common.models.RiskJSONModel(results));
        if (includeOpen) {
            menu.addItem(
                new sas.hc.ui.unified.MenuItem({
                    text: _getResourceBundle().getText("modelGroupRunsTable.button.open.label"),
                    select: function (evt) {
                        var data = this.getModel().getData();
                        var fileToOpen = data.files[0];
                        _openFile(fileToOpen);
                    }
                })
            );
        }

        menu.addItem(
            new sas.hc.ui.unified.MenuItem({
                text: _getResourceBundle().getText("modelGroupRunsTable.button.download.label"),
                select: function (evt) {
                    var data = this.getModel().getData();
                    var fileToDownload = data.files[0];
                    _downloadFile(fileToDownload);
                }
            })
        );

		var resultsFile = results.files[0];
		if (resultsFile) {
			var displayName = resultsFile.displayName;
			if (displayName === _getResourceBundle().getText('modelGroupRun.outputfile.sas.results.txt')) {
				menu.addItem(
					new sas.hc.ui.unified.MenuItem({
						text: _getResourceBundle().getText("modelGroupRunsTable.button.load.label"),
						select: function (evt) {
							$.proxy(myController.onExecuteLoad(resultsFile), myController);
						}
					})
				);
			}
		}

        button.attachPress(function () {
            var eDock = sap.ui.core.Popup.Dock;
            menu.open(
                false, /*First item already highlighted*/
                null, /*Dom reference which gets the focus back when the menu is closed*/
                eDock.BeginTop, /*"Edge" of the menu (see sap.ui.core.Popup)*/
                eDock.BeginBottom, /*"Edge" of the related opener position (see sap.ui.core.Popup)*/
                button.getDomRef()
            );
            /*Related opener position (see sap.ui.core.Popup)*/
        });
        return button;
    }

    function _createResultsButton(results, iconURL, tooltipText, includeOpen,projectIsLocked) { // S1388487
		// Per a discussion with Dan we decided that it was more consistent
		// to show the menus the same way regardless of whether there was only one file
		// or multiple.  This makes the UI consistent.  If UX doesn't like it
		// then these lines should be uncommented.
//		if (results.files && results.files.length > 1) {
            return _createMultipleResultsButton(
                results,
                iconURL,
                tooltipText,
                includeOpen,
                projectIsLocked // S1388487
            );
		// }
		// else {
		// 	return _createOneFileResultsButton(
		// 		results,
		// 		iconURL,
		// 		tooltipText,
		// 		includeOpen
		// 	);
		// }
    }

    function _createResultsContent(id, context) {
        var result = context.getProperty(context.sPath);
        var projectIsLocked=context.getModel().getProperty("/projectIsLocked");// S1388487`
        switch (result.type) {
            case sas.stress.models.ModelGroupRunConstants.FILE_TYPE_SAS_CODE:
                return _createResultsButton(
                    result,
                    sas.icons.HC.CODE,
                    "modelGroupRunsTable.results.button.sascode.tooltip",
                    true,
                    projectIsLocked // S1388487
                );
            case sas.stress.models.ModelGroupRunConstants.FILE_TYPE_SAS_DATA_SET:
                return _createResultsButton(
                    result,
                    sas.icons.HC.SASDATASET,
                    "modelGroupRunsTable.results.button.sasdataset.tooltip",
                    true,
                    projectIsLocked // S1388487
                );
            case sas.stress.models.ModelGroupRunConstants.FILE_TYPE_SAS_LOG:
                // Build the tooltip for any errors we might have
                var tooltipText = "modelGroupRunsTable.results.button.log.tooltip";  // This could be a problem
                if (!_.isUndefined(result.files) && result.files.length > 0 && !_.isUndefined(result.files[0].errors)) {
                    if (result.files[0].errors.length > 0) {
                        tooltipText = _.reduce(result.files[0].errors, function (tooltip, errorText) {
                            return tooltip + "\n" + errorText;
                        });
                    }
                }
                return _createResultsButton(result, sas.icons.HC.LOG, tooltipText, true, projectIsLocked); // S1388487
        }
        // We got a bad type, so return something so the client doesn't fail.
        return new sas.hc.m.Label({ text: "" });
    }

    /*
     * Downloads the file referenced by the specified link. If the file is large (>MAX_FILE_SIZE),
     * the user is presented with a confirmation dialog prior to downloading.
     * @param file
     * @private
     */
    function _downloadFile(file) {
        if (!file) {
            return;
        }
        if (file.size && file.size > sas.stress.models.ModelGroupRunConstants.MAX_FILE_SIZE) {
            sas.stress.models.ModelGroupRunUtils.showConfirmationDialog(
                _getResourceBundle().getText("modelGroupRuns.confirm.download.title.fmt.txt", file.filename),
                _getResourceBundle().getText("modelGroupRuns.confirm.download.fmt.txt", sas.stress.models.ModelGroupRunUtils.convertBytesToFileSizeString(file.size)),
                null,
                function () {
                    sas.stress.models.ModelGroupRunAPI.downloadFile(file);
                }
            );
        } else {
            sas.stress.models.ModelGroupRunAPI.downloadFile(file);
        }
    }

    /*
     * Opens the file referenced in SAS Stuio. If the file is large (>10MB),
     * the user is presented with a warning message and the open is cancelled.
     * @param file
     * @private
     */
    function _openFile(file) {
        if (!file) {
            return;
        }
        if (file.size && file.size > sas.stress.models.ModelGroupRunConstants.MAX_FILE_SIZE) {
            sas.stress.models.ModelGroupRunUtils.showWarningInformationDialog(
                _getResourceBundle().getText("modelGroupRuns.confirm.cannotopen.title.fmt.txt", file.filename),
                _getResourceBundle().getText("modelGroupRuns.confirm.cannotopen.fmt.txt", sas.stress.models.ModelGroupRunUtils.convertBytesToFileSizeString(file.size))
            );
        } else {
            if (file.fullFilePath) {
                sas.stress.wkbench.SASStudioProxy.openFile(file.fullFilePath);
            }
        }
    }

    function _getResultsTemplate(controller) {
        return new sas.hc.ui.layout.HorizontalLayout({
            content: {
                path: "results",
                factory: $.proxy(_createResultsContent, this)
            }
        });
    }

    function _getPertubationType(type){
        switch (type){
           case "ABS":
              return sas.stress.models.ModelGroupRunUtils.getCommonResourceBundle().getText("modelSensitivityAnalysisPanel.shockType.absolute.txt");
              break;
           case "REL":sas.stress.models.ModelGroupRunUtils.getCommonResourceBundle().getText("modelSensitivityAnalysisPanel.shockType.relative.txt");
              break;
        }
    }

    function _getRunStatusTemplate(controller) {
        return new sas.stress.wkbench.widget.StressObjStatus({
            icon: {
                path: "status",
                formatter: function (status) {
                    if (_.isUndefined(status) || _.isNull(status) || status === 0) {
                        return sas.icons.HC.NEUTRALSTATUS;
                    }
                    var status = status.toUpperCase();
                    switch (status) {
                        case "NOTRUN":
                            return sas.icons.HC.NEUTRALSTATUS;
                        case "READYTORUN":
                        case "QUEUEDTORUN":
                            return sas.icons.HC.SCHEDULEDSTATUS;
                        case "COMPLETED":
                            return sas.icons.HC.COMPLETESTATUS;
                        case "ERROR":
                            return sas.icons.HC.ERRORSTATUS;
                        case "RUNNING":
                            return sas.icons.HC.ANIMATEDPROGRESSINDICATOR;
                        case "TERMINATEDBYSHUTDOWN":
                            return sas.icons.HC.TERMINATEPROCESS;
                        default:
                            return sas.icons.HC.NEUTRALSTATUS;
                    }
                }
            },
            text: {
                parts: ["status", "resultsHaveBeenLoaded", "partialDataLoad", "countOfDataValuesLoaded"],
                formatter: function (status, loaded, partial, count) {
                    if (_.isUndefined(status) || _.isNull(status) || status === 0) {
                        return _getResourceBundle().getText("modelGroupRuns.execution.status.NotStarted.txt");
                    }
                    var status = status.toUpperCase();
                    if (status === "COMPLETED" && loaded) {
                        if (count < 1) {
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.noneLoaded.txt");
                        } else if (!!partial){
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.partialLoaded.txt");
                        } else {
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.resultsLoaded.txt");
                        }
                    }
                    switch (status) {
                        case "NOTRUN":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.NotStarted.txt");
                        case "READYTORUN":
                        case "QUEUEDTORUN":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.ReadyToRun.txt");
                        case "COMPLETED":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.Completed.txt");
                        case "ERROR":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.Error.txt");
                        case "RUNNING":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.Executing.txt");
                        case "TERMINATEDBYSHUTDOWN":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.Terminated.txt");
                        default:
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.NotStarted.txt");
                    }
                }
            },
            tooltip: {
                parts: ["status", "resultsHaveBeenLoaded", "partialDataLoad", "countOfDataValuesLoaded"],
                formatter: function (status, loaded, partial, count) {
                    if (_.isUndefined(status) || _.isNull(status) || status === 0) {
                        return _getResourceBundle().getText("modelGroupRuns.execution.status.NotStarted.txt");
                    }
                    var status = status.toUpperCase();
                    if (status === "COMPLETED" && loaded) {
                        var results = _getResourceBundle().getText("modelGroupRun.outputfile.sas.results.txt");
                        var unloaded = _getResourceBundle().getText("modelGroupRun.outputfile.sas.nonloadabledata.txt");
                        if (partial) {
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.partialLoaded.tip.fmt", [count, results, unloaded]);
                        } else {
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.resultsLoaded.tip.fmt", [count, results]);
                        }
                    }
                    switch (status) {
                        case "NOTRUN":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.NotStarted.txt");
                        case "READYTORUN":
                        case "QUEUEDTORUN":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.ReadyToRun.txt");
                        case "COMPLETED":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.Completed.txt");
                        case "ERROR":
                            var tooltipText = _getResourceBundle().getText("modelGroupRuns.execution.status.Error.txt");  // This could be a problem
                            // Check for errors and build a different tooltip if there are any.
                            var context = this.getBindingContext();
                            var errors = context.getObject().errors;
                            if (!_.isUndefined(errors) && errors.length > 0) {
                                tooltipText = _.reduce(errors, function (tooltip, errorText) {
                                    return tooltip + "\n" + errorText;
                                });
                            }
                            return tooltipText;
                        case "RUNNING":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.Executing.txt");
                        case "TERMINATEDBYSHUTDOWN":
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.Terminated.tip");
                        default:
                            return _getResourceBundle().getText("modelGroupRuns.execution.status.NotStarted.txt");
                    }
                }
            }
        }).addStyleClass("stressObjectStatus");
    }

    function _createModelGroupRunTable(controller) {
        runsTable = new sas.hc.ui.table.Table({
            width: "100%",
            editable: false,
            rowHeight: 36,
            showTableOptions: true,
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowSelector,   // S1391565
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            showSelectionControls: true,
            minAutoRowCount: 1,   // S1196469,
            toolbar: _createTableHeader(controller),
            noData: "{i18N>modelGroupRunsTable.noData.loading.txt}",
//          cellClick:$.proxy(controller.onRowSelectionChange,controller),
            afterColumnsDialogClose: $.proxy(controller.filterModelGroupRuns, controller),
            rowSelectionChange: $.proxy(controller.onRowSelectionChange, controller)
//            columnSelect:$.proxy(controller.onColumnSelect,controller)
        });

        // add a new column to be used as default sort S1382451
        var idView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: {
                path: "id",
                formatter:function(id){
                    return ((id !== null) ? id.toString() : "");   // conver to string
                }
            }}).bindProperty("text", "id");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("id"),
            name: "id",
            label: new sas.hc.m.Label({ text: "ID" }),
            template: idView,
            sortProperty: "id",
            width: "3rem",
            visible:false
        }));
        var nameView = new sas.hc.m.Text({ wrapping: false, tooltip: "{name}" }).bindProperty("text", "name");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("name"),
            name: "name",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.name.header.title}" }),
            template: nameView,
            sortProperty: "name",
            width: "15rem"
        }));

        var scenarioNameView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{scenarioName}"
        }).bindProperty("text", "scenarioName");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("scenarioName"),
            name: "scenarioName",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.scenarioName.header.title}" }),
            template: scenarioNameView,
            sortProperty: "scenarioName"
            //width: "10%"
        }));

        var entityNameView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{entityName}"
        }).bindProperty("text", "entityName");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("entityName"),
            name: "entityName",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.entityName.header.title}" }),
            template: entityNameView,
            sortProperty: "entityName"
            //width: "10%"
        }));

        var descriptionView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{description}"
        }).bindProperty("text", "description");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("description"),
            name: "description",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.description.header.title}" }),
            template: descriptionView,
            sortProperty: "description",
            //width: "15%",
            visible: false
        }));

        var modelGroupNameView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{modelGroupName}"
        }).bindProperty("text", "modelGroupName");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("modelGroupName"),
            name: "modelGroupName",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.modelGroupName.header.title}" }),
            template: modelGroupNameView,
            sortProperty: "modelGroupName"
            //width: "8%"
        }));
        var modelGroupNameVersionView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{modelGroupVersion}"
        }).bindProperty("text", "modelGroupVersion");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("modelGroupVersion"),
            name: "modelGroupVersion",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.version.header.title}" }),
            template: modelGroupNameVersionView,
            sortProperty: "modelGroupVersion",
            //width: "5%"
        }));

        var shockButton = new sas.hc.m.Button({
            icon: sas.icons.HC.OBJECTINSPECTOR,
            tooltip: _getResourceBundle().getText("modelGroupRunsTable.column.shock.tooltip.txt"),
            layoutData:new sas.hc.m.FlexItemData({ shrinkFactor: 0 }),
            visible:{
                parts:["shocks"],
                formatter:function(shocks){
                    if (!_.isNull(shocks) && !_.isEmpty(shocks)) {
                        return true;
                    }
                    return false;
                }
            },
            press: function(oEvent) {
                var model = runsTable.getModel();
                var context = model.getContext(oEvent.oSource.getBindingContext().getPath());
                var objectInspector = new sas.hc.m.objectInspector.ObjectInspector({
                    showOpenButton: false,
                    showEditButton: false,
                    showFavoritesButton:false,
                    openButtonPress: function() {
                        objectInspector.close();
                    }
                });

                var modelGroupRunRow = model.getProperty(context.getPath());
                if (modelGroupRunRow.shocks && modelGroupRunRow.shocks.length>0)
                {
                    var data = modelGroupRunRow.shocks[0];
                    //convert variables to JSON array
                    var bundle = sas.stress.models.ModelGroupRunUtils.getCommonResourceBundle();
                    var objectData=[
                        {
                            name:bundle.getText("modelSensitivityAnalysisPanel.shocks.table.scenarioName.column.txt"),
                            value: data.shockName
                        },
                        {
                            name: bundle.getText("modelSensitivityAnalysisPanel.shocks.table.sourceName.column.txt"),
                            value: data.sourceName
                        },
                        {
                            name: bundle.getText("modelSensitivityAnalysisPanel.shocks.table.shockType.column.txt"),
                            value: _getPertubationType(data.perturbType)
                        },
                        {
                            name: bundle.getText("modelSensitivityAnalysisPanel.shocks.table.shockValue.column.txt"),
                            value: data.shockValue
                        },
                        {
                            name: bundle.getText("modelSensitivityAnalysisPanel.shocks.table.floor.column.txt"),
                            value: data.floor
                        },
                        {
                            name: bundle.getText("modelSensitivityAnalysisPanel.cap.label.txt"),
                            value: data.cap
                        },
                        {
                            name: bundle.getText("modelSensitivityAnalysisPanel.shocks.table.fadeIn.column.txt"),
                            value: data.fadeIn
                        },
                        {
                            name: bundle.getText("modelSensitivityAnalysisPanel.shocks.table.fadeOut.column.txt"),
                            value: data.fadeOut
                        },


                    ];
                    objectInspector.setObjectData(objectData);
                }

                objectInspector.openBy(this);
            }});

        var shockText = new sas.hc.m.Text({wrapping:false}).bindProperty("text", {
            parts:["shocks"],
            layoutData: new sas.hc.m.FlexItemData({ growFactor: 1, minWidth: "0" }),
            formatter:function(shocks){
                if (!_.isNull(shocks) && !_.isEmpty(shocks)) {
                    return shocks[0].shockName;
                }
                return "";
            }});

        var shockTemplate = new sas.hc.m.FlexBox({
            alignItems: sap.m.FlexAlignItems.Center,
            renderType: sap.m.FlexRendertype.Bare,
            items:[shockButton,shockText]
        });

        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("modelGroupShock"),
            name: "modelGroupShock",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.shock.header.title}" }),
            template: shockTemplate,
            sortProperty: "shocks/0/shockName"
        }));

        var workGroupView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{workGroup}"
        }).bindProperty("text", "workGroup");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("workGroup"),
            name: "workGroup",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.workGroup.header.title}" }),
            template: workGroupView,
            sortProperty: "workGroup"
            //width: "10%"
        }));

        var runAsOfDateView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: {
                path: "asOfDate",
                formatter: sas.stress.models.ModelGroupRunUtils.dateFormatter
            }
        });
        runAsOfDateView.bindProperty("text", {
            path: "asOfDate",
            formatter: sas.stress.models.ModelGroupRunUtils.dateFormatter
        });
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("asOfDate"),
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.asOfDate.header.title}" }),
            template: runAsOfDateView,
            sortProperty: "asOfDate",
            //width: "8%",
            visible: true
        }));

        var executedByView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{runByUserName}"
        }).bindProperty("text", "runByUserName");
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("runByUserName"),
            name: "runByUserName",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.executedBy.header.title}" }),
            template: executedByView,
            sortProperty: "runByUserName",
            //width: "15%",
            visible: false
        }));

        var completedOnView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: {
                path: "ended",
                formatter: sas.stress.models.ModelGroupRunUtils.dateTimeFormatter
            }
        });
        completedOnView.bindProperty("text", {
            path: "ended",
            formatter: sas.stress.models.ModelGroupRunUtils.dateTimeFormatter
        });
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("ended"),
            name: "ended",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.completedOn.header.title}" }),
            template: completedOnView,
            sortProperty: "ended"
            //width: "8%"
        }));

        var statusColumn = new sas.hc.ui.table.Column({
            id:self.createId("status"),
            name: "status",
            //width: "9%",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.status.header.title}" }),
            sortProperty: "status",
            template: _getRunStatusTemplate(controller)
        });
        runsTable.addColumn(statusColumn);

        var resultsColumn = new sas.hc.ui.table.Column('mgrResultsColumn', {
            id:self.createId("results"),
            name: "results",
            //width: "11%",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.results.header.title}" }),
            showSortMenuEntry: false,
            showFilterMenuEntry: false,
            template: _getResultsTemplate(controller)
        });
        runsTable.addColumn(resultsColumn);

        var runTypeFormatter = function (value) {
            if (_.isUndefined(value) || _.isNull(value)) {
                return "";
            }
            var capitalizedValue = value.toUpperCase();
            switch (capitalizedValue) {
                case "STANDARD":
                    return sas.stress.models.ModelGroupRunUtils.getResourceBundle().getText("modelGroupRuns.wizard.step1.runtype.standard.txt");
                case "SENSITIVITY":
                    return sas.stress.models.ModelGroupRunUtils.getResourceBundle().getText("modelGroupRuns.wizard.step1.runtype.sensitivity.txt");
                case "ATTRIBUTION":
                    return sas.stress.models.ModelGroupRunUtils.getResourceBundle().getText("modelGroupRuns.wizard.step1.runtype.attribution.txt");
            }
            return "Unknown";
        };

        var runTypeView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: {
                path: "runType",
                formatter: runTypeFormatter
            },
            text: {
                path: "runType",
                formatter: runTypeFormatter
            }
        });

        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("runType"),
            name: "runType",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.runType.header.title}" }),
            template: runTypeView,
            sortProperty: "runType",
            //width: "15%",
            visible: false
        }));

        var ranWithDebugView = new sas.hc.ui.core.Icon({
            src: {
                path: "debug",
                formatter: function (bValue) {
                    return bValue ? sas.icons.HC.COMMITCHANGES : sas.icons.HC.BLANK;
                }
            }
        });
        runsTable.addColumn(new sas.hc.ui.table.Column({
            id:self.createId("debug"),
            name: "debug",
            label: new sas.hc.m.Label({ text: "{i18N>modelGroupRunsTable.column.ranwithdebug.header.title}" }),
            template: ranWithDebugView,
            sortProperty: "debug",
            //width: "10%",
            visible: false
        }));

        runsTable.bindRows("/modelGroupRuns");

        //S1256573: Default sort by Name asc.
        var bindingInfo = runsTable.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [new sap.ui.model.Sorter("name", false)];   // default sort on ID S1382451
            bindingInfo.sorter = sorters;
        }
        return runsTable;
    }

    return {
        getControllerName: function () {
            return "sas.stress.models.view.execution.ModelGroupRunsManager";
        },

        createContent: function (controller) {

            self = this;
            myController = controller;
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            this.setHeight("100%");

            var table = _createModelGroupRunTable(controller);
            this.runsTable=table;
            this.breadCrumb = new sas.stress.wkbench.widget.CustomBreadcrumb({
                customData: new sap.ui.core.CustomData({key: "size", value: "3rem"})
            }).addStyleClass("innerBreadCrumb");

            this.breadCrumb.addBreadcrumbItem("Top", sas.stress.models.ModelGroupRunUtils.getResourceBundle().getText("modelGroupRuns.breadcrumb.label"));

            this.viewLayout = new sas.risk.common.widgets.RiskPage({
                mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel(
                    {
                        type: sas.risk.common.widgets.WidgetFactory.OBJECT,
                        top: new sas.risk.common.widgets.RiskToolbar({
                            homeFunction: function () {
                                sas.stress.wkbench.ViewManager.loadProjectManager();
                            },
                            headerContent: [
                                new sas.hc.m.Label({text: rb.getText("iteration.label.txt")}).addStyleClass("toolbarLabel"),
                                new sas.hc.m.Label({text: "{/currentIterationNo}"}).addStyleClass("toolbarValue"),
                                new sas.hc.m.Label({text: rb.getText("workbench.toolbar.asOfDate.label")}).addStyleClass("toolbarLabel"),
                                new sas.hc.m.Label({
                                    text: {
                                        path: "/asOfDate",
                                        formatter: function (asOfDate) {
                                            var dateFormat = new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}}, {target: {pattern: "MMM dd, yyyy"}}, asOfDate);
                                            return dateFormat.formatValue(asOfDate, "string");
                                        }
                                    }
                                }).addStyleClass("toolbarValue"),
                                sas.stress.notification.NotificationUtil.createNotificationButton()
                            ],
                            title: new sas.hc.m.Label({text: "{/projectName}"})
                        }),
                        content: new sas.risk.common.widgets.RiskPage({
                            mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel({
                                top: this.breadCrumb,
                                content: table
                            }),
                            iconBar: new sas.stress.wkbench.widget.StressIconBar({
                                section: sas.stress.wkbench.widget.StressIconBar.MODELGROUPS
                            })
                        })
                    }
                )
            });

            return this.viewLayout;
        },

        destroy: function () {
            sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
        },

        getSelectedRuns: function () {
            var model = this.getModel();
            var indices = runsTable.getSelectedIndices();
            var items = [];
            _.each(indices, function (selectedIdx) {
                var context = runsTable.getContextByIndex(selectedIdx);
                if (!_.isUndefined(context)) {
                    var path = context.sPath;
                    var item = runsTable.getModel().getProperty(path);
                    items.push(item);
                }
            });

            return items;
        },

        setTableBusy: function (isBusy) {
            runsTable.setBusy(isBusy);
        },

        getTable: function () {
            return runsTable;
        },

        getSearchField: function () {
            return searchField;
        },

        fireSelectionEvent: function () {
            this.fireEvent(this.getId() + "_SELECTION_CHANGED", { items: this.getSelectedRuns() });
        },

        getRunButton: function () {
            return runButton;
        },

        getLoadButton: function () {
            return loadButton;
        },

        getDeleteButton: function () {
            return deleteButton;
        },

        getSensitivityButton: function() {
            return sensitivityButton;
        }

    };

}()));