jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.TextArea");
jQuery.sap.require("sas.hc.ui.layout.form.Form");
jQuery.sap.require("sas.hc.ui.layout.form.ResponsiveGridLayout");
jQuery.sap.require("sas.hc.ui.layout.form.FormContainer");
jQuery.sap.require("sas.hc.ui.layout.form.FormElement");
jQuery.sap.require("sas.hc.ui.table.SelectionBehavior");
jQuery.sap.require("sas.hc.ui.table.SelectionMode");
jQuery.sap.require("sas.risk.common.widgets.SelectTable");
jQuery.sap.require("sas.stress.reports.StressAPI");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.widget.ValidFiltersWindowShade");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.jsfragment("sas.stress.reports.view.RunReportDialog", {
    createContent: function (controller) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");

        var commentLabel = new sas.hc.m.Label({
            text: rb.getText("runReportDialog.comment.control.label")
        });
        var commentInput = new sas.hc.m.TextArea({
            id: controller.getView().createId("runReportCommentInput"),
            maxLength: 256
        });
        commentInput.addAriaLabelledBy(commentLabel);

        controller.validFiltersWindowShade = new sas.stress.wkbench.widget.ValidFiltersWindowShade();
        controller.validFiltersWindowShade.setWidth('100%');

        var validFiltersLabel = new sas.hc.m.Label({
            text: rb.getText("runReportDialog.filters.title.txt"),
            //visible: "{/reportHasFilters}", //(false === controller.reportHasNoFilters)
            required: true
        });
        var scrollContainer = new sas.hc.m.ScrollContainer({
            content: controller.validFiltersWindowShade,
            height: "20rem",   //S1345308: add a fixed height to prevent dialog resize when expand/collapse filters
            //width: "100%",
            horizontal: false,
            vertical: true
        });
        //scrollContainer.addStyleClass("stressFilterContainer");    //S1345308: remove max-height

        controller.landingAreaDataList = this.createDataSelectList("/landingAreaFiles", "timestampString", controller);
        controller.landingAreaDataList.attachSelectionChange(controller.landingAreaDataListChange, controller);

        var fileSelectLabel = new sas.hc.m.Label({
            text: rb.getText("runReportDialog.landingArea.title.txt")
         });

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsL: 1,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: validFiltersLabel,
                            fields: [scrollContainer],
                            visible:"{/reportHasFilters}"
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: fileSelectLabel,
                            fields: [controller.landingAreaDataList],
                            visible: "{/reportHasLandingAreaFile}"
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: commentLabel,
                            fields: [commentInput] ,
                            visible: "{/reportHasComments}"
                        })
                    ]
                })
            ]
        });
        form.addStyleClass("stressFormLarge");

        controller.runReportButton = new sas.risk.common.widgets.Button({  // to prevent double clicking
            id: controller.getView().createId("runReportButton"),
            text: rb.getText("button.runReport.action.label"),
            press: $.proxy(controller.runReportDialogRunButtonPress, controller)
        });

        // controller.runAndExportReportButton = new sas.hc.m.Button({
        //     id: controller.getView().createId("runAndExportReportButton"),
        //     text: rb.getText("button.runAndExportReport.action.label"),
        //     press: $.proxy(controller.runReportDialogRunAndExportButtonPress, controller)
        // });

        var dialog = new sas.hc.m.Dialog({
            id: controller.getView().createId("sas.stress.reports.view.RunReportDialog"),
            initialFocus: commentInput,
            content: form,
            //contentHeight: "75%",    // need to fit both filter scrollContainer and landingArea selectTable
            buttons: [
                controller.runReportButton,
                // controller.runAndExportReportButton,
                new sas.hc.m.Button({
                    text: rb.getText("button.cancel.action.label"),
                    press: function () {
                        dialog.close();
                    }
                })
            ],
            afterClose: function () {
                dialog.destroy();
            }
        });

        if (controller.reportHasFilters && !controller.reportHasLandingAreaFile) {
            this._createFilters(controller, controller.validFiltersWindowShade, dialog);

            controller.validFiltersWindowShade.addEventDelegate({
                onAfterRendering: function () {
                    //var capabilityManager=sas.stress.wkbench.CapabilityManager.getInstance();
                    //// If all values have default/preset values, then we need to ensure the run buttons are enabled immediately.
                    //var allFiltersSet = !controller.validFiltersWindowShade.hasUnsetFilters();
                    //controller.runReportButton.setEnabled(allFiltersSet);
                    //controller.runAndExportReportButton.setEnabled(allFiltersSet &&
                    //    capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.EXPORT_TO_RDM)); //defect S1294748)
                    //
                    controller.enableRunButtons(true);
                    dialog.setBusy(false);
                }
            }, controller);
        } else {
            controller.enableRunButtons(true);
        }

        return dialog;
    },

    createDataSelectList: function (itemsPath, timePath) {
        var singleSelectTable = new sas.risk.common.widgets.SelectTable({
            listItems: "{" + itemsPath + "}",
            listName: "{i18N>runReportDialog.fileList.name.txt}",
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            //selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowSelector,
            visibleRowCount: 10,
            width:"100%"
        });

        singleSelectTable.addColumn(new sas.hc.ui.table.Column({
            template: new sas.hc.m.Text({
                text: {
                    path: timePath,
                    formatter: function (timestamp) {
                        if (!timestamp) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(timestamp)).toString();
                    }
                },
                tooltip: {
                    path: timePath,
                    formatter: function (timestamp) {
                        if (!timestamp) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(timestamp)).toString();
                    }
                },
                wrapping: false
                //maxLines: 1
            }),
            width: "35%"
        }));
        singleSelectTable.bindRows(itemsPath);
        return singleSelectTable;
    },

    _createFilters: function (controller, oWindowShade, dialog) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        sas.stress.reports.StressAPI.listValidFilters(controller.project.getId()).done(function (validFilters) {
            if (validFilters.items.length > 0) {
                var preselectedFilters = undefined;
                if (controller.runReportParams && controller.runReportParams.filters) {
                    preselectedFilters = controller.runReportParams.filters;
                }
                for (var index = 0; index < validFilters.items.length; ++index) {
                    oWindowShade.createSectionForHierarchy(validFilters.items[index], controller.runReportDialogFilterSelectHandler, controller, preselectedFilters, dialog);
                }

                // Are any filters left? If so, it means they were not found and are therefore invalid in the current setup.
                if (preselectedFilters && preselectedFilters.length && preselectedFilters.length > 0) {
                    sas.hc.m.MessageDialog.error({text: rb.getText("reports.processOrchestration.invalidFilter.fmt.txt", [preselectedFilters[0]])});
                }

                delete controller.runReportParams;
            } else {
                controller.reportHasFilters = false;
                dialog.getModel().setProperty("/reportHasFilters", false);     // hide filter section
            }
            controller.enableRunButtons(true);
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            controller.enableRunButtons(false);
        });
    }
});
