jQuery.sap.declare("sas.stress.scripts.ScriptsAPI");

/**
 * Scripts
 */
sas.stress.scripts.ScriptsAPI = {
    loadScripts:function(){
        var deferred= $.Deferred();
        $.getJSON("rest/scripts", function (scriptsJSON) {
            deferred.resolve(scriptsJSON.items);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading SCRIPTS FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    loadScriptsHeader:function(){
        var deferred= $.Deferred();
        $.getJSON("rest/scripts/thin", function (scriptsJSON) {
            deferred.resolve(scriptsJSON.items);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading SCRIPTS HEADER FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },


    loadScript:function(scriptId){
        var deferred= $.Deferred();
        $.getJSON("rest/scripts/"+scriptId, function (scriptJSON) {
            deferred.resolve(scriptJSON);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading SCRIPTS FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    createScript:function(requestPayLoad){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/scripts",
            data: JSON.stringify(requestPayLoad),
            dataType: "json"
        }).done(function (script) {
            deferred.resolve(script);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    editScript: function (scriptKey, requestPayLoad) {
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "PUT",
            url: "rest/scripts/" + scriptKey,
            data: JSON.stringify(requestPayLoad),
            dataType: "json"
        }).done(function (group) {
            deferred.resolve(group);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteScripts: function (scriptIdList) {
        var deferred = $.Deferred();
        if (scriptIdList.length === 1){ // S1398605
            $.ajax(
                {
                    type: "DELETE",
                    url: "rest/scripts/" + scriptIdList[0]
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }else {
            $.ajax({
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "DELETE",
                url: "rest/scripts",
                data: JSON.stringify(scriptIdList),
                dataType: "json"
            }).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }
        return deferred;
    },

    getScriptExecutionLog:function(scriptId,runKey){
        var deferred= $.Deferred();
        $.getJSON("rest/scripts/"+scriptId+"/run/"+runKey+"/log", function (log) {
            deferred.resolve(log);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading SCRIPT EXECUTION LOG FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    downloadFile: function (file) {
        var url = "rest/modelGroups/runs/" + file.runId + "/results/" + file.filename;
        window.open(url, "_self");
    },

    executeParameterSASCode:function(code){
        //var deferred= $.Deferred();

        //return deferred;
    },

    loadScriptDetailsByID:function(scriptId,projectId){
        var deferred= $.Deferred();
        var URL;
        if (projectId){
            URL="rest/config/scripts/"+scriptId+"/scriptDetail/"+projectId;
        }else{
            URL="rest/config/parameterSet/scriptDetail/"+scriptId;
        }
        $.getJSON(URL, function(parameters){
            deferred.resolve(parameters);
        }).fail(function(jqXHR, textStatus, errorThrown){
            jQuery.sap.log.error("loading SCRIPT PARAMETER SET FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    }
};