jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.stress.models.Model");
jQuery.sap.require("sas.stress.scripts.model.Script");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.widget.AccessControlsDialog");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");
jQuery.sap.require("sas.stress.models.model.ModelGroup");

sap.ui.controller("sas.stress.scripts.view.ScriptManager", {

    onInit: function (event) {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("Scripts", this._init, this);
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());

        new sas.risk.common.utils.LayoutService(this.getView().scriptTable, "ScriptsTable");

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.SCRIPTS);
    },

    _init:function(){
        this.updateModel();
    },

    updateModel: function() {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.scripts");
        view.scriptTable.setNoData(rb.getText("script.table.noData.loading.txt"));
        view.scriptTable.setBusyIndicatorDelay(0);
        view.scriptTable.setBusy(true);

        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var controller = this;

        this.i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.scripts.messagebundle" });
        view.setModel(this.i18N, "i18N");
        if (view.getModel() instanceof sap.ui.model.json.JSONModel) {
            var modelExists = true;
        } else {
            var modelGroupsModel = new sas.risk.common.models.RiskJSONModel();
            view.setModel(modelGroupsModel);
            //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
            //var sorters = [new sap.ui.model.Sorter("name", false), new sap.ui.model.Sorter("version", false)];
            //view.scriptTable.getBinding("rows").sort(sorters);
        }

        var model = view.getModel();
        model.setProperty("/scripts", []);
        // we would need to load the model groups too.. projects are loaded because that's the landing page for any user
        $.when(sas.stress.scripts.model.Script.getScriptsHeader(true)/*,
            sas.stress.models.model.ModelGroup.getModelGroups(true)*/).done(function(scripts){    // fixes for S1300562
            model.setProperty("/scripts", scripts);
            if (modelExists !== true) {
                //Default sort by group Name
                //view.scriptTable.sort(view.scriptTable.getColumns()[0], sas.hc.ui.table.SortOrder.Ascending);
            }
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.scriptTable.setBusy(false);
            view.scriptTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                text: "{i18N>script.table.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>script.create.header.title}",
                        press: $.proxy(controller.onCreateClick, controller),
                        enabled:capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.CREATE)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        });
    },

    onCreateClick: function () {
        sas.stress.wkbench.ViewManager.loadScriptEditor(0);
    },

    onRemoveButtonClick: function () {
        var view = this.getView();
        var table = view.scriptTable;
        var selectedIndices = table.getSelectedIndices();
        var idList=[];
        var nameList = [];
        _.each(selectedIndices, function (selectedIndex) {
            var ctx = table.getContextByIndex(selectedIndex);
            var script = ctx.getObject();
            idList.push(script.getId());
            nameList.push(script.nameVersion);
        });
        var rb = this.i18N.getResourceBundle();
        var groupDeleteMsg = sas.stress.wkbench.StressUtil.getConfirmationMessage(rb.getText("script.delete.confirmation.txt"), nameList);

        sas.hc.m.MessageDialog.warning({
            text: groupDeleteMsg,
            title: rb.getText("script.delete.window.title"),
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getMessageDialogAction();
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                    //Delete all the items in the selected list
                    sas.stress.scripts.model.Script.deleteScripts(idList).done(function () {
                        // or should i just refresh? What if other users have deleted the groups?
                        for (var a = 0; a < idList.length; a++) {
                            var model = view.getModel();
                            var scriptsData = model.getData()["scripts"];
                            for (var i = scriptsData.length - 1; i >= 0; i--) {
                                if (scriptsData[i].id === idList[a]) {
                                    scriptsData.splice(i, 1);
                                    break;
                                }
                            }
                        }
                        model.refresh();
                        table.clearSelection();
                    }).fail(function (jqxhr) {
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                }
            }
        });

    },

    _getModelGroup: function (index) {
        var view = this.getView();
        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            index = view.scriptTable.getSelectedIndex();
        }
        var modelGroup = null;
        if (index >= 0) {
            var context = view.scriptTable.getContextByIndex(index);
            var modelGroupNameVersion = context.getObject().nameVersion;
            modelGroup = _.find(view.getModel().getData()["scripts"], function (group) {
                if (group.nameVersion === modelGroupNameVersion) {
                    return group;
                }
            });
        }
        return modelGroup;
    },

    onEditClick: function () {

        var view = this.getView();
        var table = view.scriptTable;
        var selectedIndex = table.getSelectedIndex();

        var ctx = table.getContextByIndex(selectedIndex);
        var script = ctx.getObject();
        var scriptId = script.id;

        sas.stress.scripts.model.Script.loadScriptById(scriptId).done(function() {
            view.setBusy(false);
            sas.stress.wkbench.ViewManager.loadScriptEditor(scriptId);
        }).fail(function(jqxhr){
            view.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

       // var script = this._getModelGroup();
       // sas.stress.wkbench.ViewManager.loadScriptEditor(script.getId(), true);
    },

    //modelGroupAccessControlsClick: function (event) {
    //    // get the model group that is currently selected
    //    var modelGroup = this._getModelGroup();
    //    if (null !== modelGroup) {
    //        this.accessControlsDialog = new sas.stress.wkbench.widget.AccessControlsDialog({});
    //        var controller = this;
    //        var view = this.getView();
    //        view.scriptTable.setBusy(true);
    //        sas.stress.wkbench.StressAPI.getModelGroupAccessControls(modelGroup.id).done(function (accessControls) {
    //            view.scriptTable.setBusy(false);
    //            controller.accessControlsDialog.setup(modelGroup.name, accessControls, $.proxy(controller.modelGroupAccessControlsSaveButtonPress, controller));
    //        }).fail(function (jqxhr) {
    //            view.scriptTable.setBusy(false);
    //            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
    //        });
    //    }
    //},
    //
    //modelGroupAccessControlsSaveButtonPress: function (workgroups) {
    //    var controller = this;
    //    // get the model group that is currently selected
    //    var modelGroup = this._getModelGroup();
    //    if (null !== modelGroup) {
    //        sas.stress.wkbench.StressAPI.saveModelGroupAccessControls(modelGroup.id, workgroups).done(function () {
    //            controller.accessControlsDialog.setBusy(false);
    //            controller.accessControlsDialog.close();
    //        }).fail(function (jqxhr) {
    //            controller.accessControlsDialog.setBusy(false);
    //            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
    //        });
    //    }
    //},
    //
    filterModels: function () {
        var search = this.getView().searchField.getValue() || "";
        var table = this.getView().scriptTable;

        // fixes for S1250390
        var filter0 = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, search);
        var filter1 = new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, search);
        var filter2 = new sap.ui.model.Filter("modelInterval", sap.ui.model.FilterOperator.Contains, search);
        var filter3 = new sap.ui.model.Filter("createdByUserId", sap.ui.model.FilterOperator.Contains, search);
        var filter4 = new sap.ui.model.Filter("updatedByUserId", sap.ui.model.FilterOperator.Contains, search);
        var filter5 = new sap.ui.model.Filter("version", sap.ui.model.FilterOperator.Contains, search);
        var allFilters = new sap.ui.model.Filter([filter0, filter1, filter2, filter3, filter4, filter5]);

        //filters.push(new sap.ui.model.Filter("name",sap.ui.model.FilterOperator.Contains,search));
        var orFilters = new sap.ui.model.Filter(allFilters); // the filters on the visible columns OR'd
        var binding = table.getBinding("rows");

        binding.filter(orFilters);
    },
    onModelDeleted: function () {

    },
    onExit: function () {
        this.router.detachExactRouteMatched("Scripts", this._init, this);
        this.router.setView("sas.stress.scripts.view.ScriptManager", null);
        this.navChecker.stop();
    },

    _onCellClick: function (e) {
        var view = this.getView();
        view.setBusy(true);
        var ctx = e.getSource().getContextByIndex(e.getParameter("rowIndex"));
        var scriptId = ctx.getObject().id;

        sas.stress.scripts.model.Script.loadScriptById(scriptId).done(function() {
            view.setBusy(false);
            sas.stress.wkbench.ViewManager.loadScriptEditor(scriptId);
        }).fail(function(jqxhr){
            view.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

    },

    onRowSelectionChange: function (oEvent) {
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        switch (oEvent.getSource().getSelectedIndices().length) {
            case 0: // nothing selected
                view.deleteModelGroupButton.setEnabled(false);
                view.editModelGroupButton.setEnabled(false);
                //view.modelGroupAccessControlsButton.setEnabled(false);
                break;
            case 1: // one item selected
                view.deleteModelGroupButton.setEnabled(capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.DELETE));
                view.editModelGroupButton.setEnabled(capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.UPDATE)); // TODO: Srini: there should be another flag coming in soon
                /*view.modelGroupAccessControlsButton.setEnabled(true);*/
                break;
            default: // more than one selected
                view.deleteModelGroupButton.setEnabled(capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.DELETE));
                view.editModelGroupButton.setEnabled(false);
                /*view.modelGroupAccessControlsButton.setEnabled(false);*/
                break;
        }
    }
});