jQuery.sap.declare("sas.stress.wkbench.model.Formatter");
jQuery.sap.require("sap.ui.core.format.NumberFormat");

sas.stress.wkbench.model.Formatter = function (type, precision, scale) {
    this.type = type;
    if (precision!==undefined && 0 <= precision) {
        this.precision = precision;
    } else {
        // Default precision to 2 if unset.
        this.precision = 2;
    }
    if (scale && 0 < scale) {
        this.scale = scale;
    } else {
        // Default scale to 1 if unset.
        this.scale = 1;
    }
    // Get the current browser locale, we will use it to ensure numbers are properly formatted.
    var locale = sap.ui.getCore().getConfiguration().getLocale();
    if ("Percent" === type) {
        this.formatter = sap.ui.core.format.NumberFormat.getPercentInstance({decimals:this.precision}, locale);
    } else {
        // It must be numeric.
        this.formatter = sap.ui.core.format.NumberFormat.getCurrencyInstance({decimals:this.precision}, locale);
    }
};
sas.stress.wkbench.model.Formatter.prototype = {
    format: function (value) {
        value = this._pruneEquals(value);
        if (/*_.isNumber(value)*/isFinite(value) && !_.isNull(value) && !_.isNaN(value)) {
            var theformattednumber=value;
            if (this.scale>1) {  // only divide if the integer is >= the scale
                theformattednumber/=this.scale;
            }
            // Format is based on the format options settings (or locale defaults)
            value = this.formatter.format(theformattednumber);
        }
        return value;
    },
    _pruneEquals: function (value) {
        if (null !== value) {
            if (_.isBoolean(value)) {
                return value ? "TRUE" : "FALSE";
            }
            // issue with numbers as strings when displayed... S1271591
            if (_.isNumber(value)/*$.isNumeric(value)*/ || _.isNaN(value)) {
                return Number(value);
            } else {
                if (0 === value.indexOf("=") && $.isNumeric(value.substring(1))) { //Formula after "=" is numeric?
                    return Number(value.substring(1)); //Remove "=" prefix
                }
            }
        }
        return value || "";
    }
};

sas.stress.wkbench.model.Formatter.prototype.getFormatType = function () {
    return this.type;
};

sas.stress.wkbench.model.Formatter.prototype.getFormatPrecision = function () {
    return this.precision;
};

sas.stress.wkbench.model.Formatter.prototype.getFormatScale = function () {
    return this.scale;
};

sas.stress.wkbench.model.Formatter.prototype.isDefaultFormat = function () {
    return this.scale === 1 && this.precision === 2 && this.type.toLowerCase() === "numeric";
};

