jQuery.sap.require("sas.stress.wkbench.CellPickerUtil");
jQuery.sap.require("sas.stress.wkbench.CellEvaluationUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.Allocations");
jQuery.sap.require("sas.stress.reports.StressAPI");
jQuery.sap.require("sas.stress.wkbench.widget.Slimsheet");

sap.ui.controller("sas.stress.wkbench.view.CellEditor",{
	onInit: function(){
		var copyCellsModel = new sap.ui.model.json.JSONModel({
			copyCells: null,
			isRangeCopy: false
		});
		this.getView().setModel(copyCellsModel);
	},
	gainedFocus:function(){
		var view=this.getView();
		var liveModel=view.getModel("live");
		var selected=liveModel.getProperty("/selected");
		liveModel.setProperty("/editing",selected);

		this.checkEditMode();
	},
	formulaLiveChange:function(e){
		var length = e.getParameter("length")?e.getParameter("length"):0;	// RQSSTR-796
		var view=this.getView();
		var value=view.formulaField.getValue();
		var liveModel=view.getModel("live");
		liveModel.setProperty("/cellValueUpdated", true);
		// fixes for S1203451.. update the field with the uppercase always and use the same value
		//value=value.toLocaleUpperCase();
		//this.getView().formulaField.setValue(value);
		var inplaceEdit=this._getInPlaceEditControl();
		if(inplaceEdit&&inplaceEdit.style.visibility==="visible"){
			var caretPos=sas.stress.wkbench.StressUtil.getCaretPosition(inplaceEdit.id);
			inplaceEdit.value=value;	// RQSSTR-796.. set up the value and then setup the caret position?
			sas.stress.wkbench.StressUtil.setCaretPosition(inplaceEdit.id,caretPos+length);	// RQSSTR-796
		}
		if(this.checkEditMode()){
			this.highlightFormulaRefs(value);
		}
	},
	checkEditMode:function(){
		switch(sas.stress.wkbench.CellPickerUtil.getOperationMode()){
			case sas.stress.wkbench.CellPickerUtil.EDIT_MODE:
				//this._highlightCell();
				return true;
			default:
				this._highlightCell();
				if(this.getView().formulaField.getValue().indexOf("=")===0){
					this.enterEditMode();
					return true;
				}else{	// RQSSTR-796
					var inplaceEdit=this._getInPlaceEditControl();
					var liveModel = sas.stress.wkbench.ViewManager.getCellEditor().getModel("live");
					var cellValueUpdated =liveModel.getProperty("/cellValueUpdated");
					if((inplaceEdit&&inplaceEdit.style.visibility==="visible") && cellValueUpdated) {
						// RQSSTR-796 activate the inplace mode only if there is change done and inPlace visible
						sas.stress.wkbench.CellPickerUtil.activateInPlaceVisibleMode();
					}
				}
				break;
		}
		return false;
	},
	enterEditMode:function(){
		sas.stress.wkbench.CellPickerUtil.activateEditMode();
		//this.highlightFormulaRefs(this.getView().formulaField.getValue());
	},
	leaveEditMode:function(){
		if (!!!this.destroyed){
			sas.stress.wkbench.CellPickerUtil.deactivateEditMode();	// first deactivate
			this.clearFormulaRefs();

			var view=this.getView();
			var liveModel=view.getModel("live");
			liveModel.setProperty("/cellValueUpdated", false);
			var slimsheet=liveModel.getProperty("/slimsheet");
			var inPlaceEdit=this._getInPlaceEditControl();
			if(inPlaceEdit&&inPlaceEdit.style.visibility==="visible"){
				setTimeout(function(){
					// RQSSTR-796
					//sas.stress.wkbench.widget.Slimsheet.dimInPlaceEdit(slimsheet);
					if(inPlaceEdit&&(sas.stress.wkbench.CellPickerUtil.getOperationMode() !== sas.stress.wkbench.CellPickerUtil.EDIT_MODE)){
						inPlaceEdit.style.visibility="hidden";
					}
					liveModel.setProperty("/inPlaceEditorId", null);
				},0);
			}
			var selected=liveModel.getProperty("/editing");
			if(!selected){
				return;
			}

			liveModel.setProperty("/editing",null);
			slimsheet.fireCellSelected({selected:true,row:selected.row,col:selected.column/*+1*/,filter:selected.getSheetFilters()/*scenario:selected.getScenario()*/});
		}
		return;
	},
	saveCellEdits:function(){
		var errored=false;
		var view=this.getView();
		var formula=view.formulaField.getValue();
		var locale = sap.ui.getCore().getConfiguration().getLocale();		// fixes for S1302379
		if(formula===null || formula.trim()===""||formula.trim()==="\"\""||formula.trim()==="''"){	// add some extra validations
			return;
		}
		var formattedFormula=formula;
		if (locale.toLocaleString().indexOf("en")===-1){	// fixes for S1302379
			console.warn("Language is not en-*... updating the formula to be en-*");
			formattedFormula=sas.stress.wkbench.StressUtil.unFormatFormula(formattedFormula);
		}
		var liveModel=view.getModel("live");
		var selected=liveModel.getProperty("/selected");

		var key="$"+selected.colLong+"$"+selected.rowLong;
		var data=selected.getFilterData();
		var overlay=data.ncFormulas[key];
		var rb = sas.stress.wkbench.StressUtil.getBundle();
		var userName = sas.stress.wkbench.StressUtil.getUserName();
		var previousValue="";
		if (overlay) {
			previousValue = this._getPreviousValue(overlay, data);	// RQSSTR-836
			var cellValue=sas.stress.wkbench.model.Project.getCurrentProject().getCellValueByDataKey(overlay.dataKey);	// S1345410
			var showEquals = (overlay.expr.type !== "NAME" && overlay.expr.type !== "PREFIX") || (overlay.expr.name && overlay.expr.name.indexOf("$") !== -1);
			try {
				previousValue = (showEquals ? "=" : "") + sas.stress.wkbench.parser.ExpressionUtil.print(overlay.expr, $.proxy(data.printShortContext, data));
				// S1345410
				if (showEquals && !_.isNull(cellValue/*data.data[row][col]*/) &&
					(!_.isNumber(cellValue/*data.data[row][col]*/) && !_.isBoolean(cellValue/*data.data[row][col]*/) && !_.isString(cellValue/*data.data[row][col]*/))) {	//fixes for S1301860.. ignore numeric, text and boolean values
					previousValue = cellValue/*data.data[row][col]*/ + " " + previousValue;
				}
			} catch (e) {
				console.error("Unable to evaluate expression " + sas.stress.wkbench.parser.ExpressionUtil.toString(overlay.expr), overlay);
				previousValue = "#NAME? " + overlay.txt;
			}
		}

		var slimsheet = liveModel.getProperty("/slimsheet");
		try {
			sas.stress.wkbench.CellEvaluationUtil.saveNewFormula(formattedFormula, selected, true);
			var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
			wkbenchView.getController().navChecker.setDirty(true);
			wkbenchView.getModel("live").setProperty("/toggleButtons", true);

			if (selected.getSheetFilters().getSheetInfo().getSheet().getProject().doesRequireComments()) {	// comments required if set
				selected.getSheetFilters().getData().addComment(key, selected.row, selected.column, userName,
					rb.getText("cell.override.default.comment.txt")+previousValue,"sheetOverride", true);	// isMandatory = true

				// S1342467 .. need to refresh the cell comment navigations
				wkbenchView.getScenarioEditorView().getController().updateModel(false,selected);
			}
		}catch(e) {
			errored=true;
			console.error("ERROR saving cell edit: " + e);
			// fixes for CET defect S1266931
			var messages = [];
			messages.push(rb.getText("list.bullet.txt") + e);

			var json = {
				message: rb.getText("ERROR editing cell: "),
				details: messages
			};
			sas.stress.wkbench.StressUtil.handleServerFailure(e);
		}finally{
			this.leaveEditMode();
			if(!errored&&selected.getSheetFilters().getSheetInfo().getSheet().getProject().doesRequireComments()){	// comments required if set
				// get the current scenario view
				var scView=wkbenchView.getScenarioEditorView();
				if (!scView.layout.getPropertyPaneVisible()){	// if the pane is not visible
					// make it visible and select the comment pane.. this would handle the re-rendering of the sheet
					scView.layout.setSelectedPropertyPaneIndex(1);
					scView.layout.setPropertyPaneVisible(true);
				}else {
					// re-render the slimsheet
					slimsheet.rerender();
					// select the comment pane
					if(scView.layout.getSelectedPropertyPaneIndex()!==1){
						scView.layout.setSelectedPropertyPaneIndex(1);//  .getPropertyPanes()[1].getButtonData().firePress({});
					}
				}
				scView.getModel().setProperty("/autoCloseCommentPaneOnEsc",true);
			}else{
				slimsheet.rerender();	// if comments are not mandatory after override rerender the slimsheet
			}
		}
	},
	enableAllocationActions:function(cellRef){
		var canBeEnabled=false;
		var actions=["even","equally","proportional"];
		var controller = this;
		var rb=sas.stress.wkbench.StressUtil.getBundle();
		if (cellRef) {
			var isEditable = !!cellRef && cellRef.getInfoData().isEditable(cellRef.row, cellRef.column);
			// fixes for allocations on columns.. need to look at row and column.. if row is false then look at column
			var isSubtotalNode = cellRef.isSubTotalNode();
			canBeEnabled=isEditable&&isSubtotalNode;
		}else{
			return canBeEnabled;
		}

		if (canBeEnabled) {
			var CellEvaluationUtil = sas.stress.wkbench.CellEvaluationUtil;
			var oldExpr = CellEvaluationUtil.getLatestExpr(cellRef);
			if (oldExpr) {	// make sure there is something... if null then we cannot allocate
				if (oldExpr.args) {
				} else {
					if (isFinite(oldExpr.name)) {
						return false;
					} else {
						try {
							sas.stress.wkbench.model.CellRef.getCellRefs(cellRef.getSheetFilters().getData().printShortContext(oldExpr.name), cellRef.getSheetFilters());
						} catch (e) {
							// the value cannot be parsed to a cell reference, we cannot proceed ahead... could be plain TEXT
							return false;
						}
					}
				}
			}else{
				return false;		// we should not enabling if the value is null.. we do not allocate to children that are null
			}
			var view = this.getView();
			// do this in a timeout because 2 events getting called and we need to be careful one does not tread on other
			setTimeout(function(){
				view.actions.removeAllItems();

				_.each(actions, function (action) {
					var enabled=true;
					if (action==="equally"){	// if equally
						// check for contra children
						enabled=cellRef.getRowNode().getParent().isAnyChildMarkedContra()?false:true;	// and yes, enabled = false
					}
					var item = new sas.hc.m.MenuItem({
						key: action,
						text: rb.getText("cell.editor.allocation.rule." + action + ".menu.item.txt"),
						press: [controller.performAllocations, controller],
						enabled:enabled
					});
					view.actions.addItem(item);
				});
			},0);
		}

		return canBeEnabled;
	},
	performAllocations:function(e){
		var item = e.getSource();
		var action = item.getKey();
		var view=this.getView();
		var formula=view.formulaField.getValue();
		if(formula===null || formula.trim()===""){
			return;
		}
		var formattedFormula=formula;
		var locale = sap.ui.getCore().getConfiguration().getLocale();		// fixes for S1302379 and S1348891
		if (locale.toLocaleString().indexOf("en")===-1){	// fixes for S1302379
			console.warn("Language is not en-*... updating the formula to be en-*");
			formattedFormula=sas.stress.wkbench.StressUtil.unFormatFormula(formattedFormula);	// S1302379.. i18N for formula during allocations
		}
		var liveModel=view.getModel("live");
		var selected=liveModel.getProperty("/selected");
		var slimsheet = liveModel.getProperty("/slimsheet");
		try {
			var allocations=new sas.stress.wkbench.Allocations(formattedFormula, selected, action);		// S1302379.. i18N for formula during allocations
			allocations.performAllocations(formattedFormula, selected, action);	// S1302379.. i18N for formula during allocations
			delete allocations;
			selected.destroy();
			var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
			wkbenchView.getController().navChecker.setDirty(true);
			wkbenchView.getModel("live").setProperty("/toggleButtons", true);
			wkbenchView.getScenarioEditorView().getController().updateModel(false,selected);	// S1348891
		}catch(e) {
			if (e.stack){
				console.error("PERFORM ALLOCATIONS FAILURE: " + e.stack);
			}
			// need to show a message box about the invalidity of the input
			var message,rb=sas.stress.wkbench.StressUtil.getBundle();
			if (e===sas.stress.wkbench.CellEvaluationUtil.INVALID_ALLOCATION_VALUE) {
				message =rb.getText("cell.editor.allocation.invalid.value.fmt", formula);
			}else{
				message =rb.getText("cell.editor.allocation.failure.fmt", e);
			}
			sas.hc.m.MessageDialog.error({text: message});
		}finally{
			slimsheet.rerender();
			this.leaveEditMode();
		}

	},
	clearFormulaRefs:function(){
		var liveModel=this.getView().getModel("live");
		liveModel.setProperty("/informula",[]);
	},
	highlightFormulaRefs:function(formula){
		var liveModel=this.getView().getModel("live");
		var selected=liveModel.getProperty("/selected");
		var filters/*scenario*/=selected.getSheetFilters();
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		try{
			var expression=CellEvaluationUtil.parseFormula(formula);
			var refs=CellEvaluationUtil.getCellRefs(expression,filters/*scenario*/);
			console.info("Highlighting "+refs.length+" cells",new Date());
			liveModel.setProperty("/informula",refs);
			/*var slimsheet=liveModel.getProperty("/slimsheet");
			slimsheet.addInFormulaStyleClassToCells(refs);*/
		}catch(e){
			//console.error(e); // ignore for now
		}
	},
	pressDrillDownButton:function(e){
		var view=this.getView();
		var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
		wkbenchView.setBusy(true);
		var iterationNumber=view.getModel("live").getProperty("/iterationNumber");
		var project=sas.stress.wkbench.model.Project.getCurrentProject();

		var rb=sas.stress.wkbench.StressUtil.getBundle();
		var liveModel=view.getModel("live");
		var cellRef=liveModel.getProperty("/selected");
		var cellRefs=liveModel.getProperty("/selecteds");

		var cellReferenceList = [],
			dataKeyList = [];

		var selectedSheetId=wkbenchView.getSelectedSheetId();
		var scenario,
			dataKey,
			key;
		if (cellRef){
			key = "$"+cellRef.colLong+"$"+cellRef.rowLong;
			scenario=cellRef.getSheetFilters();
			cellReferenceList.push(cellRef.colShort+cellRef.rowShort);
			dataKey=scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			dataKeyList.push(dataKey);
		} else if (cellRefs && cellRefs.length){
			_.each(cellRefs, function (cellRef) {
				key = "$"+cellRef.colLong+"$"+cellRef.rowLong;
				scenario=cellRef.getSheetFilters();
				var drillDownOverlay = scenario.getData().cDrillDowns[key];
				if (drillDownOverlay && (drillDownOverlay.propertyKey > 0 || drillDownOverlay.inheritedPropertyKey)) {
					cellReferenceList.push(cellRef.colShort + cellRef.rowShort);
					dataKey = scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
					dataKeyList.push(dataKey);
				}
			});
		}
		var drillDownMessage = sas.stress.wkbench.StressUtil.getConfirmationMessage(rb.getText("drilldown.selected.cells.generate.busy.indicator.message.txt"), cellReferenceList);
		wkbenchView.getController().openIfRequiredAndUpdateBzDialog(drillDownMessage);

		var data={sheetKey: selectedSheetId, dataKeys: dataKeyList};
		sas.stress.reports.StressAPI.drillDown(project.getId(),data,iterationNumber||project.getCurrentIterationNo())// fixes for S1302840, S1304459
			.done(function () {
				wkbenchView.setBusy(false);
				wkbenchView.getController().closeBzDialog();
			})
			.fail(function (error) {
				wkbenchView.getController().closeBzDialog();
				wkbenchView.setBusy(false);
				sas.stress.wkbench.StressUtil.handleServerFailure(error, rb.getText("drilldown.title.txt"));
			});
	},
	openCellCommentPopup: function (selected,slimsheet) {
		var cell = sas.stress.wkbench.widget.Slimsheet.findCell(slimsheet, selected.row, slimsheet.getSlimsheetSource().getCalculatedColumn(selected.column/*+1*/, selected.filterId/*.scenarioId*/), false);
		if (selected && slimsheet && cell) {
			var existingPopup=sap.ui.getCore().byId("sas.stress.wkbench.view.CellCommentPopup");
			if (existingPopup){
				existingPopup.destroy();
			}
			var cellCommentEditor = sas.stress.wkbench.ViewManager.loadCellCommentEditor(selected,true,false);
			var cellCommentPopup = new sas.hc.m.ResponsivePopover({
				id: "sas.stress.wkbench.view.CellCommentPopup",
				title: sas.stress.wkbench.StressUtil.getBundle().getText("cell.comment.dialog.title.txt"),
				verticalScrolling: false,
				placement: sap.m.PlacementType.Top,
				modal: true,
				initialFocus: cellCommentEditor.commentTextArea,
				content: cellCommentEditor,
				offsetY:10
			});
			cellCommentPopup.addStyleClass("stressCommentPopover").addStyleClass("stressRequiredField");
			cellCommentEditor.attachClosing(function () {
				this.destroy();
			}, cellCommentPopup);
			cellCommentPopup.openBy(cell);
		}
	},
	_highlightCell:function(){
		/* keep the cell highlighted... Ritesh spoke about this.. keeps Excel like functionality */
		var liveModel=this.getView().getModel("live");
		var selected=liveModel.getProperty("/selected");
		if (selected) {
			var slimSheet=liveModel.getProperty("/slimsheet");
			// use the static because we do not want to fire the cell selected action.. only highlight the cell
			sas.stress.wkbench.widget.Slimsheet.selectCell(slimSheet, selected.row, selected.column, selected.filterId);
		}
	},
	/**
	 *
	 * @returns {Element}
	 * @private
     */
	_getInPlaceEditControl: function (){
		var view=this.getView();
		var liveModel=view.getModel("live");
		if (liveModel===undefined){
			return undefined;
		}
		//var slimsheet=liveModel.getProperty("/slimsheet");
		var inPlaceEditId=liveModel.getProperty("/inPlaceEditorId");
		//inplaceEdit=document.getElementById(inPlaceEditId);
		var inplaceEdit=document.getElementById(inPlaceEditId);
		if (!inplaceEdit){
			var allInPlaces=document.getElementsByClassName("StressSlimsheetInplace");
			inplaceEdit=_.find(allInPlaces, function(ourInPlace){
				return ourInPlace.style.visibility === "visible";
			});
		}
		return inplaceEdit;
	},
	onExit:function(){
		this.destroyed=true;
		sas.stress.wkbench.CellPickerUtil.deactivateEditMode();
		var view=this.getView();
		var liveModel=view.getModel("live");
		var inplaceEdit = this._getInPlaceEditControl();
		if(inplaceEdit&&inplaceEdit.style.visibility==="visible"){
			// RQSSTR-796.. disable the in place editor
			var slimsheet=liveModel.getProperty("/slimsheet");
			sas.stress.wkbench.widget.Slimsheet.dimInPlaceEdit(slimsheet);
			liveModel.setProperty("/inPlaceEditorId", null);
		}
	},
	pressCopyButton: function(){
		var view=this.getView();
		var copyCellsModel = view.getModel();
		var liveModel=view.getModel("live");
		var selected = liveModel.getProperty("/selected");
		var sourceCellRange = liveModel.getProperty("/selecteds");
		var slimSheet=liveModel.getProperty("/slimsheet");

		liveModel.setProperty("/copyButtonEnabled", true);
		copyCellsModel.setProperty("/cellRange", slimSheet.getSlimsheetSource().getCellRange());
		if (selected){
			copyCellsModel.setProperty("/copyCells", selected);
			copyCellsModel.setProperty("/isRangeCopy", false);
		} else if (sourceCellRange && sourceCellRange.length){
			copyCellsModel.setProperty("/copyCells", sourceCellRange);
			copyCellsModel.setProperty("/isRangeCopy", true);
		}
		slimSheet.getSlimsheetSource().getCellRange().isCopySource = true;
		sas.stress.wkbench.widget.Slimsheet.rangeSelectCells(slimSheet, slimSheet.getSlimsheetSource().getCellRange(), undefined);
	},
	pressPasteButton: function() {
		var view = this.getView();
		var copyCellsModel = view.getModel();
		var liveModel = view.getModel("live");
		var selected = liveModel.getProperty("/selected");
		var slimSheet=liveModel.getProperty("/slimsheet");
		var slimSheetSource = slimSheet.getSlimsheetSource();
		var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
		var rb = sas.stress.wkbench.StressUtil.getBundle();

		var copyCells = copyCellsModel.getProperty("/copyCells");
		var isRangeCopy = copyCellsModel.getProperty("/isRangeCopy");
		var sourceCellRange = copyCellsModel.getProperty("/cellRange");
		var key,
			cellOverlay,
			cellShortForm;
		var project = sas.stress.wkbench.model.Project.getCurrentProject();
		var newComment,
			previousValue;
		if (!isRangeCopy) {
			key = "$" + copyCells.colLong + "$" + copyCells.rowLong;
			cellShortForm = copyCells.colShort + copyCells.rowShort;
			cellOverlay = copyCells.getSheetFilters().getData().ncFormulas[key];
			if (cellOverlay) {
				var cellValue = project.getCellValueByDataKey(cellOverlay.dataKey);
				if (cellValue !== undefined || cellValue !== null) {
					if (typeof cellValue === "string"){	// handle strings
						cellValue = "'" + cellValue +"'";
					}
					sas.stress.wkbench.CellEvaluationUtil.saveNewFormula(cellValue.toString(), selected, true);
					if (selected.getSheetFilters().getSheetInfo().getSheet().getProject().doesRequireComments()) {	// comments required if set
						// RQSSTR-836
						var data = copyCells.getFilterData()/*getScenarioData()*/;
						var userName = sas.stress.wkbench.StressUtil.getUserName();
						previousValue = this._getPreviousValue(cellOverlay, data);
						newComment = rb.getText("cell.copy.override.default.comment.fmt", cellShortForm) + previousValue;
						data.addComment("$" + selected.colLong + "$" + selected.rowLong, selected.row, selected.column, userName, newComment);
					}
				}

				slimSheet.rerender();
			}
		}else {
			var targetRow = selected.row;
			var targetColumn = selected.column;
			var rowIndex,
				columnIndex;

			var sheetInfoData = selected.getInfoData();
			var sourceRowCount = 0;
			for (var rowIndex = sourceCellRange.previousRow; rowIndex <= sourceCellRange.currentRow; rowIndex++) {
				sourceRowCount++;
			}
			var sourceColumnCount = 0;
			for (var columnIndex = sourceCellRange.previousColumn; columnIndex <= sourceCellRange.currentColumn; columnIndex++) {
				sourceColumnCount++;
			}

			if (targetRow + sourceRowCount > sheetInfoData.rows.length){
				sas.hc.m.MessageDialog.error({
					text: rb.getText("copy.operation.invalid.rows.fmt", [sourceRowCount, sheetInfoData.rows.length - targetRow]),
					title: rb.getText("copy.paste.operation.title.txt")
				});
				return;
			}
			if (targetColumn + sourceColumnCount > sheetInfoData.columns.length){
				sas.hc.m.MessageDialog.error({
					text: rb.getText("copy.operation.invalid.columns.fmt", [sourceColumnCount, sheetInfoData.columns.length - targetColumn]),
					title: rb.getText("copy.paste.operation.title.txt")
				});
				return;
			}
			wkbenchView.getController().openIfRequiredAndUpdateBzDialog("copying cell values");
			// from the source initialrow to source endRow
			for (rowIndex = sourceCellRange.previousRow; rowIndex <= sourceCellRange.currentRow; rowIndex++) {
				// from the source initial column to source end column
				targetColumn = selected.column;
				for (columnIndex = sourceCellRange.previousColumn; columnIndex <= sourceCellRange.currentColumn; columnIndex++) {
					// get the source Cell reference
					var sourceCellRef = _.find(copyCells, function(cellRef){
						return cellRef.row === rowIndex && cellRef.column === columnIndex;
					});

					if (sourceCellRef){
						// the target cell reference
						var targetCellRef = new sas.stress.wkbench.model.CellRef(targetColumn, targetRow, slimSheetSource.getCellFilter(targetRow, targetColumn));
						key = "$" + sourceCellRef.colLong + "$" + sourceCellRef.rowLong;
						cellShortForm = sourceCellRef.colShort + sourceCellRef.rowShort;
						cellOverlay = sourceCellRef.getSheetFilters().getData().ncFormulas[key];
						if (cellOverlay) {
							var cellValue = project.getCellValueByDataKey(cellOverlay.dataKey);
							if (cellValue!== undefined || cellValue !== null) {	// cell could contain 0
								if (typeof cellValue === "string"){	// handle strings
									cellValue = "'" + cellValue +"'";
								}
								sas.stress.wkbench.CellEvaluationUtil.saveNewFormula(cellValue.toString(), targetCellRef, true);
								if (targetCellRef.getSheetFilters().getSheetInfo().getSheet().getProject().doesRequireComments()) {	// comments required if set
									// RQSSTR-836
									var data = targetCellRef.getFilterData()/*getScenarioData()*/;
									var userName = sas.stress.wkbench.StressUtil.getUserName();
									previousValue = this._getPreviousValue(cellOverlay, data);

									newComment = rb.getText("cell.copy.override.default.comment.fmt", cellShortForm) + previousValue;
									data.addComment("$" + targetCellRef.colLong + "$" + targetCellRef.rowLong, targetCellRef.row, targetCellRef.column, userName, newComment + previousValue);
								}
							}
						}
					}
					targetColumn = targetColumn + 1;
				}
				targetRow = targetRow + 1;
			}
			targetColumn = selected.column + (sourceColumnCount - 1);
			targetRow = selected.row + (sourceRowCount - 1);

			slimSheet.rerender();
			var targetCellRange = {
				previousRow: selected.row,
				currentRow: targetRow,

				previousColumn: selected.column,
				currentColumn: targetColumn,

				isCopySource: false
			};
			wkbenchView.getController().closeBzDialog();
			// show the source range of the copy operation
			sourceCellRange.isCopySource = true;
			sas.stress.wkbench.widget.Slimsheet.rangeSelectCells(slimSheet, sourceCellRange, slimSheetSource.getCellFilter(targetRow, targetColumn), false);
			// target range of the copy operation
			sas.stress.wkbench.widget.Slimsheet.rangeSelectCells(slimSheet, targetCellRange, slimSheetSource.getCellFilter(targetRow, targetColumn), true);
		}
		wkbenchView.getController().navChecker.setDirty(true);
		liveModel.setProperty("/toggleButtons", true);
		wkbenchView.getScenarioEditorView().getController().updateModel(false,selected);	// S1348891

		sas.hc.m.MessageToast.show(rb.getText("copy.paste.success.title.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
	},

	// RQSSTR-836..
	// making this a common function for cell override and copy-paste
	_getPreviousValue: function (overlay, data){
		var previousValue;
		if (overlay) {
			var cellValue=sas.stress.wkbench.model.Project.getCurrentProject().getCellValueByDataKey(overlay.dataKey);	// S1345410
			var showEquals = (overlay.expr.type !== "NAME" && overlay.expr.type !== "PREFIX") || (overlay.expr.name && overlay.expr.name.indexOf("$") !== -1);
			try {
				previousValue = (showEquals ? "=" : "") + sas.stress.wkbench.parser.ExpressionUtil.print(overlay.expr, $.proxy(data.printShortContext, data));
				// S1345410
				if (showEquals && !_.isNull(cellValue/*data.data[row][col]*/) &&
					(!_.isNumber(cellValue/*data.data[row][col]*/) && !_.isBoolean(cellValue/*data.data[row][col]*/) && !_.isString(cellValue/*data.data[row][col]*/))) {	//fixes for S1301860.. ignore numeric, text and boolean values
					previousValue = cellValue/*data.data[row][col]*/ + " " + previousValue;
				}
			} catch (e) {
				console.error("Unable to evaluate expression " + sas.stress.wkbench.parser.ExpressionUtil.toString(overlay.expr), overlay);
				previousValue = "#NAME? " + overlay.txt;
			}
		}

		return previousValue;
	}
});