/**
 * Created by sinsrj on 1/24/2018.
 */
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sap.ui.controller("sas.stress.wkbench.view.SelectSheetsForDownload",{
    onInit: function () {
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var self = this;
        var viewData = this.getView().getViewData();
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var model = new sas.risk.common.models.RiskJSONModel({
            sheetDefinitions: project.getSheets()/*,
             projectIsLocked:project.isLocked()*/,
            worksheets: [],

            categories: {}
        });

        var _categories = {};
        // the all category will contain the total count and it goes first
        var sheets = project.getSheets();	// fixes for S1231681... all the sheets should be used to figure out the categories
        _categories[rb.getText("filter.category.all.txt")] = {
            count: sheets.length,
            key: "0",
            name: rb.getText("filter.category.all.txt")
        };
        var selectedCategoryName="";
        _.each(sheets, function (sheet) {
            var categoryKey = sheet.getSheetCategory().getId();
            var categoryName = sheet.getSheetCategory().getName();

            var sheetCategoryCount = _categories[categoryName] || {count: 0, key: "0", name: ""};
            // increment the count &&
            sheetCategoryCount.count++;
            _categories[categoryName] = {count: sheetCategoryCount.count, key: categoryKey, name: categoryName};
        });

        var _workSheets = [];
        // the current worksheet
        var currentWorksheet = _.find(sheets, function(sheet){
            return sheet.getId() === viewData.currentWorksheetId;
        });
        _.each(sheets, function(sheet){
            var workSheet = {selected: false, nameVersion:"", key: 0, categoryKey: ""};
            var isSelected = sheet.getId() === viewData.currentWorksheetId;
            workSheet.selected = isSelected;
            workSheet.nameVersion = sas.stress.wkbench.StressUtil.getNameVersion(sheet.name, sheet.version);
            workSheet.key = sheet.getId();
            workSheet.categoryKey = sheet.sheetCategory.id;
            _workSheets.push(workSheet);
        });

        model.setProperty("/categories", _categories);
        model.setProperty("/worksheets", _workSheets);
        model.setProperty("/currentWorksheet", currentWorksheet);

        self.getView().setModel(model);
        var selectedCategoryItem=_.find(_categories, function(categoryItem){
            return categoryItem.key === viewData.selectedCategoryKey;
        });
        // the selected category
        model.setProperty("/selectedCategoryName", selectedCategoryItem.name);
        model.setProperty("/selectedCategoryItem", selectedCategoryItem);
    },
    onExit: function () {
        this.itemsTable=undefined;
        this.categoriesFilter=undefined;
    },

    toggleEditProjectDialogSaveButton: function(e) {
        /*var selectedItems=this.getView().workSheetList.getSelectedItems();
         if (selectedItems.length === 0){
         this.getView().okButton.setEnabled(true);
         return;
         }
         var areEqual = sas.stress.wkbench.model.Project._areSheetDefinitionsEqual(selectedItems);
         this.getView().okButton.setEnabled(!areEqual);*/
        var selectedItems= e.getParameter("selectedItems")||this.getView().workSheetList.getSelectedSheetTemplates();
        var isValid = (e.getParameter("selectedItemsLength") || selectedItems.length) !==0;
        var areEqual = false;
        if (isValid){
            areEqual = sas.stress.wkbench.model.Project._areSheetDefinitionsEqual(selectedItems);
        }
        this.getView().okButton.setEnabled(isValid&&!areEqual);
    },

    handleOKButtonPress: function() {
        // save the project
        var view = this.getView();
        var model = view.getModel();
        var sheetDefinitions = [];
        _.each(view.workSheetList.getSelectedSheetTemplates(), function (workSheet) {
            sheetDefinitions.push(Number(workSheet.key));	// the sheets to be downloaded
        });

        view.fireEvent(view.getId() + "_CLOSING", { "action": "DONE", "response": "Done", "data": sheetDefinitions });
    },

    cancelAddRemoveSheetTemplates: function() {
        var view = this.getView();
        view.fireEvent(view.getId() + "_CLOSING", { "action": "CANCEL", response: "Cancel" });
    },
    //Kept this callback outside the widget since the matching comparision may differ based on the data.
    handleTestWidgetReady: function(table, list){
        this.widgetRendered=true;
        this.itemsTable=table;
        this.categoriesFilter=list;

        this._refreshView();
    },

    _refreshView: function(){
        var self = this;
        var model = self.getView().getModel();
        var currentWorksheet = model.getProperty("/currentWorksheet");
        if (this.widgetRendered) {
            var rb=sas.stress.wkbench.StressUtil.getBundle();

            self.itemsTable.getTitle().setProperty("text", rb.getText("worksheet.export.sheetlist.title.txt"));
            // select the default or already present items
            //var inputTemplates = model.getProperty("/sheetDefinitions");
            var tableRowsBinding = self.itemsTable.getBinding("rows");
            var rowsData = tableRowsBinding.getModel().getProperty(tableRowsBinding.getPath());
            var match = _.find(rowsData, function(template) {
                return template.key === currentWorksheet.id;
            });
            if (match) {
                var index = rowsData.indexOf(match);
                self.itemsTable.addSelectionInterval(index, index);
            }

            // select the default after all events processed
            setTimeout(function(){
                // select the default category
                if (model.getProperty("/selectedCategoryItem")){
                    self.categoriesFilter.setSelectedItem(model.getProperty("/selectedCategoryItem"), true);
                }
            }, 0);

        }

        this.widgetRendered=false;  // reset once done
    }
});