jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.StressRouter");
jQuery.sap.require("sas.stress.wkbench.UserPrefs");
jQuery.sap.require("sas.stress.wkbench.EventBusConstants");
jQuery.sap.require("sas.stress.wkbench.SASStudioProxy");
jQuery.sap.require("sas.stress.wkbench.StressLocales");

sap.ui.controller("sas.stress.wkbench.view.StressShell", {
	onInit:function(){
		sas.applicationSwitcher.updateTitle("sasstress",window['sap-ui-config'].sas.appName);
        var that=this;
        $.when(
            sas.stress.wkbench.StressAPI.loadSessionProperties(),
            sas.stress.wkbench.UserPrefs.load(),
            this.initializeRFW()
        ).done(function(sessionProperties){
            sas.stress.wkbench.StressUtil.setUserName(sessionProperties.userName);
            sas.stress.wkbench.StressUtil.setUserDisplayName(sessionProperties.userDisplayName);
            sas.stress.wkbench.StressUtil.setRiskWorkGroups(sessionProperties.riskWorkGroups);
            sas.stress.wkbench.StressUtil.setUserWorkGroups(sessionProperties.userWorkgroups);
            sas.stress.wkbench.CapabilityManager.createInstance(sessionProperties.userCapabilities);

            // lets initialize the router
            that.initializeRouter();

            // let's load the locale specific separators
            sas.stress.wkbench.StressLocales._loadDefaultSeparators();

            // define the Array.prototype.fill method.. not defined in IE
            //https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/fill?v=example#Polyfill
            if (!Array.prototype.fill) {
                Object.defineProperty(Array.prototype, 'fill', {
                    value: function(value) {

                        // Steps 1-2.
                        if (this == null) {
                            throw new TypeError('this is null or not defined');
                        }

                        var O = Object(this);

                        // Steps 3-5.
                        var len = O.length >>> 0;

                        // Steps 6-7.
                        var start = arguments[1];
                        var relativeStart = start >> 0;

                        // Step 8.
                        var k = relativeStart < 0 ?
                            Math.max(len + relativeStart, 0) :
                            Math.min(relativeStart, len);

                        // Steps 9-10.
                        var end = arguments[2];
                        var relativeEnd = end === undefined ?
                            len : end >> 0;

                        // Step 11.
                        var final = relativeEnd < 0 ?
                            Math.max(len + relativeEnd, 0) :
                            Math.min(relativeEnd, len);

                        // Step 12.
                        while (k < final) {
                            O[k] = value;
                            k++;
                        }

                        // Step 13.
                        return O;
                    }
                });
            }
        }).fail(function(jqxhr){
            var rb=sas.stress.wkbench.StressUtil.getBundle();
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
		});
	},
    initializeRouter:function(){
        var oRouter = new sas.stress.wkbench.StressRouter(
            {
                HomePage: {
                    pattern: "Project/{projectId}/Home",
                    view: "HomePage",
                    viewId: "HomePageView"
                },
                Workbench: {
                    pattern: "Project/{projectId}/Workbench",
                    view: "Workbench",
                    viewId: "WorkbenchView"
                },
                ProjectManager:{
                    pattern: "ProjectManager",
                    view: "ProjectManager",
                    viewId: "ProjectManagerView"
                },
                HierarchyMapping:{
                    pattern: "Configuration/HierarchyMapping/{hierarchyId}",
                    view: "HierarchyMapping",
                    viewPath: "sas.stress.config.view",
                    viewId: "HierarchyMappingView"
                },
                Configuration:{
                    pattern: "Configuration",
                    view: "Configuration",
                    viewPath: "sas.stress.config.view",
                    viewId: "ConfigurationView"
                },
                Reports:{
                    pattern: "Project/{projectId}/Reports",
                    view: "ProjectReportsEditor",
                    viewPath: "sas.stress.reports.view",
                    viewId: "ProjectReportsEditorView"
                },
                ScenarioPage:{
                    pattern: "Project/{projectId}/Scenarios",
                    view: "ScenarioPage",
                    viewPath: "sas.stress.scenarios.view",
                    viewId: "ScenarioPageView"
                },
                PackagePage:{
                    pattern: "Project/{projectId}/Packages",
                    view: "PackagePage",
                    viewPath: "sas.stress.packages.view",
                    viewId: "PackagePageView"
                },
                NotificationPage:{
                    pattern: "Project/{projectId}/Notifications",
                    view: "NotificationPage",
                    viewPath: "sas.stress.notification.view",
                    viewId: "NotificationPageView"
                },
                GlobalScenarioPage:{
                    pattern: "Scenarios",
                    view: "GlobalScenarioPage",
                    viewPath: "sas.stress.scenarios.view",
                    viewId: "GlobalScenarioPageView"
                },
                //getCreateModelGroup:{
		        //    pattern: "ModelGroups",
		        //    view: "ModelGroups",
		        //    viewPath: "sas.stress.models.view.groups",
                 //   viewId: "ModelGroupsView"
		        //},
                ModelManager:{
                    pattern: "Project/{projectId}/Models",
                    view: "ModelManager",
                    viewPath: "sas.stress.models.view.execution",
                    viewId: "ModelManagerView"
                },
                ModelGroupRunsManager:{
                    pattern: "Project/{projectId}/ModelGroupRuns",
                    view: "ModelGroupRunsManager",
                    viewPath: "sas.stress.models.view.execution",
                    viewId: "ModelGroupRunsView"
                },
                ModelGroupRunsSensitivity:{
                    pattern: "Project/{projectId}/ModelGroupRunsSensitivity",
                    view: "ModelGroupRunsSensitivity",
                    viewPath: "sas.stress.models.view.execution",
                    viewId: "ModelGroupRunsSensitivityView"
                },
                ModelGroupRunsCreate:{
                    pattern: "Project/{projectId}/ModelGroupRunsCreate",
                    view: "ModelGroupRunsCreate",
                    viewPath: "sas.stress.models.view.execution",
                    viewId: "ModelGroupRunsCreateView"
                },
                ModelDetails:{
                    pattern: "Project/{projectId}/Models/{modelId}",
                    view: "ModelDetails",
                    viewPath: "sas.stress.models.view.execution",
                    viewId: "ModelDetailsView"
                },
                BrowseModelInventory:{
                    pattern: "Models",
                    view: "BrowseModelInventory",
                    viewPath: "sas.stress.models.view.browse",
                    viewId: "BrowseModelInventoryView"
                },
                InventoryModelDetails:{
                    pattern: "Models/{modelId}",
                    view: "InventoryModelDetails",
                    viewPath: "sas.stress.models.view.browse",
                    viewId: "InventoryModelDetailsView"
                },
                Data:{
                    pattern: "Data",
                    view: "DataEditor",
                    viewPath: "sas.stress.data.view",
                    viewId: "DataEditorView"
                },
                ProjectData:{
                    pattern: "Project/{projectId}/Data",
                    view: "ProjectDataEditor",
                    viewPath: "sas.stress.data.view",
                    viewId: "ProjectDataEditorView"
                },
                ModelGroups:{
                    pattern: "ModelGroups",
                    view: "ModelGroups",
                    viewPath: "sas.stress.models.view.groups",
                    viewId: "ModelGroupsView"
                },
                Scripts:{
                    pattern: "Scripts",
                    view: "ScriptManager",
                    viewPath: "sas.stress.scripts.view",
                    viewId: "ScriptManagerView"
                },
                ScriptDetails:{
                    pattern: "Scripts/{scriptId}",
                    view: "CreateScript",
                    viewPath: "sas.stress.scripts.view",
                    viewId: "ScriptDetailsView"
                },
                ModelGroupDetails:{
                    pattern: "ModelGroups/{modelGroupId}",
                    view: "CreateModelGroup",
                    viewPath: "sas.stress.models.view.groups",
                    viewId: "ModelGroupDetailsView"
                },
                ArchiveManager:{
                    pattern: "ArchiveManager",
                    view: "ArchiveManager",
                    viewPath: "sas.stress.archive.view",
                    viewId: "ArchiveManagerView"
                },
                CreateArchive:{
                    pattern: "CreateArchive",
                    view: "CreateArchive",
                    viewPath: "sas.stress.archive.view",
                    viewId: "CreateArchiveView"
                },
                Default:{
                    pattern: "",
                    view: "ProjectManager",
                    viewId: "ProjectManagerView"
                }
            },
            {
                targetControl: "StressShellArea",
                targetAggregation: "content",
                viewType: sap.ui.core.mvc.ViewType.JS,
                viewPath: "sas.stress.wkbench.view",
                clearTarget: true
            });

        //router initialize here
        oRouter.register("router");

    //    sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_GROUPS, this.viewModelGroups, this);
    //    sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_MODEL_GROUP, this.createModelGroupView, this);

        sap.ui.getCore().attachInit(function() {
            oRouter.initialize();
        });
    },

    onExit: function () {
        // Remove any subscriptions to ensure we don't get duplicates the next time the controller is initialized
    //    sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_MODEL_GROUPS, this.viewModelGroups, this);
    //    sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_MODEL_GROUP, this.createModelGroupView, this);
    },

    initializeRFW: function() {
        var deferred = $.Deferred();
        $.when(this._setupGlobalSettings()).done(function () {
            deferred.resolve();
        }).fail(function() {
            deferred.reject();
        });
        return deferred;
    },

    _setupGlobalSettings: function() {
        var deferred = $.Deferred();
        $.getJSON("rest/sysinfo", function(response) {
            sas.stress.wkbench.SASStudioProxy.setStudioURL(response.sasStudioUrl,"sasstress");
            sas.stress.wkbench.SASStudioProxy.setScenarioEditorURL(response.scenarioEditorUrl);
            deferred.resolve();
        }).fail(function( jqxhr, textStatus, error ) {
            sas.log.error("SASStudio: Failed to retrieve launch url:" + error);
            // TODO: Not sure what to do in this case for RFW, MIP does this
            // var url = sas.applicationSwitcher.getCurrentApplication().url;
            // url = url.replace("SASRiskAndFinanceWorkbench\/index.jsp", "SASStudio");
            // sas.stress.wkbench.SASStudioProxy.setStudioURL(url);
            // sas.stress.wkbench.SASStudioProxy.setScenarioEditorURL(url);
       });
    }

});