jQuery.sap.require({modName:"sas.stress.wkbench.view.TaskSASScriptParameters",type:"controller"});
jQuery.sap.require('sas.hc.m.Label');
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.jsview("sas.stress.wkbench.view.TaskSASScriptParameters", {
    getControllerName: function () {
        return "sas.stress.wkbench.view.TaskSASScriptParameters";
    },
    createContent: function (controller) {
        var rb=sas.stress.wkbench.StressUtil.getBundle();

        var okButton = new sas.risk.common.widgets.Button({	// prevent double click
            id: "okbutton",
            text: rb.getText("script.parameters.dialog.OK.button.txt"),
            press: $.proxy(controller.runSASScriptTask, controller),
            enabled:false
        });
        okButton.addStyleClass("sasPrimaryButton");
        this.okButton = okButton;

        this.cancelButton = new sas.hc.m.Button({
            id: "cancelButton",
            text: rb.getText("cancel.txt"),
            press: $.proxy(controller.cancel, controller)
        });

        return this._createBizParametersTabLayout(controller);
    },
    _createBizParametersTabLayout: function (controller,rb) {

        var rb=sas.stress.wkbench.StressUtil.getBundle();

        this.workGroupSelect = new sas.hc.m.Select();

        return sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({
            content: new sas.hc.ui.layout.FixFlex({
                fixContent: new sas.hc.ui.layout.form.Form({
                    layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(),
                    formContainers: [
                        new sas.hc.ui.layout.form.FormContainer({
                            formElements: [
                                new sas.hc.ui.layout.form.FormElement({
                                    label: new sas.hc.m.Label({
                                        text: rb.getText("script.parameters.dialog.workGroup.label.txt")
                                    }),
                                    fields: [
                                        this.workGroupSelect
                                    ]
                                })
                            ]
                        })
                    ]
                }),
                flexContent: this._createBizVariablesTable(controller)
            })
        });

    },
    _createBizVariablesTable: function (controller) {
        var self=this;
        var rb=sas.stress.wkbench.StressUtil.getBundle();

        this.parametersTable = new sas.hc.ui.table.Table({
            editable: false,
            width: "100%",
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.Row,
            showTableOptions: true,
            /*visibleRowCount: {
                path: "/script/defaultParameters",
                formatter: function (assumptions) {
                    return (assumptions && assumptions !== null) ? assumptions.length : 5;
                }
            },*/
            toolbar:this._createParametersTableHeader(controller)
        });

        this.parametersTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("parameter-name"),
            label: new sas.hc.m.Label({ text: rb.getText("script.parameter.column.name.txt") }),
            template: new sas.hc.m.Text({
                text: "{name}",
                tooltip: "{name}"
            }),
            width: "5rem",
            sortProperty: "name"
        }));
        this.parametersTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("parameter-type"),
            label: new sas.hc.m.Label({ text: rb.getText("script.parameter.column.type.txt") }),
            template: new sas.hc.m.Text({
                tooltip: {
                    parts: ["typeCd", "definition"],
                    formatter: $.proxy(self.getScriptParameterTypeResourceString, self)
                }/*"{periodType}"*/,
                text: {
                    parts: ["typeCd", "definition"],
                    formatter: $.proxy(self.getScriptParameterTypeResourceString, self)
                },
                wrapping: false
            }),
            width: "9rem",
            sortProperty: {
                parts: ["typeCd", "definition"],
                formatter: $.proxy(self.getScriptParameterTypeResourceString, self)
            },
            visible:false
        }));
        this.assumptionValue = new sas.hc.m.Input({
            type: sap.m.InputType.Text,
            width: "100%",
            disableValidation: false,
            maxLength: 1024,
            required: false,
            //liveChange: $.proxy(controller._parameterLiveChange, controller), // S1388739
            visible:{
                path:"typeCd",
                formatter:function(type){
                    if (type===undefined|| type===null){
                        return false;
                    }
                    return (type.toLowerCase()==="freeform")?true:false; // simple = freeform therefore, visible
                }
            },
            layoutData: new sas.hc.m.FlexItemData({
                growFactor: {
                    path:"typeCd",
                    formatter:function(type,value) {
                        if (type === undefined || type === null) {
                            return 0;
                        }
                        return (type.toLowerCase() === "freeform") ? 10 : 0; // simple = freeform therefore, visible
                    }
                }
            }),
            value:"{value}",
            customData: [
                new sap.ui.core.CustomData({    // S1383071
                    key: "parameterIndex",
                    value: "{orderNum}"
                }),
                new sap.ui.core.CustomData({    // S1383071 and S1388739
                    key: "parameterType",
                    value: "{typeCd}"
                })
            ]
        });

        var runTimeLayout=new sas.hc.m.HBox/*.HorizontalLayout*/({
            items:[
                this.assumptionValue,
                this._createRuntimeValueSelect(controller),
                new sas.hc.m.Image({
                    src: {
                        parts: ["typeCd","errors"],
                        formatter:function (type, errors){
                            if (_.isUndefined(type) || _.isNull(type)) {
                                return null;
                            }
                            if (_.isUndefined(errors) || _.isNull(errors)) {
                                return null;
                            }
                            if (type.toLowerCase()!=="sascode"){
                                return null;
                            }
                            if (errors.length===0){
                                return null;
                            }
                            var icon="ERRORSTATUS";
                            var iconInfo = sap.ui.core.IconPool.getIconInfo(sas.icons.HC[icon]);
                            return (!iconInfo) ? sas.icons.HC[icon] : iconInfo.uri;
                        }
                    },
                    tooltip:{
                        parts: ["typeCd","errors"],
                        formatter:function (type, errors){
                            if (_.isUndefined(type) || _.isNull(type)) {
                                return null;
                            }
                            if (_.isUndefined(errors) || _.isNull(errors)) {
                                return null;
                            }
                            if (type.toLowerCase()!=="sascode"){
                                return null;
                            }
                            if (errors.length===0){
                                return null;
                            }
                            return rb.getText("task.sas.script.completed.error.txt");
                        }
                    }
                }).addStyleClass("taskSASExecutionStatusIcon"),
                new sas.risk.common.widgets.Button({
                    icon: sas.icons.HC.LOG,
                    visible: {
                        parts: ["typeCd","errors"],
                        formatter:function (type, errors){
                            if (_.isUndefined(type) || _.isNull(type)) {
                                return false;
                            }
                            if (_.isUndefined(errors) || _.isNull(errors)) {
                                return false;
                            }
                            if (type.toLowerCase()!=="sascode"){
                                return false;
                            }
                            if (errors.length===0){
                                return false;
                            }
                            return true;
                        }
                    },
                    enabled: {
                        parts: ["typeCd","errors"],
                        formatter:function (type, errors){
                            if (_.isUndefined(type) || _.isNull(type)) {
                                return false;
                            }
                            if (_.isUndefined(errors) || _.isNull(errors)) {
                                return false;
                            }
                            if (type.toLowerCase()!=="sascode"){
                                return false;
                            }
                            if (errors.length===0){
                                return false;
                            }
                            return true;
                        }
                    },
                    customData: new sap.ui.core.CustomData({    // S1383071
                        key: "parameterIndex",
                        value: "{orderNum}"
                    }),
                    press: $.proxy(controller.onLogButton,controller),
                    layoutData: new sas.hc.m.FlexItemData({
                        growFactor: {
                            parts:["typeCd","errors"],
                            formatter:function(type,errors) {
                                if (type === undefined || type === null) {
                                    return 0;
                                }
                                if (_.isUndefined(errors) || _.isNull(errors)) {
                                    return 0;
                                }
                                return (type.toLowerCase() === "sascode" && errors.length>0) ? 10 : 0; // simple = freeform therefore, visible
                            }
                        }
                    })
                })
            ],
            width:"100%"
        });
        this.parametersTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("parameter-runtime-values"),
            label: new sas.hc.m.Label({ text: rb.getText("script.parameter.column.runtimevalue.txt") }),
            template: runTimeLayout,
            width: "10rem"
        }));
        this.parametersTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("parameter-description"),
            label: new sas.hc.m.Label({ text: rb.getText("script.parameter.column.description.txt") }),
            template: new sas.hc.m.Text({
                text: "{description}",
                tooltip: "{description}"
            }),
            width: "13rem"
        }));

        this.parametersTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
            icon: sas.icons.HC.ZEROSTATEZERO,
            text: rb.getText("script.no.parameters.defined.txt")
        }));

        this.parametersTable.bindRows("/script/defaultParameters");
        return this.parametersTable;
    },
    _createParametersTableHeader: function (controller) {
        return new sas.hc.m.Toolbar({
            content: [
                new sas.hc.m.Label({
                    text: {
                        parts: ["/script/name","/script/version"],
                        formatter:function(name,version){
                            return sas.stress.wkbench.StressUtil.getNameVersion(name,version);
                        }
                    }
                }).addStyleClass("stressModelGroupLabel")
            ]
        });
    },
    attachClosing: function (callBack, listener) {
        this.attachEvent(this.getId() + "_CLOSING", callBack, listener);
    },
    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    },
    _createRuntimeValueSelect:function(controller){
        var self=this;
        var dd = new sas.hc.m.Select({
            enableUnselected: true,
            visible:{
                parts:["typeCd","errors"],
                formatter:function(type,errors){
                    if ((type===undefined|| type===null)){
                        return false;
                    }
                    if((errors===undefined||errors===null)){
                        return false;
                    }
                    return (type.toLowerCase()!=="freeform"&&errors.length===0)?true:false; // simple = freeform therefore, NOT visible
                }
            },
            selectedKey:"{selectedKey}",
            required:true,
            width:"100%",
            maxWidth:"100%",

            customData: new sap.ui.core.CustomData({    // S1383071
                key: "parameterIndex",
                value: "{orderNum}"
            }),
            layoutData: new sas.hc.m.FlexItemData({
                growFactor: {
                    parts:["typeCd","errors"],
                    formatter:function(type,errors) {
                        if (type === undefined || type === null) {
                            return 0;
                        }
                        if((errors===undefined||errors===null)){
                            return 0;
                        }
                        return (type.toLowerCase() !== "freeform" && errors.length===0) ? 10 : 0; // simple = freeform therefore, visible
                    }
                }
            }),
            change: $.proxy(controller._selectDefaultValue, controller)/*function(){
             var bindingContext=this.getBindingContext();
             var object=bindingContext.getObject();
             console.log("selected: " + object);
             (controller._selectDefaultValue(object));
             }*/
        });
        //dd.attachChange($.proxy(controller._selectDefaultValue, controller));
        var itemTemplate = new sas.hc.ui.core.ListItem({
            text: {
                parts:["name","version"],
                formatter:function(name,version){
                    return sas.stress.wkbench.StressUtil.getNameVersion(name,version);
                }
            },
            key: "{id}"
        });
        dd.bindAggregation("items", {
            path: "allowableValues",
            template: itemTemplate,
            templateShareable: false
        });

        //this._defaultValuesSelect=dd;
        return dd;
    },
    getScriptParameterTypeResourceString:function(type, objectType) {
        var resourceModel = this.getModel("i18N");
        if (resourceModel && type) {
            switch (type.toLowerCase()) {
                case "simple":
                case "freeform":
                    return resourceModel.getProperty("script.parameter.type.simple.txt");
                case "fixedlist":
                    return resourceModel.getProperty("script.parameter.type.list.txt");
                case "sascode":
                    return resourceModel.getProperty("script.parameter.type.sasCode.txt");
                case "systemobject":
                    if (objectType&&(objectType==='Project'||objectType==='ModelGroup'||objectType==='SheetTemplate'||objectType==='Scenario')) {
                        var objectTypeResourceString = "scriptParameter.object.types." + objectType.toLowerCase() + "s.txt";
                        return resourceModel.getProperty("script.parameter.type.object.txt") + " (" + resourceModel.getProperty(objectTypeResourceString) + ")";
                    }
                    return "";
                case "serverpath":
                    return resourceModel.getProperty("script.parameter.type.file.txt");
            }
        }
        return "";
    },
    getSelectedDefaultValueItem: function(){
        return this._defaultValuesSelect.getSelectedItem();
    },
    _showRunValue:function(lastValue,defValue){
        return lastValue?lastValue:defValue;
    }
});