jQuery.sap.require({ modName: "sas.stress.wkbench.view.Workbench", type: "controller" });
jQuery.sap.require("sas.hc.ui.core.ListItem");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Page");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.m.FlexItemData");
jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayout");
jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayoutArea");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.stress.notification.NotificationUtil");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.UserPrefs");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");
jQuery.sap.require("sas.stress.wkbench.widget.TaskActionButtons");


sap.ui.jsview("sas.stress.wkbench.view.Workbench", {
    getControllerName: function () {
        return "sas.stress.wkbench.view.Workbench";
    },

    createContent: function (controller) {
        this.setHeight("100%");
        this.sheetSelect = this._createSheetSelect();
        this.iterationSelect = this._createIterationSelect();
        this.sheetCategorySelect = this._createCategoriesSelect();

        this.tokenizer = new sas.hc.ui.commons.facetFilter.FacetTokenizer({
            showCount: false,
            groupEnabled:false  // S1322329
            //groupTokens:100 // S1322329.. we do not want the drop down
        }).addStyleClass("filterTokenizer");

        // workaround fixes for S1322329 due to commons bug?
        //this.tokenizer.getAggregation("_tokenizer").setGroupEnabled(false);

        this.contentLayout = new sas.hc.ui.commons.layout.BorderLayout({ width: "100%", height: "100%" });
        this.contentLayout.setAreaData(sap.ui.commons.layout.BorderLayoutAreaTypes.top, {size: "2.75rem"});
        this.topLayout = new sas.hc.ui.commons.layout.BorderLayout({ height: "100%", width: "100%" });
        this.centerLayout = new sas.hc.ui.commons.layout.BorderLayout({
            height: "100%",
            width: "100%"
        }).addStyleClass("stressScenarioEditorLayout");
        this.contentLayout.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.top, this.topLayout);
        this.contentLayout.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, this.centerLayout);
        this.contentLayout.getArea(sap.ui.commons.layout.BorderLayoutAreaTypes.center);

        var worksheetLayout = new sas.hc.ui.commons.layout.BorderLayout({ width: "100%", height: "100%" });
        var headerArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            size: "2.75rem",
            content: [this._createFilterToolbar(controller).addStyleClass("worksheetFilterToolbar")]
        }).addStyleClass("stresssheetscenario");

        var sheetArea = new sas.hc.ui.commons.layout.BorderLayoutArea({ content: [this.contentLayout] }).addStyleClass("stresssheetscenario");
        worksheetLayout.setTop(headerArea);
        worksheetLayout.setCenter(sheetArea);
        var iterationLabel = new sas.hc.m.Label({ text: sas.stress.wkbench.StressUtil.getBundle().getText("workbench.toolbar.iteration.label") }).addStyleClass("workbenchToolbarLabel");
        iterationLabel.setLabelFor(this.iterationSelect);

        this.viewLayout = new sas.risk.common.widgets.RiskPage({
            mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel(
                {
                    type: sas.risk.common.widgets.WidgetFactory.OBJECT,
                    top: new sas.risk.common.widgets.RiskToolbar({
                        homeFunction: function () {
                            sas.stress.wkbench.ViewManager.loadProjectManager(true);
                        },
                        title: new sas.hc.m.Label({text: "{/name}"}),
                        headerContent: [
                            this._createTaskActionButtons(controller),
                            //new sas.hc.m.Button({icon: sas.icons.HC.BLANK, enabled: false}).addStyleClass("riskToolbarDivider"),
                            new sas.hc.m.ToolbarSeparator(),    // HC 7.0 updates
                            this._createLogButton(controller),
                            this._createRefreshButton(controller),
                            this._createSaveButton(controller),

                            this._createOverflowMenuButton(controller),

                            //new sas.hc.m.Button({icon: sas.icons.HC.BLANK, enabled: false}).addStyleClass("riskToolbarDivider"),
                            new sas.hc.m.ToolbarSeparator(),    // HC 7.0 updates
                            iterationLabel.addStyleClass("sapUiTinyMarginTop"),
                            this.iterationSelect,
                            new sas.hc.m.ToolbarSpacer({ width: ".875rem" }),   // create some space
                            //new sas.hc.m.Button({icon: sas.icons.HC.BLANK, enabled: false}).addStyleClass("riskToolbarDivider"),
                            this._createAsOfDateLayout(controller),
                            sas.stress.notification.NotificationUtil.createNotificationButton()
                        ]
                    }),
                    content: new sas.risk.common.widgets.RiskPage({
                        mainContent: worksheetLayout,
                        iconBar: new sas.stress.wkbench.widget.StressIconBar({
                            section: sas.stress.wkbench.widget.StressIconBar.WORKSHEET
                        })
                    })
                }
            )
        });

        return this.viewLayout;

    },

    _createFilterToolbar: function (controller) {
        var categoryLabel = new sas.hc.m.Label({ text: sas.stress.wkbench.StressUtil.getBundle().getText("workbench.toolbar.sheet.category.label") });
        categoryLabel.setLabelFor(this.sheetCategorySelect);
        var currentWorksheetLabel = new sas.hc.m.Label({ text: sas.stress.wkbench.StressUtil.getBundle().getText("workbench.toolbar.currentWorksheet.label") });
        currentWorksheetLabel.setLabelFor(this.sheetSelect);

        this.addRemoveSheetTemplatesButton = this._createAddRemoveSheetTemplatesButton(controller);

        return new sas.hc.m.Toolbar({
            content: [
                categoryLabel,
                this.sheetCategorySelect,
                new sas.hc.m.ToolbarSpacer({ width: "1rem" }),   // create some space
                currentWorksheetLabel,
                this.sheetSelect,
                this.addRemoveSheetTemplatesButton,
                this.tokenizer
            ]
        });
    },
    _createAddRemoveSheetTemplatesButton: function (controller){
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        var addRemoveButton=new sas.hc.m.Button({
            icon: sas.icons.HC.CROSSTAB,
            tooltip: sas.stress.wkbench.StressUtil.getBundle().getText("worksheet.edit.sheet.templates.tooltip.txt"),
            press: $.proxy(controller._addRemoveSheetTemplatesPress, controller),
            enabled: {
                path: "live>/toggleButtons",
                formatter: function (enabled) {
                    if (!controller.project.isLocked()){
                        return enabled ? false : true;
                    }
                    return false;

                }
            },
            visible: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.UPDATE)
        }).addStyleClass("compositeSelectButton");
        return addRemoveButton;
    }/*,

    _createCommentButton: function (controller) {
        this.commentButton=new sas.hc.m.Button({
            icon: sas.icons.HC.COMMENT,
            tooltip: sas.stress.wkbench.StressUtil.getBundle().getText("workbench.comment.button.tip"),
            press: $.proxy(controller.openCellCommentPopup, controller),
            enabled: {
                path: 'live>/selected',
                formatter: function (cellRef) {
                    //Disable button if iteration != current (max) iteration.
                    return !!cellRef && cellRef.getInfoData().isEditable(cellRef.row, cellRef.column, true);
                }
            }
        });
        return this.commentButton;
    }*/,
    _createOverflowMenuButton:function(controller){
        var rb=sas.stress.wkbench.StressUtil.getBundle();
        var overflowMenu = new sas.hc.m.Menu({
            items: [
                this._createImportButton(controller,rb),
                this._createExportButton(controller,rb),
                this._createExportAllButton(controller,rb)
            ]
        });

        var overflowButton = new sas.hc.m.MenuButton({
            icon: sas.icons.HC.OVERFLOW,
            menu: overflowMenu
        });

        return overflowButton;
    },
    _createLogButton: function (controller) {
        return new sas.hc.m.Button({
            id: "sas.stress.wkbench.view.infoButton",
            icon: sas.icons.HC.LOG,
            enabled: {
                path : "/sheets",
                formatter: function(sheets){
                    return sheets?sheets.length>0:false;
                }
            },
            press: $.proxy(controller.pressLogButton, controller)
        });
    },
    _createExportButton: function (controller,rb) {
        rb=rb||sas.stress.wkbench.StressUtil.getBundle();
        return new sas.hc.m.MenuItem({
            id: "sas.stress.wkbench.view.exportButton",
            //icon: sas.icons.HC.DOWNLOAD,
            text:rb.getText("workbench.export.button.txt"),
            enabled: {
                path: "live>/toggleButtons",
                formatter: function (enabled) {
                    return enabled ? false : true;
                }
            },
            press: $.proxy(controller.pressExportButton, controller)
        });
    },
    _createImportButton: function (controller,rb) {
        rb=rb||sas.stress.wkbench.StressUtil.getBundle();
        return new sas.hc.m.MenuItem({
            id: "sas.stress.wkbench.view.importButton",
            //icon: sas.icons.HC.IMPORT,
            text:rb.getText("workbench.import.button.txt"),
            enabled: {
                path: "live>/iterationNumber",
                formatter: function (preferredProjectIterationNumber) {
                    //Enable button only if iteration = current (max) iteration.
                    return (controller.project !== undefined && controller.project.getCurrentIterationNo() === Number(preferredProjectIterationNumber) &&
                            !controller.project.isLocked());
                }
            },
            press: $.proxy(controller.pressImportButton, controller)
        });
    },
    _createExportAllButton: function (controller,rb) {
        rb=rb||sas.stress.wkbench.StressUtil.getBundle();
        return new sas.hc.m.MenuItem({
            id: "sas.stress.wkbench.view.exportAllButton",
            //icon: sas.icons.HC.DOWNLOAD,
            text:rb.getText("workbench.exportAll.button.txt"),
            enabled: {
                path: "live>/toggleButtons",
                formatter: function (enabled) {
                    return enabled ? false : true;
                }
            },
            press: $.proxy(controller.pressExportAllButton, controller)
        });
    },
    _createSaveButton: function (controller) {
        return new sas.hc.m.Button({
            id: "sas.stress.wkbench.view.saveButton",
            icon: sas.icons.HC.SAVE,
            enabled: {
                path: "live>/toggleButtons",
                formatter: function (enabled) {
                    return enabled ? true : false;
                }
            },
            press: $.proxy(controller._onProjectSave, controller)
        });
    },
    _createRefreshButton: function (controller) {
        return new sas.hc.m.Button({
            id: "Refresh",
            icon: sas.icons.HC.REFRESH,
            enabled: {
                path : "/sheets",
                formatter: function(sheets){
                    return sheets?sheets.length>0:false;
                }
            },
            press: $.proxy(controller.pressRefreshButton, controller)
        });
    }/*,
    _createDrillDownButton: function(controller){
        return new sas.hc.m.Button({
            id: "Drilldown",
            icon: sas.icons.HC.DRILLDOWNARROW,
            enabled: {
                path : "live>/selected",
                formatter: function(selected){
                    if (selected){
                        var key="$"+selected.colLong+"$"+selected.rowLong;
                        var dataKey=selected.getSheetFilters().getSheetInfo().getData().getDatakeyFromRowColumn(key);
                        var drillDownOverlay=selected.getSheetFilters().getData().cDrillDowns[key];
                        if (drillDownOverlay&&(drillDownOverlay.propertyKey>0||drillDownOverlay.inheritedPropertyKey)){
                            return true;
                        }else{
                            return false;
                        }
                    }
                    return selected?true:false;
                }
            },
            press: $.proxy(controller.pressDrillDownButton,controller)
        });
    }*/,
    _createIterationSelect: function () {
        var dd = new sas.hc.m.Select({
            //displaySecondaryValues: true,      // this will make sure the width is the same as the select width
            //autoAdjustWidth: true      //deprecated since HC5.0
            maxWidth:"100%",
            enabled: {
                path : "/sheets",
                formatter: function(sheets){
                    return sheets?sheets.length>0:false;
                }
            }
        });
        dd.attachChange($.proxy(this.getController().changeIteration, this.getController()));
        var itemTemplate = new sas.hc.ui.core.ListItem({
            text: "{text}",
            key: "{key}"
        });
        dd.bindAggregation("items", "/iterations", itemTemplate);
        return dd;
    },
    _createSheetSelect: function () {
        var dd = new sas.hc.m.Select({
            displayListItemIcon: true,
            maxWidth:"100%"
        }).addStyleClass("compositeSelect");
        var itemTemplate = new sas.hc.ui.core.ListItem({
            text: "{name}",
            key: "{key}",
            icon:"{icon}"
        });
        dd.bindAggregation("items","/sheets",itemTemplate);
        dd.attachChange($.proxy(this.getController().openSheetNew, this.getController()));
        return dd;
    },
    _createCategoriesSelect: function (project) {
        var dd = new sas.hc.m.Select({
            displaySecondaryValues: true,
            //autoAdjustWidth: true     //deprecated since HC5.0
            maxWidth:"100%"
        });
        dd.attachChange($.proxy(this.getController().changeSheetCategory, this.getController()));
        var itemTemplate = new sas.hc.ui.core.ListItem({
            text: "{name}",
            key: "{key}",
            additionalText: {
                path: "count",
                formatter: function (count) {
                    return "(" + count + ")";
                }
            }
        });
        dd.bindAggregation("items", "/categories", itemTemplate);
        return dd;
    },

    _createTaskActionButtons:function(controller){
        var rb=sas.stress.wkbench.StressUtil.getBundle();
        this.actionButton = new sas.hc.m.MenuButton({
            id: "workbench-action-button",
            text: rb.getText("home.tasks.details.actions.txt"),
            tooltip: rb.getText("home.tasks.details.actions.txt"),
            visible: {
                path: '/actions',
                formatter: $.proxy(controller._addTaskActions, controller)
            },
            enabled:{   // S1388587
                parts:["/isProjectLocked","live>/toggleButtons"],
                formatter:function(isLocked, isDirty){
                    if (isLocked){
                        return false;    // if project locked button disabled
                    }else if (isDirty){
                        return false;    // if project dirty button disabled
                    }
                    return true;
                }
            }
        });
        this.actions = new sas.hc.m.Menu({});
        this.actionButton.setMenu(this.actions);
        return this.actionButton;
    },

    _createAsOfDateLayout:function(controller) {
        var dateFormat = new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}},
            {target: {pattern: "MMM dd, yyyy"}});
        //var fmt = sap.ui.core.format.DateFormat.getDateTimeInstance();
        //return fmt.format(new Date(dttm),false);
        //new sap.ui.model.type.Date({source: {pattern: "timestamp"}, style: "medium"})
        this.asOfDateLabel = new sas.hc.m.Label({
            text: {
                path: "/asOfDate",
                formatter: function (asofDate) {
                    return dateFormat.formatValue(asofDate, "string");
                }
            }
        })/*.addStyleClass("toolbarLabel")*/;

        var labelLayout = new sas.hc.ui.commons.layout.MatrixLayout({
            layoutFixed : false
        });

        labelLayout.createRow(
            new sas.hc.m.Label({text: sas.stress.wkbench.StressUtil.getBundle().getText("workbench.toolbar.asOfDate.label")}).addStyleClass("toolbarLabel"),
            this.asOfDateLabel.addStyleClass("toolbarValue")
        );

        return labelLayout.addStyleClass("sapUiTinyMarginTop");
    },

    getSelectedSheetId: function () {
        return this.sheetSelect.getSelectedKey();
    },
    getSelectedIterationId: function () {
        return this.iterationSelect.getSelectedKey();
    },
    getSelectedSheetCategoryKey: function () {
        return this.sheetCategorySelect.getSelectedKey();
    },
    getDefaultSheetId: function (sheets) {
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var defSheetId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet_key", project.getId());
        if (project.getSheet(defSheetId)===undefined){  // make sure it exists
            defSheetId=sheets[0].id;    // else, use the first one.. and let the logic take it's course
        }
        if (defSheetId &&
            _.some(sheets, function (sheet) {
                return sheet.id === defSheetId && !sheet.errored;
            })) {
            return defSheetId;
        }
        var firstNonErroredSheet= _.find(sheets,function(sheet){
            return sheet.errored===false;
        });
        return firstNonErroredSheet?firstNonErroredSheet.id:sheets[0].id;
    },

    getDefaultFilterId: function (sheetId) {
        if (sheetId) {
            return "sheet_" + sheetId + ".filters";
        } else {
            return "sheet_" + this.getSelectedSheetId() + ".filters";
        }
    },

    selectSheet: function (sheetId) {
        this.sheetSelect.setSelectedKey(sheetId);
        this.getController().openSheetNew();
    },
    setCellEditorView: function (view) {
        this.topLayout.removeAllContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center);
        this.topLayout.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, view);
    },
    setScenarioEditorView: function (view) {
        this.centerLayout.removeAllContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center);
        this.centerLayout.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, view);
        this.scenarioEditorView = view;
    },
    getScenarioEditorView: function () {
        return this.scenarioEditorView;
    },

    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    },
    closeView: function () {
        //Needs to be fixed to go back to default tab entry Configuration page
        // Setting it to configuration page results in error of dup id for ConfigurationView
        // calling the destroy function results in a blank page
        // this.getController().router.setView("sas.stress.config.view.Configuration",null);
        // sap.ui.core.mvc.JSView.prototype.destroy.apply(this,arguments);
    },
    setNoData: function(controller) {
        this.centerLayout.removeAllContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center);
        this.centerLayout.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, new sas.risk.common.widgets.ZeroStateLayout({
            icon: sas.icons.HC.ZEROSTATEADD,
            text: sas.stress.wkbench.StressUtil.getBundle().getText("worksheet.templates.no.data.txt"),
            buttons: [
                new sas.hc.m.Button({
                    text: sas.stress.wkbench.StressUtil.getBundle().getText("worksheet.templates.no.data.button.txt"),
                    press: $.proxy(controller._addRemoveSheetTemplatesPress, controller)
                }).addStyleClass("sasPrimaryButton")
            ]
        }));
    }
});