jQuery.sap.declare("sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter");

jQuery.sap.require("sas.risk.common.widgets.RiskTable");
jQuery.sap.require("sas.stress.wkbench.widget.CategoryFilter");

sas.hc.m.FlexBox.extend("sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter", {

	metadata:{
        properties:{
            // list of {name, selected}
            sheetTemplates: { type: "string", defaultValue: "/worksheets", bindable: true},
            //
            categories: {type: "string", defaultValue: "/categories", bindable: true},
            //
            selectedCategoryName: { type: "string", defaultValue: "", bindable: true},
            selectedCategoryItem: {type: "any", defaultValue: null, bindable: true},

            /*currentWorksheetName: { type: "string", defaultValue: "", bindable: true},
            currentWorksheetItem: {type: "any", defaultValue: null, bindable: true},*/

            "columnDetails": {type: "any"},

            inputTemplates: { type: "any[]", defaultValue: "[]", bindable: true},

            readyCallback: {type: "any"},
            selectedSheetTemplates: {type: "any[]", defaultValue: null},

            title: { type: "string", defaultValue: "", bindable: true}
        },
        /*aggregations: {
            "selectTable": { type: "sap.ui.core.Control", multiple: false, visibility: "hidden" }
        },*/
        publicMethods: ["getSelectedSheetTemplates", "getSelectedSheetTemplate"],
        events:{
            selectionChange: {},
            categoryChange: {}
        }
    },

    /*setSheetTemplates: function (sheetTemplates) {
        if (sheetTemplates) {
            this.setProperty("sheetTemplates", sheetTemplates, true);
        }
    },

    setCategories: function (categories){
        if (categories){
            this.setProperty("categories", categories, true);
        }
    },*/

    setInputTemplates: function (input){
        if (input){
            this.setProperty("inputTemplates", input, true);
        }
    },

    getSelectedSheetTemplates: function () {
        return this._selectedTemplates;
    },

    getSelectedSheetTemplate: function () {
        return this._selectedTemplates?this._selectedTemplates[0]: undefined;
    },

    setSelectedSheetTemplates: function (selectedTemplates){
        this._selectedTemplates = selectedTemplates;
    },

    getSelectedCategoryKey: function (categoryName) {
        //var catName = this.getCategoryName();
        categoryName=categoryName?categoryName:this._categoryFilter.getSelectedCategory();
        var data = this._categoryFilter.getListItems();

        var selectedCategoryItem = _.find(data, function (categoryItem) {
            return (categoryName === categoryItem.getProperty("title"));
        });
        return selectedCategoryItem?Number(selectedCategoryItem.getProperty("additionalText")): 0;
    },

    renderer : "sas.hc.m.FlexBoxRenderer",

    rb: sas.stress.wkbench.StressUtil.getBundle()
});

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype.init = function () {
    this.setWidth("100%");
    this.setHeight("100%");
    this.setRenderType(sap.m.FlexRendertype.Bare);

    this.addItem(this.createMainContainer());

    this._addColumns();
    this._selectTable.bindRows(this.getSheetTemplates());
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype.applySettings = function () {
    if (sas.hc.m.FlexBox.prototype.applySettings) {
        sas.hc.m.FlexBox.prototype.applySettings.apply(this, arguments);
    }
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype.onBeforeRendering = function(){
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype.onAfterRendering = function(){
    var readyCallBack = this.getReadyCallback();
    if(readyCallBack){
        readyCallBack(this._selectTable, this._categoryFilter);
    }
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype.createMainContainer = function(){
    this._selectTable = this._createSelectTable();
    this._categoryFilter = this._createCategoryFilter();
    this._searchField;

    var categoriesPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
        topPadding: 0,
        leftPadding: "0.5rem",
        content: this._categoryFilter
    }).addStyleClass("riskFacetPanel");

    categoriesPanel.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({
        size: sas.stress.wkbench.widget.CategoryFilter.DEFAULT_WIDTH,
        minSize: 150
    }));

    var panel = sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({
        content: new sas.hc.ui.layout.Splitter({
            contentAreas: [
                sas.risk.common.widgets.WidgetFactory.buildPanel({
                    rightPadding: "0.5rem",
                    content: this._selectTable
                }),
                categoriesPanel
            ],
            orientation: sap.ui.core.Orientation.Horizontal,
            width: "100%",
            height: "100%"
        })
    });

    return panel;
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype._createCategoryFilter = function () {
    var categoryFilter =  new sas.stress.wkbench.widget.CategoryFilter({
        title: this.rb.getText("categoryFilter.header.title"),
        itemsPath: this.getCategories(), //"/categories",
        itemTemplate: new sas.hc.m.StandardListItem({
            title: "{name}",
            counter: "{count}",
            additionalText: "{key}",
            displaySecondaryValues: false
        }),
        selectionChange: $.proxy(this._categoriesListSelectionChange, this)
    });

    return categoryFilter;
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype._createSelectTable = function () {
    var selectTable = new sas.risk.common.widgets.RiskTable({
        title: this.rb.getText("project.worksheets.label"),
        visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
        selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
        selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowSelector,
        toolbar: this._createTableToolbar(),
        noData: this.rb.getText("project.noData.filter.txt"),

        showSelectionControls: true,
        showTableOptions: false,
        enableColumnFreeze: false,

        minAutoRowCount: 1, // required for scrolling with < 5 rows,
        rowSelectionChange: $.proxy(this._onRowSelectionChange, this)
    });

    return selectTable;
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype._addColumns = function(){
    // S1389396 add names to filterable columns
    this._selectTable.addColumn(new sas.hc.ui.table.Column({
        label: new sas.hc.m.Label({text: this.rb.getText("projectsTable.column.name.header.title")}),
        template: new sas.hc.m.Text({
            text: "{nameVersion}",
            tooltip: "{nameVersion}"
        }),
        sortProperty: "nameVersion",
        width: "15rem"
    }));

    this._selectTable.addColumn(new sas.hc.ui.table.Column({
        label: new sas.hc.m.Label({text: "category"}),
        template: new sas.hc.m.Text({
            text: {
                path: "categoryKey",
                formatter: function(key){
                    return key?key.toString():"";
                }
            }
        }),
        visible: false
    }));
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype._createTableToolbar= function () {
    return new sas.hc.m.ResponsiveToolbar({
        content: [
            this._createSearchField()
        ]
    });
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype._createSearchField= function () {
    return this._searchField = new sas.hc.m.SearchField({
        liveChange: $.proxy(this._filterSheetTemplates, this),
        filterMode: true
    });
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype._onRowSelectionChange = function (event) {
    var table = event.getSource();
    var model = table.getBinding("rows").oModel;
    var selectedContexts = table.getSelectedContextPaths();
    var selectedTemplates = [];
    _.each(selectedContexts, function(contextPath){
        selectedTemplates.push(model.getProperty(contextPath));
    });
    /*var outputTemplatesPath = this.getOutputTemplatesPath();
    model.setProperty(outputTemplatesPath, selectedTemplates);*/
    this.setSelectedSheetTemplates(selectedTemplates);

    this.fireSelectionChange({selectedItems: selectedTemplates, selectedItemsLength: selectedTemplates.length});
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype._categoriesListSelectionChange = function (event) {
    var source=event.getSource();
    var categoryName = source.getSelectedCategory(); //this.getCategoryName(); // this._categoryFilter.getSelectedCategory();
    var categoryKey = this.getSelectedCategoryKey(categoryName);
    var binding = this._selectTable.getBinding("rows");

    switch (categoryKey){
        case 0:
            binding.filter(new sap.ui.model.Filter("categoryKey", sap.ui.model.FilterOperator.NE, ""));
            break;

        default:
            binding.filter(new sap.ui.model.Filter("categoryKey", sap.ui.model.FilterOperator.EQ, categoryKey));
            break;
    }
};

sas.stress.wkbench.widget.SelectSheetTemplatesWithCategoryFilter.prototype._filterSheetTemplates = function(){
    var search = this._searchField.getValue() || "";

    var i18NModel=this.getView().getModel("i18N");
    var table = this.getView().table;
    var filters = [];
    filters.push(new sap.ui.model.Filter("nameVersion", sap.ui.model.FilterOperator.Contains, search));

    var orFilters = new sap.ui.model.Filter(filters); // the filters on the visible columns OR'd
    var binding = table.getBinding("rows");

    switch (this._getSelectedCategoryKey()) {
        case 0: // All category will not have the filter for category name
            binding.filter(orFilters);
            break;
        default:
        {
            var categoryFilter = new sap.ui.model.Filter("category", sap.ui.model.FilterOperator.EQ, this.getSelectedCategoryKey());
            var andFilters = new sap.ui.model.Filter([orFilters, categoryFilter], true); // this is the addtional filter for the category name
            binding.filter(andFilters);
        }
    }
};