jQuery.sap.declare("sas.stress.wkbench.widget.SlimsheetSource");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.model.TreeNode");
jQuery.sap.require("sas.stress.wkbench.ViewManager");

sas.stress.wkbench.widget.SlimsheetSource = function (sheetId, filterId/*scenarioId*/) {
    this.sheetId = sheetId;
    this.filterId/*scenarioId*/ = filterId;
    this.widths = null;
    this.info = undefined;
    this.selectedCol=0;
    this.selectedRow=0;

    this.cellRange = [];
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.init = function () {
    var project = sas.stress.wkbench.model.Project.getCurrentProject();
    var sheet = project.getSheet(this.sheetId);
    if (sheet){
        this.sheetName=sheet.getName();
        var info = sheet.getInfo();
        this.sheetFilters = info.getFilters();
        this.data = this.sheetFilters.getData();
        this.infoData = info.getData();
        this.projectCreatedTime = project.getCreatedTime();
    }
    this.callOuts=[];   // S1344081
};
/*sas.stress.wkbench.widget.SlimsheetSource.prototype.getCellScenario=function(row,col){
 return this.scenario;
 };*/
sas.stress.wkbench.widget.SlimsheetSource.prototype.getCellFilter = function (row, col) {
    return this.sheetFilters;
};
/**
 * Common method for filling an array with the node.
 *
 * @param node
 * @param previousSpan
 * @param data
 * @returns {previousSpan}
 */
function fillArray(node, previousSpan, data) {
    var span = previousSpan + (node.isNodeExpanded()? node.getSpan(): 1);
    data.nodes.fill(node, previousSpan, span);
    previousSpan = span;
    return previousSpan;
}
sas.stress.wkbench.widget.SlimsheetSource.prototype.getInfo = function () {
    if (this.info){
        return this.info;
    }
    if (!this.infoData){
        return undefined;
    }
    var that=this;
    var i, col, idx, j, data, nodes, node;
    var reAdjustWidth=true;
    var columns = this.infoData.columns;
    var headerCount = (this.infoData.columnHeadersCount > 0) ? this.infoData.columnHeadersCount + 1 : 2;
    var rowHierarchyCount = (this.infoData.rowHierarchiesCount > 0) ? this.infoData.rowHierarchiesCount + 1 : 2;

    // we are coming in again... check the local storage for widths
    var stored=this.getFromLocalStorage()||null;
    if (stored){
        if(stored.fixed.widths.length===rowHierarchyCount&&stored.scrolled.widths.length===columns.length){ // something needs to match correctly
            this.widths=[];
            this.widths[0]=stored.fixed.widths;
            this.widths[1]=stored.scrolled.widths;

            // let's not adjust because the user has selected a wdth that he likes for this sheet fixes for S1308666
            reAdjustWidth=false;
        }else{
            // delete the user preference... it has wrong data
            var project=this.sheetFilters.getSheetInfo().getSheet().getProject();
            var sheetDefn= _.find(project.getSheetDefinitions(),function(sheetDefinition){
                return sheetDefinition.sheetTemplateName.toLowerCase()===that.sheetName.toLowerCase();
            });
            if (sheetDefn) {
                sas.stress.wkbench.UserPrefs.delete("sas.stress.wkbench.sheet.template." + sheetDefn.sheetTemplateKey + ".columns.widths", project.getId());
            }
        }
    }

    var eachColumnWidth=24;
    var lastColumnWidth=(rowHierarchyCount-1)*(eachColumnWidth+6)+200;
    if (!this.widths) {//default column widths
        this.widths = [];
        if (rowHierarchyCount === 2) {
            this.widths[0] = [50, lastColumnWidth];   // there can be this case and therefore, the width is now reduced to a minimum
        } else {
            this.widths[0]=new Array(rowHierarchyCount);    // S1348658
            this.widths[0][0]=Math.max(that.getWidthOfText(this.getRowCount().toString())+8, 32); // the row number
            this.widths[0].fill(eachColumnWidth,1,rowHierarchyCount-1); // each of the hierarchies that are parents

            this.widths[0][rowHierarchyCount-1]=(lastColumnWidth);	// fixes for S1230285
        }
        this.widths[1] = new Array(columns.length);
        this.widths[1].fill(132);
    }
    var fixedWidth = 0, scrolledWidth = 132*(columns.length), labelName = new Array(headerCount), rowLabel = new Array(rowHierarchyCount);
    for (idx = 0; idx < headerCount; idx++) {
        labelName[idx] = "label_" + idx;
    }
    for (idx = 0; idx < rowHierarchyCount; idx++) {
        rowLabel[idx] = "label_" + idx;
    }

    if(stored) {
        scrolledWidth=0;
        for (i = 0; i < this.widths[1].length; i++) {
            scrolledWidth += this.widths[1][i];
        }
    }
    this.info={
        rowCount: this.getRowCount(),
        headerCount: headerCount,
        groupCount: 1,
        fixed: {
            widths: this.widths[0],
            width: fixedWidth,
            colCount: rowHierarchyCount,
            rowCount: this.infoData.rowLength
        },
        scrolled: {
            widths: this.widths[1],
            width: scrolledWidth,
            colCount: columns.length/*+1*/
        },
        collapsedNodes: {}
    };
    var columnHeadersMap, rowHeadersMap;
    if (this.infoData._getColumnHeadersMap()) {
        columnHeadersMap = this.infoData._getColumnHeadersMap();
    }
    if (this.infoData._getRowHeadersMap()) {
        rowHeadersMap = this.infoData._getRowHeadersMap();
    }

    for (idx = 0; idx < headerCount; idx++) {
        this.info.scrolled[labelName[idx]]=columnHeadersMap?[]:[""];
    }
    for (idx = 0; idx < rowHierarchyCount; idx++) {
        this.info.fixed[rowLabel[idx]]=rowHeadersMap?undefined:["",""];
        if (idx === 0 && rowHeadersMap) {
            this.info.fixed[rowLabel[idx]]=[""];
        }
    }

//var t0 = performance.now();
    var previousSpan=0;
    for (i = 0; i < columns.length; i++) {
        col = columns[i];
        j = 1;
        this.info.scrolled[labelName[0]].push(col.short);
        while (j < headerCount/*-1*/) {
            if (columnHeadersMap) {
                previousSpan=0; // re-initialize
                data = { nodes: new Array(columns.length), label: new Array(columns.length) };
                nodes = columnHeadersMap["headerrow_" + (j - 1)] || [];
                if (j === (headerCount - 1)){
                    data.nodes=nodes;
                }else {
                    for (idx = 0; idx < nodes.length; ++idx) {
                        node = nodes[idx];
                        previousSpan = fillArray(node, previousSpan, data);
                    }
                }
                this.info.scrolled[labelName[j]]=data;
            } else {
                this.info.scrolled[labelName[j]].push(col?col.long:"");
            }
            j++;
        }
    }
    /*var t1 = performance.now();
     console.log("Call to process column hierarchy took " + (t1 - t0) + " milliseconds.");*/
    j = 1;
    previousSpan=0;
    var prevWidth=0;
    if (reAdjustWidth) {
        var sortedNodes=_.sortBy(rowHeadersMap["headerrow_"+(rowHierarchyCount-2)],function(node){
            return -(node.getName().length);
        });
        var longestText=sortedNodes[0].getName()||"";
        prevWidth = that.getWidthOfText(longestText)+4; // should not hard-coded font info.. done
    }
//t0 = performance.now();
    while (j < rowHierarchyCount) {
        data = {nodes: [], label: []};
        if (rowHeadersMap) {
            data = { nodes: new Array(this.infoData.rowLength), label: new Array(this.infoData.rowLength) };
            nodes = rowHeadersMap["headerrow_" + (j - 1)] || [];
            previousSpan=0; // re-initialize
            if (nodes) {
                if (j===(rowHierarchyCount-1)){
                    data.nodes=rowHeadersMap["headerrow_" + (j - 1)];
                }else {
                    for (idx = 0; idx < nodes.length; ++idx) {
                        node = nodes[idx];
                        previousSpan = fillArray(node, previousSpan, data);
                    }
                }
                this.info.fixed[rowLabel[j]] = data;
            } else {
                this.info.fixed[rowLabel[j]] = data;
            }
        } else {
            this.info.scrolled[rowLabel[j]].push("");
        }
        j++;    // next
    }
//t1 = performance.now();
//console.log("Call to process row hierarchy took " + (t1 - t0) + " milliseconds.");
    if (reAdjustWidth||prevWidth>0) {   // fixes for S1308666
        if (rowHierarchyCount>3){
            this.info.fixed.widths[j - 1] = Math.max(Math.min(this.info.fixed.widths[j - 1], prevWidth), 120);
        }else{
        	// https://sirius.na.sas.com/Sirius/ShowTrack.aspx?trknum=7612765799
        	// S1508683
            this.info.fixed.widths[j - 1] = Math.max(Math.min(this.info.fixed.widths[j - 1], prevWidth),120); // S1409388
        }
    }
    fixedWidth = this.widths[0][0] + this.widths[0][1];
    if (rowHierarchyCount > 2) {
        fixedWidth = 0;
        for (idx = 0; idx < rowHierarchyCount; idx++) {
            fixedWidth += this.widths[0][idx];
        }
    }
    this.info.fixed.width = fixedWidth;

    return this.info;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype._createBalancedTree = function(node, j, rowHierarchyCount, dataLabel, nodesLength, prefix) {
    var children = node.getChildren();
    var childrenLength = children.length;
    var previousSpan = 0;
    var isScrolled = prefix === "scrolled"? true: false;
    for (var childIdx = 0; childIdx < childrenLength; ++childIdx) {
        var node = children[childIdx],
            data = isScrolled ? this.info.scrolled[dataLabel[j]] : this.info.fixed[dataLabel[j]],
            emptyIndex = 0,
            colData;

        if (node.getSubtotalLikey() !== undefined) {
            for (var column = j; column < rowHierarchyCount; column++) {
                colData = isScrolled ? this.info.scrolled[dataLabel[column]] : this.info.fixed[dataLabel[column]];

                _.find(colData.nodes, function(data, idx){
                    if (column === j) { // the idx for the current rowLabel index
                        emptyIndex = idx;
                    }
                    return data === undefined;
                });
                colData.nodes[emptyIndex] = (node);
                //colData.label[rowIdx] = node.getName();
            }
            previousSpan = emptyIndex + 1;
            continue;
        } else {
            if (isScrolled && /*node.getChildren().length === 0 &&*/ previousSpan === 0){
                colData = this.info.scrolled[dataLabel[j]];
                _.find(colData.nodes, function(data, idx){
                    emptyIndex = idx;
                    return data === undefined;
                });
                previousSpan = emptyIndex ;//=== 0 ? emptyIndex : emptyIndex + 1;
            }
        }
        previousSpan = fillArray(node, previousSpan, data);

        // if this a child because of ragged hierarchy and child is not at the end.. we need to create emptyNodes
        if (node.getChildren().length === 0 && node.getLevel() < (rowHierarchyCount-1)){
            for (var level = node.getLevel() +1; level < rowHierarchyCount; ++level) {
                var emptyNode = this._createEmptyNodesFromGivenNode(node);
                emptyIndex = 0;
                colData = isScrolled ? this.info.scrolled[dataLabel[level]] : this.info.fixed[dataLabel[level]];

                _.find(colData.nodes, function(data, idx){
                    emptyIndex = idx;
                    return data === undefined;
                });
                colData.nodes[emptyIndex] = (emptyNode);
            }
        }
        this._createBalancedTree(node, (j+1), rowHierarchyCount, dataLabel, nodesLength, prefix);
    }
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.getRowCount = function () {
    return this.infoData.rows.length;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getRowHeadersCount = function () {
    return this.infoData.rowHierarchiesCount;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getColumnHeadersCount = function () {
    return this.infoData.columnHeadersCount;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getColumnCount = function () {
    return this.infoData.columns.length;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getFixedRowData = function (rowNum) {
    var rowInfo = this.infoData.rows[rowNum];
    var tab = "";
    for (var i = 0; i < rowInfo.level; i++) {
        tab += "     ";
    }
    return [(rowNum + 1).toString(), tab + rowInfo.name];
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getScrolledRowData = function (rowNum) {
    return this.data.data[rowNum];
};
/**
 * this uses the project to load the data for the datakey...
 */
sas.stress.wkbench.widget.SlimsheetSource.prototype.getScrolledRowColumnValue = function (rowNum,col) {
    //var rowInfo = this.infoData.rows[rowNum];
    var sheetInfoData=this.data.sheetInfo.getData();
    var colLong=sheetInfoData.getLongColumn(col);
    var rowLong=sheetInfoData.getLongRow(rowNum);
    var key="$"+colLong+"$"+rowLong;
    var dataKey=sheetInfoData.getDatakeyFromRowColumn(key);
    var cellValue = sas.stress.wkbench.model.Project.getCurrentProject().getCellValueByDataKey(dataKey);//  this.data.data[rowNum];
    // S1431431.. if text startsWith ' and does not contain a '! it is TEXT do not parse anything
    if (cellValue && (typeof cellValue === "string" &&  (cellValue.startsWith("'") || cellValue.startsWith("='")) && cellValue.indexOf("'!") === -1)) {
        // we need to not display the initial '
        if (cellValue.startsWith("='")) {
            return cellValue.substr(2);
        }
        return cellValue.substr(1);
    }else{
        return cellValue;
    }
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.getCellIsReadonlyOverrideValue = function(row,col){
    var sheetInfoData=this.data.sheetInfo.getData();
    var colLong=sheetInfoData.getLongColumn(col);
    var rowLong=sheetInfoData.getLongRow(row);
    var key="$"+colLong+"$"+rowLong;
    var dataKey=sheetInfoData.getDatakeyFromRowColumn(key);

    var currentProject = sas.stress.wkbench.model.Project.getCurrentProject();
    return currentProject.isCellByDatakeyReadonly(dataKey);
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.getSubtotalCellValues=function(row,col){
    var sheetInfoData=this.data.sheetInfo.getData();
    var colLong=sheetInfoData.getLongColumn(col);
    var rowLong=sheetInfoData.getLongRow(row);
    var key="$"+colLong+"$"+rowLong;
    var dataKey=sheetInfoData.getDatakeyFromRowColumn(key);

    var cellOverlay=sas.stress.wkbench.model.Project.getCurrentProject().getCellOverlayByDataKey(dataKey);
    var values;
    if (cellOverlay&&(cellOverlay.value&&cellOverlay.value.parsedFormula)){
        values=sas.stress.wkbench.parser.ExpressionUtil.evaluate(cellOverlay.value.parsedFormula, $.proxy(this.data.sheetInfo.getFilters().getData().evalContext, this.data.sheetInfo.getFilters().getData()), false);
        var what=Object.prototype.toString;
        if (what.call(values[0]) === "[object Array]" ){
            return values[0];
        } else if (what.call(values) === "[object Array]"){
            return values;
        }
    }
    return values;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.isSelectable = function (row, col) {
    return !isNaN(row) && !isNaN(col) /*&& col!==0*/ && this.infoData.rows[row].long !== null;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.decorateCells = function (control) {
    var i, j, overlay, isOverriden, isFormulaic,prevClass, isSystemGenerated, projectOverlay, isAutoAggregated; //HF3
    var cell;
    var that = this;
    var processed=[];   // declaration missing S1392611.. do not understand why removed in the first place -- SRINI
    var currentProject = sas.stress.wkbench.model.Project.getCurrentProject();

    var collapsedNode,
        rowIndex,
        columnIndex,
        dataKey = "",
        rowLong,
        colLong;

    var currentRowIndex = 0,
        skipRowIndices = [],
        index;
    for (i in this.data.ncFormulas) {
        overlay = this.data.ncFormulas[i];
        collapsedNode = _.find(this.info.collapsedNodes, function(value) {
            return currentRowIndex === value.row;
        });
        if (collapsedNode && skipRowIndices.indexOf(overlay.rowIndex)){
            currentRowIndex = overlay.rowIndex;
            continue;
        }
        rowIndex = overlay.rowIndex;
        columnIndex = overlay.columnIndex;
        dataKey="";
        if (collapsedNode){
            rowLong = this.infoData.getLongRow([collapsedNode.nextRow - 1]);
            colLong = this.infoData.getLongColumn([columnIndex]);
            dataKey = this.infoData.getDatakeyFromRowColumn("$"+colLong+"$"+rowLong);
            overlay = this.data.ncFormulas["$"+colLong+"$"+rowLong];

            for (index = collapsedNode.row + 1; index <= collapsedNode.nextRow; index++){
                skipRowIndices.push(index);
            }
        } else{
            currentRowIndex = overlay.rowIndex;
        }
        projectOverlay=currentProject.getCellOverlayByDataKey(overlay ? overlay.dataKey : dataKey);
        isOverriden = overlay && !overlay.systemGenerated?"SlimOverriden":""; //(!overlay.timestamp || (overlay.timestamp > this.projectCreatedTime)) ? "SlimOverriden" : "";
        isFormulaic = overlay && (overlay.expr.type !== "NAME" || overlay.txt.indexOf("=")===0) ? " SlimFormulaic" : "";
        // HF3
        //isSystemGenerated = overlay.systemGenerated?" slimSystemGenerated":"";
        isSystemGenerated = projectOverlay&&projectOverlay.value?("Subtotal"===projectOverlay.value.systemGenerated||"HierarchyFormula"===projectOverlay.value.systemGenerated?" slimSystemGenerated":""):"";
        isAutoAggregated = projectOverlay&&projectOverlay.value?("Aggregated"===projectOverlay.value.systemGenerated?" slimSystemGenerated slimAutoAggregated":""):"";
        if (isOverriden || isFormulaic || isSystemGenerated || isAutoAggregated) {
            cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, rowIndex, columnIndex/*+1*/);
            if (cell) {
                prevClass = cell.className;
                // HF3
                cell.className = prevClass ? prevClass + " " + isOverriden + isFormulaic + isSystemGenerated + isAutoAggregated : isOverriden + isFormulaic + isSystemGenerated + isAutoAggregated;
                if (isFormulaic) {  // only add if there is a formula
                    that._dynamicallyAugmentCells(cell, "SlimFormulaicDiv");
                }
            }
        }
    }

    currentRowIndex = 0;
    index = 0;
    skipRowIndices = [];
    for (i in this.data.cDrillDowns){
        overlay=this.data.cDrillDowns[i];
        collapsedNode = _.find(this.info.collapsedNodes, function(value) {
            return overlay.rowIndex === value.row;
        });
        if (collapsedNode && skipRowIndices.indexOf(overlay.rowIndex)){
            currentRowIndex = overlay.rowIndex;
            continue;
        }
        rowIndex = overlay.rowIndex;
        columnIndex = overlay.columnIndex;
        dataKey="";
        if (collapsedNode){
            var rowLong = this.infoData.getLongRow([collapsedNode.nextRow - 1]);
            var colLong = this.infoData.getLongColumn([columnIndex]);
            dataKey = this.infoData.getDatakeyFromRowColumn("$"+colLong+"$"+rowLong);
            overlay = this.data.cDrillDowns["$"+colLong+"$"+rowLong];

            for (index = collapsedNode.row + 1; index < collapsedNode.nextRow; index++){
                skipRowIndices.push(index);
            }
        } else{
            currentRowIndex = overlay.rowIndex;
        }
        if (overlay&&overlay.propertyKey) {
            cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, overlay.rowIndex, overlay.columnIndex);
            if (cell) {
                prevClass = cell.className;
                cell.className = prevClass ? prevClass + " SlimDrillDownAble" : "SlimDrillDownAble";
                // we need to create the inner DIV here.. see method (augmentCell) for comments
                that._dynamicallyAugmentCells(cell, "SlimDrillDownAbleDiv");
            }

            // if there are stored widths.. this will occur if the user resized the columns
            var parsedWidths = this.getFromLocalStorage();
            if (parsedWidths) {
                // we will set up the right on the drill-down CSS to make sure the decorator comes at the right place
                $(".SlimDrillDownAble>div.SlimDrillDownAbleDiv").css("right", parsedWidths["scrolled"].widths[overlay.columnIndex] - 14 + "px");
            }
        }
    }

    currentRowIndex = 0;
    skipRowIndices = [];
    for (i in this.data.ncComments) {
        overlay = this.data.ncComments[i];
        collapsedNode = _.find(this.info.collapsedNodes, function(value) {
            return overlay.rowIndex === value.row;
        });
        if (collapsedNode && skipRowIndices.indexOf(overlay.rowIndex)){
            currentRowIndex = overlay.rowIndex;
            continue;
        }
        rowIndex = overlay.rowIndex;
        columnIndex = overlay.columnIndex;
        dataKey="";
        if (collapsedNode){
            var rowLong = this.infoData.getLongRow([collapsedNode.nextRow - 1]);
            var colLong = this.infoData.getLongColumn([columnIndex]);
            dataKey = this.infoData.getDatakeyFromRowColumn("$"+colLong+"$"+rowLong);
            overlay = this.data.ncComments["$"+colLong+"$"+rowLong];

            for (index = collapsedNode.row + 1; index < collapsedNode.nextRow; index++){
                skipRowIndices.push(index);
            }
        } else{
            currentRowIndex = overlay.rowIndex;
        }
        if (overlay) {
            cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, overlay.rowIndex, overlay.columnIndex/*+1*/);
            if (cell) {
                prevClass = cell.className;
                cell.className = prevClass ? prevClass + " SlimCommented" : "SlimCommented";
                // we need to create the inner DIV here.. see method (augmentCell) for comments
                that._dynamicallyAugmentCells(cell, "SlimCommentedDiv");

                var validationoverlaykey = "$" + overlay.columnIndex + "$" + overlay.rowIndex;
                var validationoverlay = this.data.cValidationErrors[validationoverlaykey];
                if (validationoverlay) {
                    prevClass = cell.className;
                    cell.className = prevClass ? prevClass + " SlimValidationError" : "SlimValidationError";
                    // we need to create the inner DIV here.. see method (augment cell) for comments
                    that._dynamicallyAugmentCells(cell, "SlimValidationErrorDiv");
                    processed.push(validationoverlaykey);
                }

                var callout = new sas.hc.m.ResponsivePopover({
                    showHeader: false,
                    verticalScrolling: false,
                    placement: sap.m.PlacementType.Auto,
                    modal: false,
                    offsetY: 2,
                    offsetX: 0,
                    autoClose: true,
                    showSharkfin: true
                });
                callout.addStyleClass("stressCommentPopover");
                this.callOuts.push(callout);
                cell.onmouseenter = (function (overlay, filter, callout, cell) {
                    return function () {
                        var $cell = $(cell);
                        var cellRef = new sas.stress.wkbench.model.CellRef(overlay.columnIndex, overlay.rowIndex, filter/*scenario*/);
                        validationoverlaykey = "$" + cellRef.column + "$" + cellRef.row;
                        validationoverlay = that.data.cValidationErrors[validationoverlaykey];
                        var cellCommentEditor = sas.stress.wkbench.ViewManager.loadCellCommentEditor(cellRef, true, validationoverlay !== undefined);
                        cellCommentEditor.attachClosing(function () {
                            callout.close();
                            if (!!!cellCommentEditor.bIsDestroyed) {
                                callout.removeAllContent(); // remove content
                                callout.setVisible(false);
                                cellCommentEditor.destroy(); // destroy the view rather than keeping it around, causes other issues
                            }
                        }, callout);
                        callout.addContent(cellCommentEditor);  // add the new content
                        callout._currentControl = {
                            $: function () {
                                return $cell;
                            },
                            getDomRef: function () {
                                return cell;
                            },
                            getFocusInfo: function () {
                                return {id: control.id};
                            },
                            applyFocusInfo: function (obj) {
                                $("#" + obj.id).focus();
                            },
                            getTooltip: function () {
                                return callout;
                            }
                        };
                        //return callout.onmouseover.apply(callout, arguments);
                        setTimeout(function () {
                            if (callout.getContent().length > 0 && !!!callout.getContent()[0].bIsDestroyed) {
                                callout.setVisible(true);
                                return callout.openBy(cell);
                            } else {
                                return;
                            }
                        }, 0)
                    };

                })(overlay, this.sheetFilters, callout, cell);
                cell.onmouseleave = (function (callout) {
                    return function (e) {
                        //callout.onmouseout.apply(callout, arguments);
                        console.warn("Cell onmouseout");
                        callout.setVisible(false);
                        var content = callout.getContent()[0];
                        if (content && !!!content.bIsDestroyed) {
                            content.fireEvent(content.getId() + "_CLOSING", {
                                "view": content,
                                "action": "CANCEL"
                            });
                        }
                    };
                })(callout);
            }
        }
    }
    for (i in this.data.cValidationErrors) {
        if (processed.indexOf(i, 0) === -1) {	// not processed
            overlay = this.data.cValidationErrors[i];
            cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, overlay.rowIndex, overlay.columnIndex/*+1*/);
            if (cell) {
                prevClass = cell.className;
                cell.className = prevClass ? prevClass + " SlimValidationError" : "SlimValidationError";
                // we need to create the inner DIV here.. see method (augmentCell) for comments
                that._dynamicallyAugmentCells(cell, "SlimValidationErrorDiv");

                var callOut = new sas.hc.m.ResponsivePopover({
                    showHeader: false,
                    verticalScrolling: false,
                    placement: sap.m.PlacementType.Auto,
                    modal: false,
                    offsetY: 2,
                    offsetX: 0,
                    autoClose:true,
                    showSharkfin:true
                });
                callOut.addStyleClass("stressCommentPopover");
                this.callOuts.push(callOut);
                cell.onmouseover = (function (overlay, filter/*scenario*/, callout, cell) {
                    return function () {
                        var $cell = $(cell);
                        var cellRef = new sas.stress.wkbench.model.CellRef(overlay.columnIndex, overlay.rowIndex, filter/*scenario*/);
                        var cellCommentEditor = sas.stress.wkbench.ViewManager.loadCellCommentEditor(cellRef, false, true);
                        cellCommentEditor.attachClosing(function () {
                            callout.close();
                            if (!!!cellCommentEditor.bIsDestroyed){
                                callout.removeAllContent(); // remove content
                                callout.setVisible(false);
                                cellCommentEditor.destroy(); // destroy the view rather than keeping it around, causes other issues
                            }
                        }, callout);
                        callout.addContent(cellCommentEditor);  // add the new content
                        callout._currentControl = {
                            $: function () {
                                return $cell;
                            },
                            getDomRef: function () {
                                return cell;
                            },
                            getFocusInfo: function () {
                                return { id: control.id };
                            },
                            applyFocusInfo: function (obj) {
                                $("#" + obj.id).focus();
                            },
                            getTooltip: function () {
                                return callout;
                            }
                        };
                        //return callout.onmouseover.apply(callout, arguments);
                        setTimeout(function(){
                            console.info("cell comment editor is destroyed? " + (callout.getContent()[0]!==undefined?callout.getContent()[0].getId():"true"));
                            if (callout.getContent().length>0) {
                                callout.setVisible(true);
                                return callout.openBy(cell);
                            }else{
                                return;
                            }
                        },100)
                    };
                })(overlay, this.sheetFilters/*.scenario*/, callOut, cell);
                cell.onmouseout = (function (callout) {
                    return function () {
                        var content=callout.getContent()[0];
                        if (content) {
                            content.fireEvent(content.getId()+"_CLOSING",{"view":content,"action":"CANCEL"});
                        }
                    };
                })(callOut);
            }
        }
    }

    var cells;
    for (i = 0; i < this.infoData.rowsLocked.length; i++) {
        cells = sas.stress.wkbench.widget.Slimsheet.findRow(control, this.infoData.rowsLocked[i].index);
        for (j = 0; j < cells.length; j++) {
            cell = cells[j];
            if (cell) {
                prevClass = cell.className;
                cell.className = prevClass ? prevClass + " SlimLocked" : "SlimLocked";
            }
        }
    }
    for (i = 0; i < this.infoData.rowsRoots.length; i++) {
        cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, this.infoData.rowsRoots[i].index, 1, true);
        if (cell) {
            prevClass = cell.className;
            cell.className = prevClass ? prevClass + " SlimHierarchyRoot" : "SlimHierarchyRoot";
        }
    }
};
/**
 *
 * @param cell
 * @param augmentingType    string: (SlimFormulaic), (SlimCommented), (SlimValidationError), (SlimDrilldownAble)
 * @private
 */
sas.stress.wkbench.widget.SlimsheetSource.prototype._dynamicallyAugmentCells=function(cell,augmentingType){
    // we need to create the inner DIV here.. see method (augmentCell) for comments
    var innerdiv = document.createElement("DIV");
    innerdiv.className = augmentingType;
    cell.appendChild(innerdiv);
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.augmentCell = function (control,dims,source,id,row,col,i,j,inRange,values,isScrolled,td) {
    if (isScrolled) {
        var currentProject = sas.stress.wkbench.model.Project.getCurrentProject();
        if (currentProject.isLocked()){ //if the project is locked.. we should not be able to edit
            td.className = td.className?td.className+ " SlimNotEditable":"SlimNotEditable";
        }else {
            var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
            if (Number(wkbenchView.iterationSelect.getSelectedKey()) < currentProject.getCurrentIterationNo()) {
                // if iteration does not match.. we are not editable
                td.className = "SlimNotEditable";
            } else {
                // use what was given
                if (this.getCellIsReadonlyOverrideValue(row, col)) {
                    td.className = "SlimNotEditable";
                }
            }
        }
        // do not augment the extra DIVs here... there is a feeling that the DOM is becoming too huge to render fast..
        //  therefore, rather the suggested idea we will do it whenever required
    }
    return td;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getFormatter = function (row) {
    var rowInfo = this.infoData.rows[row];
    return rowInfo && rowInfo.formatter;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.getFormatterRowCol = function (row, col) {
	//var DEFAULT_FORMATTER = new sas.stress.wkbench.model.Formatter("Numeric",2,1);
    var rowFormatter = this.getFormatter(row);
    
    var colInfo = this.infoData.columns[col];
    if (colInfo){
    	var colFormatter = colInfo.formatter;
    	if (rowFormatter.isDefaultFormat() && !colFormatter.isDefaultFormat()){
    		return colFormatter;
    	} else if (!rowFormatter.isDefaultFormat() && colFormatter.isDefaultFormat()){
    		return rowFormatter;
    	} else if (rowFormatter.isDefaultFormat() && colFormatter.isDefaultFormat()){
    		// both are defaults.. does not matter who wins
    		return rowFormatter;
    	} else {
    		// both are not defaults.. let column formating take precedence
    		return colFormatter;
    	}
    }
    
    return rowFormatter;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.getRealColumn = function (column) {
    return column;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getCalculatedColumn = function (col) {
    return col;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.setSelectedColumn = function (col) {
    this.selectedCol = col;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getSelectedColumn = function () {
    return this.selectedCol;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.setSelectedRow = function (row) {
    this.selectedRow = row;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getSelectedRow = function () {
    return this.selectedRow;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.setCellRange = function (cellRange) {
    this.cellRange = cellRange;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.getCellRange = function () {
    return this.cellRange;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.putToLocalStorage=function(){
    var that=this;
    var stored={fixed:{width:0, widths:[]}, scrolled:{width:0, widths:[]}};
    stored.fixed.width=this.info.fixed.width;
    stored.fixed.widths=this.info.fixed.widths;

    stored.scrolled.width=this.info.scrolled.width;
    stored.scrolled.widths=this.info.scrolled.widths;
    var project=this.sheetFilters.getSheetInfo().getSheet().getProject();
    var sheetDefn= _.find(project.getSheetDefinitions(),function(sheetDefinition){
        return sheetDefinition.sheetTemplateName.toLowerCase()===that.sheetName.toLowerCase();
    });
    if (sheetDefn) {
        sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.sheet.template." + sheetDefn.sheetTemplateKey + ".columns.widths", JSON.stringify(stored), project.getId());
    }
    //localStorage[this.sheetName]=JSON.stringify(stored);
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getFromLocalStorage=function(){
    var that=this;
    var project=this.sheetFilters.getSheetInfo().getSheet().getProject();
    var sheetDefn= _.find(project.getSheetDefinitions(),function(sheetDefinition){
        return sheetDefinition.sheetTemplateName.toLowerCase()===that.sheetName.toLowerCase();
    });
    if (sheetDefn) {
        var stored = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet.template." + sheetDefn.sheetTemplateKey + ".columns.widths",project.getId());
        if (stored) {
            return JSON.parse(stored);
        }
    }
    return undefined;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.clearInfo=function(){
    this.info={};
    this.info=undefined;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.longestString = function (a) {
    var c = 0, d = 0, l = 0, i = a.length;
    if (i) while (i--) {
        d = a[i].length;
        if (d > c) {
            l = i; c = d;
        }
    }
    return a[l];
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.getWidthOfText = function(txt){
    /*
    fixes for https://sirius.na.sas.com/Sirius/ShowTrack.aspx?trknum=7612716769
    http://sww.sas.com/cgi-bin/quick_browse?defectid=S1499279 
    The intention of the method is to dynamically figure out the maximum length the fixed column should be so
    that the name is visible to the user. For that we had hard coded the fontName and fontSize. there was a
    need to not use 'hard-coded' names and sizes and use the ones already defined. However, the problem
    with this is the customer could be on a OLDER OS and the concept may not work. 
    The window.getComputedStyle() is not always avaialable in all browsers.
    
    Fix is to catch the exception and go back to the old way of doing things.
    */
    try {
        var style=window.getComputedStyle(document.firstElementChild);

        // Create dummy span
        var dummySpan = document.createElement('span');
        // Set font-size
        if (style) {
            dummySpan.style.fontSize = parseInt(style.getPropertyValue("font-size"), 10) - 1 + "px";
            // Set font-face / font-family
            dummySpan.style.fontFamily = style.getPropertyValue("font-family");
        }
        // Set text
        dummySpan.innerHTML = txt;
        document.body.appendChild(dummySpan);
        // Get width NOW, since the dummy span is about to be removed from the document
        var w = dummySpan.offsetWidth;
        // Cleanup
        document.body.removeChild(dummySpan);
        // All right, we're done
        return w;
    } catch (e) {
        // Create dummy span
        this.e = document.createElement('span');
        // Set font-size
        this.e.style.fontSize = "13px";
        // Set font-face / font-family
        this.e.style.fontFamily = "'AvenirNext', Helvetica, arial,sans,sans-serif";
        // Set text
        this.e.innerHTML = txt;
        document.body.appendChild(this.e);
        // Get width NOW, since the dummy span is about to be removed from the document
        var w = this.e.offsetWidth;
        // Cleanup
        document.body.removeChild(this.e);
        // All right, we're done
        return w;
    }
};

sas.stress.wkbench.widget.SlimsheetSource.prototype._createEmptyNodesFromGivenNode = function(node){
    var emptyNode = new sas.stress.wkbench.model.TreeNode("", "", node.order, node.cellValueIndex,
        node.dataKey, node.subtotallikey, node.readOnly, false, node);  // does not have children

    // the formats
    emptyNode.setFormat(node.getFormat());
    emptyNode.setFormatPrecision(node.getFormatPrecision());
    emptyNode.setFormatScale(node.getFormatScale());
    emptyNode.setFormatType(node.getFormatType());
    emptyNode.setFormulaCode(node.getFormulaCode());
    emptyNode.setLocked(node.getLocked());
    if (node.getDataKey()) {
        emptyNode.likey = node.likey;
    }
    emptyNode.setSpan(1);
    return emptyNode;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype._setupPrevAndNextNodes = function(allnodes){
    var idx=0,prevIdx= 0;
    if (allnodes) {
        while (idx < allnodes.length) {
            // current
            var node = allnodes[idx];
            // the next
            node.setNext(allnodes[idx + 1]);
            if (prevIdx < idx) {
                // the previous
                node.setPrevious(allnodes[prevIdx]);
            }
            // increment
            prevIdx = idx++;
        }
    }
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.destroy=function(){
    this.widths=[];
    this.widths=null;
    this.infoData=undefined;
    this.sheetFilters=undefined;

    var keys=[],idx;
    if (this.info){
        keys= _.keys(this.info.fixed);
        for (idx=0;idx<keys.length;++idx){
            var data=this.info.fixed[keys[idx]];
            if (data&&data.nodes) {
                data.nodes.length=0;
                data.label.length=0;
                delete data.nodes;
                delete data.label;
            }
        }
        keys= _.keys(this.info.scrolled);
        for (idx=0;idx<keys.length;++idx){
            data = this.info.scrolled[keys[idx]];
            if (data&&data.nodes) {
                data.nodes.length=0;
                data.label.length=0;
                delete data.nodes;
                delete data.label;
            }
        }
    }
    if (this.callOuts) {
        for (idx=0;idx<this.callOuts.length;++idx){
            var callout = this.callOuts[idx];
            callout.destroy();
        }
        this.callOuts.length = 0;
    }
    delete this.callOuts;
    this.info={};
    this.info=undefined;
    delete this.info;
    this.infoData=undefined;
    delete this.infoData;
    this.sheetFilters=undefined;
    delete this.sheetFilters;
    this.data=undefined;
    delete this.data;
};